@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,28 +11,31 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-endpoint for heat AWS-compatible CloudWatch API
-"""
+"""Endpoint for heat AWS-compatible CloudWatch API."""
+
+from oslo_log import log as logging
+import oslo_messaging as messaging
+import six
+
 from heat.api.aws import exception
 from heat.api.aws import utils as api_utils
 from heat.common import exception as heat_exception
+from heat.common.i18n import _
+from heat.common.i18n import _LE
+from heat.common.i18n import _LW
 from heat.common import policy
 from heat.common import wsgi
-from heat.openstack.common.gettextutils import _
-from heat.openstack.common import log as logging
-from heat.openstack.common.rpc import common as rpc_common
-from heat.rpc import api as engine_api
+from heat.rpc import api as rpc_api
 from heat.rpc import client as rpc_client
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class WatchController(object):
 
-    """
-    WSGI controller for CloudWatch resource in heat API
-    Implements the API actions
+    """WSGI controller for CloudWatch resource in heat API.
+
+    Implements the API actions.
     """
 
     def __init__(self, options):
@@ -58,10 +60,10 @@ class WatchController(object):
 
     @staticmethod
     def _reformat_dimensions(dims):
-        '''
-        Reformat dimensions list into AWS API format
-        Parameter dims is a list of dicts
-        '''
+        """Reformat dimensions list into AWS API format.
+
+        :param dims: a list of dicts.
+        """
         newdims = []
         for count, d in enumerate(dims, 1):
             for key in d.keys():
@@ -69,63 +71,55 @@ class WatchController(object):
         return newdims
 
     def delete_alarms(self, req):
-        """
-        Implements DeleteAlarms API action
-        """
+        """Implements DeleteAlarms API action."""
         self._enforce(req, 'DeleteAlarms')
         return exception.HeatAPINotImplementedError()
 
     def describe_alarm_history(self, req):
-        """
-        Implements DescribeAlarmHistory API action
-        """
+        """Implements DescribeAlarmHistory API action."""
         self._enforce(req, 'DescribeAlarmHistory')
         return exception.HeatAPINotImplementedError()
 
     def describe_alarms(self, req):
-        """
-        Implements DescribeAlarms API action
-        """
+        """Implements DescribeAlarms API action."""
         self._enforce(req, 'DescribeAlarms')
 
         def format_metric_alarm(a):
-            """
-            Reformat engine output into the AWS "MetricAlarm" format
-            """
+            """Reformat engine output into the AWS "MetricAlarm" format."""
             keymap = {
-                engine_api.WATCH_ACTIONS_ENABLED: 'ActionsEnabled',
-                engine_api.WATCH_ALARM_ACTIONS: 'AlarmActions',
-                engine_api.WATCH_TOPIC: 'AlarmArn',
-                engine_api.WATCH_UPDATED_TIME:
+                rpc_api.WATCH_ACTIONS_ENABLED: 'ActionsEnabled',
+                rpc_api.WATCH_ALARM_ACTIONS: 'AlarmActions',
+                rpc_api.WATCH_TOPIC: 'AlarmArn',
+                rpc_api.WATCH_UPDATED_TIME:
                 'AlarmConfigurationUpdatedTimestamp',
-                engine_api.WATCH_DESCRIPTION: 'AlarmDescription',
-                engine_api.WATCH_NAME: 'AlarmName',
-                engine_api.WATCH_COMPARISON: 'ComparisonOperator',
-                engine_api.WATCH_DIMENSIONS: 'Dimensions',
-                engine_api.WATCH_PERIODS: 'EvaluationPeriods',
-                engine_api.WATCH_INSUFFICIENT_ACTIONS:
+                rpc_api.WATCH_DESCRIPTION: 'AlarmDescription',
+                rpc_api.WATCH_NAME: 'AlarmName',
+                rpc_api.WATCH_COMPARISON: 'ComparisonOperator',
+                rpc_api.WATCH_DIMENSIONS: 'Dimensions',
+                rpc_api.WATCH_PERIODS: 'EvaluationPeriods',
+                rpc_api.WATCH_INSUFFICIENT_ACTIONS:
                 'InsufficientDataActions',
-                engine_api.WATCH_METRIC_NAME: 'MetricName',
-                engine_api.WATCH_NAMESPACE: 'Namespace',
-                engine_api.WATCH_OK_ACTIONS: 'OKActions',
-                engine_api.WATCH_PERIOD: 'Period',
-                engine_api.WATCH_STATE_REASON: 'StateReason',
-                engine_api.WATCH_STATE_REASON_DATA: 'StateReasonData',
-                engine_api.WATCH_STATE_UPDATED_TIME: 'StateUpdatedTimestamp',
-                engine_api.WATCH_STATE_VALUE: 'StateValue',
-                engine_api.WATCH_STATISTIC: 'Statistic',
-                engine_api.WATCH_THRESHOLD: 'Threshold',
-                engine_api.WATCH_UNIT: 'Unit'}
+                rpc_api.WATCH_METRIC_NAME: 'MetricName',
+                rpc_api.WATCH_NAMESPACE: 'Namespace',
+                rpc_api.WATCH_OK_ACTIONS: 'OKActions',
+                rpc_api.WATCH_PERIOD: 'Period',
+                rpc_api.WATCH_STATE_REASON: 'StateReason',
+                rpc_api.WATCH_STATE_REASON_DATA: 'StateReasonData',
+                rpc_api.WATCH_STATE_UPDATED_TIME: 'StateUpdatedTimestamp',
+                rpc_api.WATCH_STATE_VALUE: 'StateValue',
+                rpc_api.WATCH_STATISTIC: 'Statistic',
+                rpc_api.WATCH_THRESHOLD: 'Threshold',
+                rpc_api.WATCH_UNIT: 'Unit',
+            }
 
             # AWS doesn't return StackId in the main MetricAlarm
             # structure, so we add StackId as a dimension to all responses
-            a[engine_api.WATCH_DIMENSIONS].append({'StackId':
-                                                  a[engine_api.WATCH_STACK_ID]
-                                                   })
+            a[rpc_api.WATCH_DIMENSIONS].append({'StackId':
+                                                a[rpc_api.WATCH_STACK_ID]})
 
             # Reformat dimensions list into AWS API format
-            a[engine_api.WATCH_DIMENSIONS] = self._reformat_dimensions(
-                a[engine_api.WATCH_DIMENSIONS])
+            a[rpc_api.WATCH_DIMENSIONS] = self._reformat_dimensions(
+                a[rpc_api.WATCH_DIMENSIONS])
 
             return api_utils.reformat_dict_keys(keymap, a)
 
@@ -138,7 +132,7 @@ class WatchController(object):
 
         try:
             watch_list = self.rpc_client.show_watch(con, watch_name=name)
-        except rpc_common.RemoteError as ex:
+        except messaging.RemoteError as ex:
             return exception.map_remote_error(ex)
 
         res = {'MetricAlarms': [format_metric_alarm(a)
@@ -148,61 +142,54 @@ class WatchController(object):
         return result
 
     def describe_alarms_for_metric(self, req):
-        """
-        Implements DescribeAlarmsForMetric API action
-        """
+        """Implements DescribeAlarmsForMetric API action."""
         self._enforce(req, 'DescribeAlarmsForMetric')
         return exception.HeatAPINotImplementedError()
 
     def disable_alarm_actions(self, req):
-        """
-        Implements DisableAlarmActions API action
-        """
+        """Implements DisableAlarmActions API action."""
         self._enforce(req, 'DisableAlarmActions')
         return exception.HeatAPINotImplementedError()
 
     def enable_alarm_actions(self, req):
-        """
-        Implements EnableAlarmActions API action
-        """
+        """Implements EnableAlarmActions API action."""
         self._enforce(req, 'EnableAlarmActions')
         return exception.HeatAPINotImplementedError()
 
     def get_metric_statistics(self, req):
-        """
-        Implements GetMetricStatistics API action
-        """
+        """Implements GetMetricStatistics API action."""
         self._enforce(req, 'GetMetricStatistics')
         return exception.HeatAPINotImplementedError()
 
     def list_metrics(self, req):
-        """
-        Implements ListMetrics API action
+        """Implements ListMetrics API action.
+
         Lists metric datapoints associated with a particular alarm,
-        or all alarms if none specified
+        or all alarms if none specified.
         """
         self._enforce(req, 'ListMetrics')
 
-        def format_metric_data(d, fil={}):
-            """
-            Reformat engine output into the AWS "Metric" format
+        def format_metric_data(d, fil=None):
+            """Reformat engine output into the AWS "Metric" format.
+
             Takes an optional filter dict, which is traversed
             so a metric dict is only returned if all keys match
-            the filter dict
+            the filter dict.
             """
+            fil = fil or {}
             dimensions = [
-                {'AlarmName': d[engine_api.WATCH_DATA_ALARM]},
-                {'Timestamp': d[engine_api.WATCH_DATA_TIME]}
+                {'AlarmName': d[rpc_api.WATCH_DATA_ALARM]},
+                {'Timestamp': d[rpc_api.WATCH_DATA_TIME]}
             ]
-            for key in d[engine_api.WATCH_DATA]:
-                dimensions.append({key: d[engine_api.WATCH_DATA][key]})
+            for key in d[rpc_api.WATCH_DATA]:
+                dimensions.append({key: d[rpc_api.WATCH_DATA][key]})
 
             newdims = self._reformat_dimensions(dimensions)
 
             result = {
-                'MetricName': d[engine_api.WATCH_DATA_METRIC],
+                'MetricName': d[rpc_api.WATCH_DATA_METRIC],
                 'Dimensions': newdims,
-                'Namespace': d[engine_api.WATCH_DATA_NAMESPACE],
+                'Namespace': d[rpc_api.WATCH_DATA_NAMESPACE],
             }
 
             for f in fil:
@@ -212,16 +199,16 @@ class WatchController(object):
                         # Filter criteria not met, return None
                         return
                 except KeyError:
-                    logger.warning(_("Invalid filter key %s, ignoring") % f)
+                    LOG.warn(_LW("Invalid filter key %s, ignoring"), f)
 
             return result
 
         con = req.context
         parms = dict(req.params)
         # FIXME : Don't yet handle filtering by Dimensions
-        filter_result = dict((k, v) for (k, v) in parms.iteritems() if k in
+        filter_result = dict((k, v) for (k, v) in six.iteritems(parms) if k in
                              ("MetricName", "Namespace"))
-        logger.debug(_("filter parameters : %s") % filter_result)
+        LOG.debug("filter parameters : %s" % filter_result)
 
         try:
             # Engine does not currently support query by namespace/metric
@@ -230,7 +217,7 @@ class WatchController(object):
                            'metric_name': None}
             watch_data = self.rpc_client.show_watch_metric(con,
                                                            **null_kwargs)
-        except rpc_common.RemoteError as ex:
+        except messaging.RemoteError as ex:
             return exception.map_remote_error(ex)
 
         res = {'Metrics': []}
@@ -243,16 +230,12 @@ class WatchController(object):
         return result
 
     def put_metric_alarm(self, req):
-        """
-        Implements PutMetricAlarm API action
-        """
+        """Implements PutMetricAlarm API action."""
         self._enforce(req, 'PutMetricAlarm')
         return exception.HeatAPINotImplementedError()
 
     def put_metric_data(self, req):
-        """
-        Implements PutMetricData API action
-        """
+        """Implements PutMetricData API action."""
         self._enforce(req, 'PutMetricData')
 
         con = req.context
@@ -267,7 +250,7 @@ class WatchController(object):
         # need to process (each dict) for dimensions
         metric_data = api_utils.extract_param_list(parms, prefix='MetricData')
         if not len(metric_data):
-            logger.error(_("Request does not contain required MetricData"))
+            LOG.error(_LE("Request does not contain required MetricData"))
             return exception.HeatMissingParameterError("MetricData list")
 
         watch_name = None
@@ -293,22 +276,20 @@ class WatchController(object):
 
         try:
             self.rpc_client.create_watch_data(con, watch_name, data)
-        except rpc_common.RemoteError as ex:
+        except messaging.RemoteError as ex:
             return exception.map_remote_error(ex)
 
         result = {'ResponseMetadata': None}
         return api_utils.format_response("PutMetricData", result)
 
     def set_alarm_state(self, req):
-        """
-        Implements SetAlarmState API action
-        """
+        """Implements SetAlarmState API action."""
         self._enforce(req, 'SetAlarmState')
 
         # Map from AWS state names to those used in the engine
-        state_map = {'OK': engine_api.WATCH_STATE_OK,
-                     'ALARM': engine_api.WATCH_STATE_ALARM,
-                     'INSUFFICIENT_DATA': engine_api.WATCH_STATE_NODATA}
+        state_map = {'OK': rpc_api.WATCH_STATE_OK,
+                     'ALARM': rpc_api.WATCH_STATE_ALARM,
+                     'INSUFFICIENT_DATA': rpc_api.WATCH_STATE_NODATA}
 
         con = req.context
         parms = dict(req.params)
@@ -322,23 +303,21 @@ class WatchController(object):
                     'expecting one of %(expect)s') % {
                         'state': state,
                         'expect': state_map.keys()}
-            logger.error(msg)
+            LOG.error(msg)
             return exception.HeatInvalidParameterValueError(msg)
 
-        logger.debug(_("setting %(name)s to %(state)s") % {
-                     'name': name, 'state': state_map[state]})
+        LOG.debug("setting %(name)s to %(state)s" % {
+                  'name': name, 'state': state_map[state]})
         try:
             self.rpc_client.set_watch_state(con, watch_name=name,
                                             state=state_map[state])
-        except rpc_common.RemoteError as ex:
+        except messaging.RemoteError as ex:
             return exception.map_remote_error(ex)
 
         return api_utils.format_response("SetAlarmState", "")
 
 
 def create_resource(options):
-    """
-    Watch resource factory method.
-    """
+    """Watch resource factory method."""
     deserializer = wsgi.JSONRequestDeserializer()
     return wsgi.Resource(WatchController(options), deserializer)
