@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,6 +14,7 @@
 from webob import exc
 
 from heat.api.openstack.v1 import util
+from heat.common import serializers
 from heat.common import wsgi
 from heat.rpc import client as rpc_client
 
@@ -78,6 +78,6 @@ def create_resource(options):
     Software configs resource factory method.
     """
     deserializer = wsgi.JSONRequestDeserializer()
-    serializer = wsgi.JSONResponseSerializer()
+    serializer = serializers.JSONResponseSerializer()
     return wsgi.Resource(
         SoftwareConfigController(options), deserializer, serializer)
