@@ -19,11 +19,15 @@
 
 import sys
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log
 
-from heat.db import api
+from heat.common import context
+from heat.common.i18n import _
+from heat.common import service_utils
+from heat.db import api as db_api
 from heat.db import utils
-from heat.openstack.common import log
+from heat.objects import service as service_objects
 from heat import version
 
 
@@ -32,7 +36,7 @@ CONF = cfg.CONF
 
 def do_db_version():
     """Print database's current migration level."""
-    print(api.db_version(api.get_engine()))
+    print(db_api.db_version(db_api.get_engine()))
 
 
 def do_db_sync():
@@ -40,7 +44,40 @@ def do_db_sync():
     Place a database under migration control and upgrade,
     creating first if necessary.
     """
-    api.db_sync(api.get_engine(), CONF.command.version)
+    db_api.db_sync(db_api.get_engine(), CONF.command.version)
+
+
+class ServiceManageCommand(object):
+    def service_list(self):
+        ctxt = context.get_admin_context()
+        services = [service_utils.format_service(service)
+                    for service in service_objects.Service.get_all(ctxt)]
+
+        print_format = "%-16s %-16s %-36s %-10s %-10s %-10s %-10s"
+        print(print_format % (_('Hostname'),
+                              _('Binary'),
+                              _('Engine_Id'),
+                              _('Host'),
+                              _('Topic'),
+                              _('Status'),
+                              _('Updated At')))
+
+        for svc in services:
+            print(print_format % (svc['hostname'],
+                                  svc['binary'],
+                                  svc['engine_id'],
+                                  svc['host'],
+                                  svc['topic'],
+                                  svc['status'],
+                                  svc['updated_at']))
+
+    @staticmethod
+    def add_service_parsers(subparsers):
+        service_parser = subparsers.add_parser('service')
+        service_parser.set_defaults(command_object=ServiceManageCommand)
+        service_subparsers = service_parser.add_subparsers(dest='action')
+        list_parser = service_subparsers.add_parser('list')
+        list_parser.set_defaults(func=ServiceManageCommand().service_list)
 
 
 def purge_deleted():
@@ -68,6 +105,8 @@ def add_command_parsers(subparsers):
         choices=['days', 'hours', 'minutes', 'seconds'],
         help=_('Granularity to use for age argument, defaults to days.'))
 
+    ServiceManageCommand.add_service_parsers(subparsers)
+
 command_opt = cfg.SubCommandOpt('command',
                                 title='Commands',
                                 help='Show available commands.',
@@ -75,13 +114,14 @@ command_opt = cfg.SubCommandOpt('command',
 
 
 def main():
+    log.register_options(CONF)
+    log.setup(CONF, "heat-manage")
     CONF.register_cli_opt(command_opt)
     try:
         default_config_files = cfg.find_config_files('heat', 'heat-engine')
         CONF(sys.argv[1:], project='heat', prog='heat-manage',
              version=version.version_info.version_string(),
              default_config_files=default_config_files)
-        log.setup("heat")
     except RuntimeError as e:
         sys.exit("ERROR: %s" % e)
 
