@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -13,10 +13,10 @@
 
 import uuid
 
+from migrate.versioning import util as migrate_util
 import sqlalchemy
 
-from migrate.versioning import util as migrate_util
-from heat.openstack.common.gettextutils import _
+from heat.common.i18n import _
 
 
 def upgrade(migrate_engine):
@@ -34,7 +34,7 @@ def downgrade(migrate_engine):
 
     try:
         event.c.id.alter(type=sqlalchemy.Integer, primary_key=True)
-    except:
+    except Exception:
         # NOTE(pafuent): since there is no way to downgrade just passing
         # The same is did in 018_resource_id_uuid.py
         migrate_util.log.warning(_('If you really want to downgrade to this '
