@@ -10,21 +10,23 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import copy
-
 from migrate.versioning import util as migrate_util
+from oslo_serialization import jsonutils
 import six
+import sqlalchemy
 from sqlalchemy.orm import sessionmaker
 
 from heat.common.i18n import _
-from heat.db.sqlalchemy import models
 
 
 def upgrade(migrate_engine):
     Session = sessionmaker(bind=migrate_engine)
     session = Session()
 
-    raw_templates = session.query(models.RawTemplate).all()
+    meta = sqlalchemy.MetaData(bind=migrate_engine)
+
+    templ_table = sqlalchemy.Table('raw_template', meta, autoload=True)
+    raw_templates = templ_table.select().execute()
 
     CFN_TO_HOT_RESOURCE_ATTRS = {'Type': 'type',
                                  'Properties': 'properties',
@@ -49,10 +51,10 @@ def upgrade(migrate_engine):
         return changed
 
     for raw_template in raw_templates:
-        if 'heat_template_version' in raw_template.template:
+        template = jsonutils.loads(raw_template.template)
+        if 'heat_template_version' in template:
 
             changed = False
-            template = copy.deepcopy(raw_template.template)
 
             resources = template.get('resources', {})
             if _translate(resources, CFN_TO_HOT_RESOURCE_ATTRS):
@@ -63,7 +65,10 @@ def upgrade(migrate_engine):
                 changed = True
 
             if changed:
-                raw_template.template = template
+                (templ_table.update().
+                    where(templ_table.c.id == raw_template.id).
+                    values(template=jsonutils.dumps(template)).
+                    execute())
                 session.commit()
     session.close()
 
