@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,8 +12,11 @@
 #    under the License.
 
 import collections
+
 import six
 
+from heat.common import exception
+from heat.common.i18n import _
 from heat.engine.cfn import functions as cfn_funcs
 from heat.engine import function
 from heat.engine import parameters
@@ -34,11 +36,13 @@ _RESOURCE_KEYS = (
 class CfnTemplate(template.Template):
     '''A stack template.'''
 
-    SECTIONS = (VERSION, ALTERNATE_VERSION, DESCRIPTION, MAPPINGS,
-                PARAMETERS, RESOURCES, OUTPUTS) = \
-               ('AWSTemplateFormatVersion', 'HeatTemplateFormatVersion',
-                'Description', 'Mappings', 'Parameters', 'Resources', 'Outputs'
-                )
+    SECTIONS = (
+        VERSION, ALTERNATE_VERSION,
+        DESCRIPTION, MAPPINGS, PARAMETERS, RESOURCES, OUTPUTS
+    ) = (
+        'AWSTemplateFormatVersion', 'HeatTemplateFormatVersion',
+        'Description', 'Mappings', 'Parameters', 'Resources', 'Outputs'
+    )
 
     SECTIONS_NO_DIRECT_ACCESS = set([PARAMETERS, VERSION, ALTERNATE_VERSION])
 
@@ -69,77 +73,93 @@ class CfnTemplate(template.Template):
         # to be consistent with an empty json section.
         return self.t.get(section) or default
 
-    def param_schemata(self):
+    def param_schemata(self, param_defaults=None):
         params = self.t.get(self.PARAMETERS) or {}
+        pdefaults = param_defaults or {}
+        for name, schema in six.iteritems(params):
+            if name in pdefaults:
+                params[name][parameters.DEFAULT] = pdefaults[name]
+
         return dict((name, parameters.Schema.from_dict(name, schema))
                     for name, schema in six.iteritems(params))
 
-    def parameters(self, stack_identifier, user_params):
+    def get_section_name(self, section):
+        return section
+
+    def parameters(self, stack_identifier, user_params, param_defaults=None):
         return parameters.Parameters(stack_identifier, self,
-                                     user_params=user_params)
+                                     user_params=user_params,
+                                     param_defaults=param_defaults)
+
+    def validate_resource_definitions(self, stack):
+        resources = self.t.get(self.RESOURCES) or {}
+        allowed_keys = set(_RESOURCE_KEYS)
+
+        try:
+
+            for name, snippet in resources.items():
+                data = self.parse(stack, snippet)
+
+                if not self.validate_resource_key_type(RES_TYPE,
+                                                       six.string_types,
+                                                       'string',
+                                                       allowed_keys,
+                                                       name, data):
+                    args = {'name': name, 'type_key': RES_TYPE}
+                    msg = _('Resource %(name)s is missing '
+                            '"%(type_key)s"') % args
+                    raise KeyError(msg)
+
+                self.validate_resource_key_type(
+                    RES_PROPERTIES,
+                    (collections.Mapping, function.Function),
+                    'object', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_METADATA,
+                    (collections.Mapping, function.Function),
+                    'object', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_DEPENDS_ON,
+                    collections.Sequence,
+                    'list or string', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_DELETION_POLICY,
+                    six.string_types,
+                    'string', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_UPDATE_POLICY,
+                    (collections.Mapping, function.Function),
+                    'object', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_DESCRIPTION,
+                    six.string_types,
+                    'string', allowed_keys, name, data)
+        except TypeError as ex:
+            raise exception.StackValidationFailed(message=six.text_type(ex))
 
     def resource_definitions(self, stack):
+        resources = self.t.get(self.RESOURCES) or {}
+
         def rsrc_defn_item(name, snippet):
             data = self.parse(stack, snippet)
 
-            def get_check_type(key, valid_types, typename, default=None):
-                if key in data:
-                    field = data[key]
-                    if not isinstance(field, valid_types):
-                        args = {'name': name, 'key': key, 'typename': typename}
-                        msg = _('Resource %(name)s %(key)s type'
-                                'must be %(typename)s') % args
-                        raise TypeError(msg)
-                    return field
-                else:
-                    return default
-
-            resource_type = get_check_type(RES_TYPE, basestring, 'string')
-            if resource_type is None:
-                args = {'name': name, 'type_key': RES_TYPE}
-                msg = _('Resource %(name)s is missing "%(type_key)s"') % args
-                raise KeyError(msg)
-
-            properties = get_check_type(RES_PROPERTIES,
-                                        (collections.Mapping,
-                                         function.Function),
-                                        'object')
-
-            metadata = get_check_type(RES_METADATA,
-                                      (collections.Mapping,
-                                       function.Function),
-                                      'object')
-
-            depends = get_check_type(RES_DEPENDS_ON,
-                                     collections.Sequence,
-                                     'list or string',
-                                     default=[])
-            if isinstance(depends, basestring):
+            depends = data.get(RES_DEPENDS_ON)
+            if isinstance(depends, six.string_types):
                 depends = [depends]
 
-            deletion_policy = get_check_type(RES_DELETION_POLICY,
-                                             basestring,
-                                             'string')
+            kwargs = {
+                'resource_type': data.get(RES_TYPE),
+                'properties': data.get(RES_PROPERTIES),
+                'metadata': data.get(RES_METADATA),
+                'depends': depends,
+                'deletion_policy': data.get(RES_DELETION_POLICY),
+                'update_policy': data.get(RES_UPDATE_POLICY),
+                'description': data.get(RES_DESCRIPTION) or ''
+            }
 
-            update_policy = get_check_type(RES_UPDATE_POLICY,
-                                           (collections.Mapping,
-                                            function.Function),
-                                           'object')
-
-            description = get_check_type(RES_DESCRIPTION,
-                                         basestring,
-                                         'string',
-                                         default='')
-
-            defn = rsrc_defn.ResourceDefinition(name, resource_type,
-                                                properties, metadata,
-                                                depends,
-                                                deletion_policy,
-                                                update_policy,
-                                                description=description)
+            defn = rsrc_defn.ResourceDefinition(name, **kwargs)
             return name, defn
 
-        resources = self.t.get(self.RESOURCES) or {}
         return dict(rsrc_defn_item(name, data)
                     for name, data in resources.items())
 
