@@ -13,13 +13,14 @@
 
 from glanceclient import client as gc
 from glanceclient import exc
+from oslo_log import log as logging
+from oslo_utils import uuidutils
 
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common.i18n import _LI
 from heat.engine.clients import client_plugin
 from heat.engine import constraints
-from heat.openstack.common import log as logging
-from heat.openstack.common import uuidutils
 
 LOG = logging.getLogger(__name__)
 
@@ -37,7 +38,7 @@ class GlanceClientPlugin(client_plugin.ClientPlugin):
         args = {
             'auth_url': con.auth_url,
             'service_type': 'image',
-            'project_id': con.tenant,
+            'project_id': con.tenant_id,
             'token': self.auth_token,
             'endpoint_type': endpoint_type,
             'cacert': self._get_client_option('glance', 'ca_file'),
@@ -54,6 +55,9 @@ class GlanceClientPlugin(client_plugin.ClientPlugin):
     def is_over_limit(self, ex):
         return isinstance(ex, exc.HTTPOverLimit)
 
+    def is_conflict(self, ex):
+        return isinstance(ex, exc.HTTPConflict)
+
     def get_image_id(self, image_identifier):
         '''
         Return an id for the specified image name or identifier.
@@ -89,11 +93,11 @@ class GlanceClientPlugin(client_plugin.ClientPlugin):
                 _("Error retrieving image list from glance: %s") % ex)
         num_matches = len(image_list)
         if num_matches == 0:
-            LOG.info(_("Image %s was not found in glance") %
+            LOG.info(_LI("Image %s was not found in glance"),
                      image_identifier)
             raise exception.ImageNotFound(image_name=image_identifier)
         elif num_matches > 1:
-            LOG.info(_("Multiple images %s were found in glance with name") %
+            LOG.info(_LI("Multiple images %s were found in glance with name"),
                      image_identifier)
             raise exception.PhysicalResourceNameAmbiguity(
                 name=image_identifier)
