@@ -14,11 +14,11 @@
 from neutronclient.common import exceptions
 from neutronclient.neutron import v2_0 as neutronV20
 from neutronclient.v2_0 import client as nc
+from oslo_utils import uuidutils
 
 from heat.common import exception
 from heat.engine.clients import client_plugin
 from heat.engine import constraints
-from heat.openstack.common import uuidutils
 
 
 class NeutronClientPlugin(client_plugin.ClientPlugin):
@@ -54,32 +54,40 @@ class NeutronClientPlugin(client_plugin.ClientPlugin):
                 ex.status_code == 404)
 
     def is_conflict(self, ex):
-        if not isinstance(ex, exceptions.NeutronClientException):
-            return False
-        return ex.status_code == 409
+        bad_conflicts = (exceptions.OverQuotaClient,)
+        return (isinstance(ex, exceptions.Conflict) and
+                not isinstance(ex, bad_conflicts))
 
     def is_over_limit(self, ex):
         if not isinstance(ex, exceptions.NeutronClientException):
             return False
         return ex.status_code == 413
 
+    def is_no_unique(self, ex):
+        return isinstance(ex, exceptions.NeutronClientNoUniqueMatch)
+
     def find_neutron_resource(self, props, key, key_type):
         return neutronV20.find_resourceid_by_name_or_id(
             self.client(), key_type, props.get(key))
 
+    def _resolve(self, props, key, id_key, key_type):
+        if props.get(key):
+            props[id_key] = self.find_neutron_resource(
+                props, key, key_type)
+            props.pop(key)
+        return props[id_key]
+
     def resolve_network(self, props, net_key, net_id_key):
-        if props.get(net_key):
-            props[net_id_key] = self.find_neutron_resource(
-                props, net_key, 'network')
-            props.pop(net_key)
-        return props[net_id_key]
+        return self._resolve(props, net_key, net_id_key, 'network')
 
     def resolve_subnet(self, props, subnet_key, subnet_id_key):
-        if props.get(subnet_key):
-            props[subnet_id_key] = self.find_neutron_resource(
-                props, subnet_key, 'subnet')
-            props.pop(subnet_key)
-        return props[subnet_id_key]
+        return self._resolve(props, subnet_key, subnet_id_key, 'subnet')
+
+    def resolve_router(self, props, router_key, router_id_key):
+        return self._resolve(props, router_key, router_id_key, 'router')
+
+    def resolve_port(self, props, port_key, port_id_key):
+        return self._resolve(props, port_key, port_id_key, 'port')
 
     def network_id_from_subnet_id(self, subnet_id):
         subnet_info = self.client().show_subnet(subnet_id)
@@ -121,12 +129,19 @@ class NeutronClientPlugin(client_plugin.ClientPlugin):
 
 class NetworkConstraint(constraints.BaseCustomConstraint):
 
-    expected_exceptions = (exceptions.NeutronClientException,)
+    expected_exceptions = (exceptions.NeutronClientException,
+                           exception.NovaNetworkNotFound,
+                           exception.PhysicalResourceNameAmbiguity)
 
     def validate_with_client(self, client, value):
-        neutron_client = client.client('neutron')
-        neutronV20.find_resourceid_by_name_or_id(
-            neutron_client, 'network', value)
+        try:
+            neutron_client = client.client('neutron')
+        except Exception:
+            # is not using neutron
+            client.client_plugin('nova').get_nova_network_id(value)
+        else:
+            neutronV20.find_resourceid_by_name_or_id(
+                neutron_client, 'network', value)
 
 
 class PortConstraint(constraints.BaseCustomConstraint):
