@@ -66,7 +66,7 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
         extensions = computeshell._discover_extensions(NOVACLIENT_VERSION)
 
         args = {
-            'project_id': self.context.tenant,
+            'project_id': self.context.tenant_id,
             'auth_url': self.context.auth_url,
             'service_type': 'compute',
             'username': None,
@@ -353,7 +353,10 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         If that's the case, confirm the resize, if not raise an error.
         """
         self.refresh_server(server)
-        while server.status == 'RESIZE':
+        # resize operation is asynchronous so the server resize may not start
+        # when checking server status (the server may stay ACTIVE instead
+        # of RESIZE).
+        while server.status in ('RESIZE', 'ACTIVE'):
             yield
             self.refresh_server(server)
         if server.status == 'VERIFY_RESIZE':
