@@ -13,13 +13,12 @@
 
 import six
 
+import oslo_db.exception
+
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common import identifier
-from heat.db import api as db_api
-from heat.openstack.common import log as logging
-
-LOG = logging.getLogger(__name__)
+from heat.objects import event as event_object
 
 
 class Event(object):
@@ -52,16 +51,16 @@ class Event(object):
     @classmethod
     def load(cls, context, event_id, event=None, stack=None):
         '''Retrieve an Event from the database.'''
-        from heat.engine import parser
+        from heat.engine import stack as parser
 
-        ev = event if event is not None else\
-            db_api.event_get(context, event_id)
+        ev = (event if event is not None else
+              event_object.Event.get_by_id(context, event_id))
         if ev is None:
             message = _('No event exists with id "%s"') % str(event_id)
             raise exception.NotFound(message)
 
-        st = stack if stack is not None else\
-            parser.Stack.load(context, ev.stack_id)
+        st = (stack if stack is not None else
+              parser.Stack.load(context, ev.stack_id))
 
         return cls(context, st, ev.resource_action, ev.resource_status,
                    ev.resource_status_reason, ev.physical_resource_id,
@@ -87,7 +86,22 @@ class Event(object):
         if self.timestamp is not None:
             ev['created_at'] = self.timestamp
 
-        new_ev = db_api.event_create(self.context, ev)
+        try:
+            new_ev = event_object.Event.create(self.context, ev)
+        except oslo_db.exception.DBError:
+            # Attempt do drop the largest key and re-store as we expect
+            # This to mostly happen with one large config blob property
+            max_key, max_val = max(ev['resource_properties'].items(),
+                                   key=lambda i: len(repr(i[1])))
+            err = 'Resource properties are too large to store'
+            ev['resource_properties'].update({'Error': err})
+            ev['resource_properties'][max_key] = '<Deleted, too large>'
+            try:
+                new_ev = event_object.Event.create(self.context, ev)
+            except oslo_db.exception.DBError:
+                # Give up and drop all properties..
+                ev['resource_properties'] = {'Error': err}
+                new_ev = event_object.Event.create(self.context, ev)
         self.id = new_ev.id
         return self.id
 
