@@ -13,13 +13,14 @@
 
 import collections
 import itertools
-import json
 
-from oslo.utils import encodeutils
-from oslo.utils import strutils
+from oslo_serialization import jsonutils
+from oslo_utils import encodeutils
+from oslo_utils import strutils
 import six
 
 from heat.common import exception
+from heat.common.i18n import _
 from heat.engine import constraints as constr
 
 
@@ -80,7 +81,8 @@ class Schema(constr.Schema):
                         message=_('Default must be a comma-delimited list '
                                   'string: %s') % err)
             try:
-                self.validate_constraints(default_value, context)
+                self.validate_constraints(default_value, context,
+                                          [constr.CustomConstraint])
             except (ValueError, TypeError,
                     exception.StackValidationFailed) as exc:
                 raise exception.InvalidSchemaError(
@@ -206,31 +208,35 @@ class Parameter(object):
         self.name = name
         self.schema = schema
         self.user_value = value
+        self.user_default = None
 
     def validate(self, validate_value=True, context=None):
-        '''
-        Validates the parameter.
+        """Validates the parameter.
 
         This method validates if the parameter's schema is valid,
         and if the default value - if present - or the user-provided
         value for the parameter comply with the schema.
-        '''
-        self.schema.validate(context)
-
-        if not validate_value:
-            return
+        """
+        err_msg = _("Parameter '%(name)s' is invalid: %(exp)s")
 
         try:
-            if self.has_default():
-                self._validate(self.default(), context)
+            self.schema.validate(context)
+
+            if not validate_value:
+                return
+
             if self.user_value is not None:
                 self._validate(self.user_value, context)
-            elif not self.has_default():
+            elif self.has_default():
+                self._validate(self.default(), context)
+            else:
                 raise exception.UserParameterMissing(key=self.name)
         except exception.StackValidationFailed as ex:
-            msg = _("Parameter '%(name)s' is invalid: %(exp)s") % dict(
-                name=self.name, exp=six.text_type(ex))
+            msg = err_msg % dict(name=self.name, exp=six.text_type(ex))
             raise exception.StackValidationFailed(message=msg)
+        except exception.InvalidSchemaError as ex:
+            msg = err_msg % dict(name=self.name, exp=six.text_type(ex))
+            raise exception.InvalidSchemaError(message=msg)
 
     def value(self):
         '''Get the parameter value, optionally sanitising it for output.'''
@@ -244,7 +250,7 @@ class Parameter(object):
 
     def has_value(self):
         '''Parameter has a user or default value.'''
-        return self.user_value or self.has_default()
+        return self.user_value is not None or self.has_default()
 
     def hidden(self):
         '''
@@ -263,14 +269,20 @@ class Parameter(object):
 
     def has_default(self):
         '''Return whether the parameter has a default value.'''
-        return self.schema.default is not None
+        return (self.schema.default is not None or
+                self.user_default is not None)
 
     def default(self):
         '''Return the default value of the parameter.'''
+        if self.user_default is not None:
+            return self.user_default
         return self.schema.default
 
+    def set_default(self, value):
+        self.user_default = value
+
     def __str__(self):
-        '''Return a string representation of the parameter'''
+        '''Return a string representation of the parameter.'''
         value = self.value()
         if self.hidden():
             return '******'
@@ -282,11 +294,11 @@ class NumberParam(Parameter):
     '''A template parameter of type "Number".'''
 
     def __int__(self):
-        '''Return an integer representation of the parameter'''
+        '''Return an integer representation of the parameter.'''
         return int(super(NumberParam, self).value())
 
     def __float__(self):
-        '''Return a float representation of the parameter'''
+        '''Return a float representation of the parameter.'''
         return float(super(NumberParam, self).value())
 
     def _validate(self, val, context):
@@ -330,7 +342,13 @@ class CommaDelimitedListParam(Parameter, collections.Sequence):
 
     def __init__(self, name, schema, value=None):
         super(CommaDelimitedListParam, self).__init__(name, schema, value)
-        self.parsed = self.parse(self.user_value or self.default())
+        if self.has_value():
+            if self.user_value is not None:
+                self.parsed = self.parse(self.user_value)
+            else:
+                self.parsed = self.parse(self.default())
+        else:
+            self.parsed = []
 
     def parse(self, value):
         # only parse when value is not already a list
@@ -347,7 +365,10 @@ class CommaDelimitedListParam(Parameter, collections.Sequence):
         return value
 
     def value(self):
-        return self.parsed
+        if self.has_value():
+            return self.parsed
+
+        raise exception.UserParameterMissing(key=self.name)
 
     def __len__(self):
         '''Return the length of the list.'''
@@ -372,22 +393,32 @@ class JsonParam(Parameter):
 
     def __init__(self, name, schema, value=None):
         super(JsonParam, self).__init__(name, schema, value)
-        self.parsed = self.parse(self.user_value or self.default())
+        if self.has_value():
+            if self.user_value is not None:
+                self.parsed = self.parse(self.user_value)
+            else:
+                self.parsed = self.parse(self.default())
+        else:
+            self.parsed = {}
 
     def parse(self, value):
         try:
             val = value
-            if not isinstance(val, basestring):
-                val = json.dumps(val)
+            if not isinstance(val, six.string_types):
+                # turn off oslo_serialization's clever to_primitive()
+                val = jsonutils.dumps(val, default=None)
             if val:
-                return json.loads(val)
+                return jsonutils.loads(val)
         except (ValueError, TypeError) as err:
             message = _('Value must be valid JSON: %s') % err
             raise ValueError(message)
         return value
 
     def value(self):
-        return self.parsed
+        if self.has_value():
+            return self.parsed
+
+        raise exception.UserParameterMissing(key=self.name)
 
     def __getitem__(self, key):
         return self.parsed[key]
@@ -401,7 +432,7 @@ class JsonParam(Parameter):
     def __str__(self):
         if self.hidden():
             return super(JsonParam, self).__str__()
-        return encodeutils.safe_decode(json.dumps(self.value()))
+        return encodeutils.safe_decode(jsonutils.dumps(self.value()))
 
     def _validate(self, val, context):
         val = self.parse(val)
@@ -420,12 +451,14 @@ class Parameters(collections.Mapping):
         'AWS::StackId', 'AWS::StackName', 'AWS::Region'
     )
 
-    def __init__(self, stack_identifier, tmpl, user_params=None):
+    def __init__(self, stack_identifier, tmpl, user_params=None,
+                 param_defaults=None):
         '''
         Create the parameter container for a stack from the stack name and
         template, optionally setting the user-supplied parameter values.
         '''
         user_params = user_params or {}
+        param_defaults = param_defaults or {}
 
         def user_parameter(schema_item):
             name, schema = schema_item
@@ -444,6 +477,10 @@ class Parameters(collections.Mapping):
                             p) for p in itertools.chain(pseudo_parameters,
                                                         user_parameters))
 
+        for pd in six.iterkeys(param_defaults):
+            if pd in self.params:
+                self.params[pd].set_default(param_defaults[pd])
+
     def validate(self, validate_value=True, context=None):
         '''
         Validates all parameters.
@@ -510,8 +547,8 @@ class Parameters(collections.Mapping):
                     raise exception.InvalidTemplateParameter(key=name)
 
     def _pseudo_parameters(self, stack_identifier):
-        stack_id = stack_identifier.arn() \
-            if stack_identifier is not None else 'None'
+        stack_id = (stack_identifier.arn()
+                    if stack_identifier is not None else 'None')
         stack_name = stack_identifier and stack_identifier.stack_name
 
         yield Parameter(self.PARAM_STACK_ID,
@@ -524,12 +561,12 @@ class Parameters(collections.Mapping):
             yield Parameter(self.PARAM_REGION,
                             Schema(Schema.STRING,
                                    default='ap-southeast-1',
-                                   constraints=
-                                   [constr.AllowedValues(['us-east-1',
-                                                          'us-west-1',
-                                                          'us-west-2',
-                                                          'sa-east-1',
-                                                          'eu-west-1',
-                                                          'ap-southeast-1',
-                                                          'ap-northeast-1']
-                                                         )]))
+                                   constraints=[
+                                       constr.AllowedValues(['us-east-1',
+                                                             'us-west-1',
+                                                             'us-west-2',
+                                                             'sa-east-1',
+                                                             'eu-west-1',
+                                                             'ap-southeast-1',
+                                                             'ap-northeast-1']
+                                                            )]))
