@@ -62,6 +62,10 @@ class UpdateReplace(Exception):
         super(Exception, self).__init__(six.text_type(msg))
 
 
+class NoActionRequired(Exception):
+    pass
+
+
 class ResourceInError(exception.HeatException):
     msg_fmt = _('Went to status %(resource_status)s '
                 'due to "%(status_reason)s"')
@@ -117,6 +121,13 @@ class Resource(object):
     # Default name to use for calls to self.client()
     default_client_name = None
 
+    # no signal actions
+    no_signal_actions = (SUSPEND, DELETE)
+
+    # Whether all other resources need a metadata_update() after
+    # a signal to this resource
+    signal_needs_metadata_updates = True
+
     def __new__(cls, name, definition, stack):
         '''Create a new Resource of the appropriate class for its type.'''
 
@@ -294,6 +305,10 @@ class Resource(object):
                     self.action, self.status,
                     "Failure occured while waiting.")
 
+    def has_nested(self):
+        # common resources have not nested, StackResource overrides it
+        return False
+
     def has_hook(self, hook):
         # Clear the cache to make sure the data is up to date:
         self._data = None
@@ -701,7 +716,8 @@ class Resource(object):
 
     def _needs_update(self, after, before, after_props, before_props,
                       prev_resource):
-        if self.status == self.FAILED:
+        if self.status == self.FAILED or \
+                (self.action == self.INIT and self.status == self.COMPLETE):
             raise UpdateReplace(self)
 
         if prev_resource is not None:
@@ -1141,7 +1157,7 @@ class Resource(object):
         to implement the signal, the base-class raise an exception if no
         handler is implemented.
         '''
-        if self.action in (self.SUSPEND, self.DELETE):
+        if self.action in self.no_signal_actions:
             self._add_event(self.action, self.status,
                             'Cannot signal resource during %s' % self.action)
             ex = Exception(_('Cannot signal resource during %s') % self.action)
@@ -1187,6 +1203,9 @@ class Resource(object):
             else:
                 reason_string = get_string_details()
             self._add_event('SIGNAL', self.status, reason_string)
+        except NoActionRequired:
+            # Don't log an event as it just spams the user.
+            pass
         except Exception as ex:
             LOG.exception(_LE('signal %(name)s : %(msg)s')
                           % {'name': six.text_type(self), 'msg': ex})
