@@ -24,7 +24,9 @@ from heat.engine.resources import stack_resource
 
 
 class NestedStack(stack_resource.StackResource):
-    """Represents a child stack to allow composition of templates."""
+    '''
+    A Resource representing a child stack to allow composition of templates.
+    '''
 
     PROPERTIES = (
         TEMPLATE_URL, TIMEOUT_IN_MINS, PARAMETERS,
@@ -41,7 +43,7 @@ class NestedStack(stack_resource.StackResource):
             update_allowed=True
         ),
         TIMEOUT_IN_MINS: properties.Schema(
-            properties.Schema.INTEGER,
+            properties.Schema.NUMBER,
             _('The length of time, in minutes, to wait for the nested stack '
               'creation.'),
             update_allowed=True
@@ -54,16 +56,15 @@ class NestedStack(stack_resource.StackResource):
     }
 
     def child_template(self):
-        template_url = self.properties[self.TEMPLATE_URL]
         try:
-            template_data = urlfetch.get(template_url)
+            template_data = urlfetch.get(self.properties[self.TEMPLATE_URL])
         except (exceptions.RequestException, IOError) as r_exc:
             raise ValueError(_("Could not fetch remote template '%(url)s': "
                              "%(exc)s") %
-                             {'url': template_url,
+                             {'url': self.properties[self.TEMPLATE_URL],
                               'exc': r_exc})
 
-        return template_format.parse(template_data, template_url)
+        return template_format.parse(template_data)
 
     def child_params(self):
         return self.properties[self.PARAMETERS]
@@ -81,18 +82,17 @@ class NestedStack(stack_resource.StackResource):
                                          self.properties[self.TIMEOUT_IN_MINS],
                                          adopt_data=resource_adopt_data)
 
-    def get_attribute(self, key, *path):
+    def handle_delete(self):
+        return self.delete_nested()
+
+    def FnGetAtt(self, key, *path):
         if key and not key.startswith('Outputs.'):
             raise exception.InvalidTemplateAttribute(resource=self.name,
                                                      key=key)
-        try:
-            attribute = self.get_output(key.partition('.')[-1])
-        except exception.NotFound:
-            raise exception.InvalidTemplateAttribute(resource=self.name,
-                                                     key=key)
+        attribute = self.get_output(key.partition('.')[-1])
         return attributes.select_from_attribute(attribute, path)
 
-    def get_reference_id(self):
+    def FnGetRefId(self):
         if self.nested() is None:
             return six.text_type(self.name)
 
@@ -102,7 +102,18 @@ class NestedStack(stack_resource.StackResource):
         # Nested stack template may be changed even if the prop_diff is empty.
         self.properties = json_snippet.properties(self.properties_schema,
                                                   self.context)
-        return self.update_with_template(self.child_template(),
+
+        try:
+            template_data = urlfetch.get(self.properties[self.TEMPLATE_URL])
+        except (exceptions.RequestException, IOError) as r_exc:
+            raise ValueError(_("Could not fetch remote template '%(url)s': "
+                             "%(exc)s") %
+                             {'url': self.properties[self.TEMPLATE_URL],
+                              'exc': r_exc})
+
+        template = template_format.parse(template_data)
+
+        return self.update_with_template(template,
                                          self.properties[self.PARAMETERS],
                                          self.properties[self.TIMEOUT_IN_MINS])
 
