@@ -66,13 +66,10 @@ class ElasticIp(resource.Resource):
         ALLOCATION_ID: attributes.Schema(
             _('ID that AWS assigns to represent the allocation of the address '
               'for use with Amazon VPC. Returned only for VPC elastic IP '
-              'addresses.'),
-            type=attributes.Schema.STRING
+              'addresses.')
         ),
     }
 
-    default_client_name = 'nova'
-
     def __init__(self, name, json_snippet, stack):
         super(ElasticIp, self).__init__(name, json_snippet, stack)
         self.ipaddress = None
@@ -88,7 +85,7 @@ class ElasticIp(resource.Resource):
                     self.ipaddress = ips['floatingip']['floating_ip_address']
             else:
                 try:
-                    ips = self.client().floating_ips.get(self.resource_id)
+                    ips = self.nova().floating_ips.get(self.resource_id)
                 except Exception as e:
                     self.client_plugin('nova').ignore_not_found(e)
                 else:
@@ -109,7 +106,7 @@ class ElasticIp(resource.Resource):
             LOG.info(_LI('ElasticIp create %s'), str(ips))
         else:
             try:
-                ips = self.client().floating_ips.create()
+                ips = self.nova().floating_ips.create()
             except Exception as e:
                 with excutils.save_and_reraise_exception():
                     if self.client_plugin('nova').is_not_found(e):
@@ -124,7 +121,7 @@ class ElasticIp(resource.Resource):
 
         instance_id = self.properties[self.INSTANCE_ID]
         if instance_id:
-            server = self.client().servers.get(instance_id)
+            server = self.nova().servers.get(instance_id)
             server.add_floating_ip(self._ipaddress())
 
     def handle_delete(self):
@@ -137,7 +134,7 @@ class ElasticIp(resource.Resource):
         instance_id = self.properties[self.INSTANCE_ID]
         if instance_id:
             try:
-                server = self.client().servers.get(instance_id)
+                server = self.nova().servers.get(instance_id)
                 if server:
                     server.remove_floating_ip(self._ipaddress())
             except Exception as e:
@@ -150,11 +147,15 @@ class ElasticIp(resource.Resource):
 
         # deallocate the eip
         if self.properties[self.DOMAIN]:
-            with self.client_plugin('neutron').ignore_not_found:
+            try:
                 self.neutron().delete_floatingip(self.resource_id)
+            except Exception as ex:
+                self.client_plugin('neutron').ignore_not_found(ex)
         else:
-            with self.client_plugin('nova').ignore_not_found:
-                self.client().floating_ips.delete(self.resource_id)
+            try:
+                self.nova().floating_ips.delete(self.resource_id)
+            except Exception as e:
+                self.client_plugin('nova').ignore_not_found(e)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
@@ -164,21 +165,17 @@ class ElasticIp(resource.Resource):
                     # no need to remove the floating ip from the old instance,
                     # nova does this automatically when calling
                     # add_floating_ip().
-                    server = self.client().servers.get(instance_id)
+                    server = self.nova().servers.get(instance_id)
                     server.add_floating_ip(self._ipaddress())
                 else:
                     # to remove the floating_ip from the old instance
                     instance_id_old = self.properties[self.INSTANCE_ID]
                     if instance_id_old:
-                        server = self.client().servers.get(instance_id_old)
+                        server = self.nova().servers.get(instance_id_old)
                         server.remove_floating_ip(self._ipaddress())
 
-    def get_reference_id(self):
-        eip = self._ipaddress()
-        if eip:
-            return six.text_type(eip)
-        else:
-            return six.text_type(self.name)
+    def FnGetRefId(self):
+        return six.text_type(self._ipaddress())
 
     def _resolve_attribute(self, name):
         if name == self.ALLOCATION_ID:
@@ -204,10 +201,7 @@ class ElasticIpAssociation(resource.Resource):
         EIP: properties.Schema(
             properties.Schema.STRING,
             _('EIP address to associate with instance.'),
-            update_allowed=True,
-            constraints=[
-                constraints.CustomConstraint('ip_addr')
-            ]
+            update_allowed=True
         ),
         ALLOCATION_ID: properties.Schema(
             properties.Schema.STRING,
@@ -221,13 +215,13 @@ class ElasticIpAssociation(resource.Resource):
         ),
     }
 
-    default_client_name = 'nova'
-
-    def get_reference_id(self):
+    def FnGetRefId(self):
         return self.physical_resource_name_or_FnGetRefId()
 
     def validate(self):
-        """Validate any of the provided parameters."""
+        '''
+        Validate any of the provided parameters
+        '''
         super(ElasticIpAssociation, self).validate()
         eip = self.properties[self.EIP]
         allocation_id = self.properties[self.ALLOCATION_ID]
@@ -284,7 +278,7 @@ class ElasticIpAssociation(resource.Resource):
                                  ignore_not_found=False):
         server = None
         try:
-            server = self.client().servers.get(instance_id)
+            server = self.nova().servers.get(instance_id)
             server.remove_floating_ip(eip)
         except Exception as e:
             is_not_found = self.client_plugin('nova').is_not_found(e)
@@ -347,7 +341,7 @@ class ElasticIpAssociation(resource.Resource):
         # if update portInfo, no need to detach the port from
         # old instance/floatingip.
         if eip:
-            server = self.client().servers.get(instance_id_update)
+            server = self.nova().servers.get(instance_id_update)
             server.add_floating_ip(eip)
         else:
             port_id, port_rsrc = self._get_port_info(ni_id_update,
@@ -361,11 +355,31 @@ class ElasticIpAssociation(resource.Resource):
             self._neutron_add_gateway_router(allocation_id, network_id)
             self._neutron_update_floating_ip(allocation_id, port_id)
 
+    def _validate_update_properties(self, prop_diff):
+        # according to aws doc, when update allocation_id or eip,
+        # if you also change the InstanceId or NetworkInterfaceId,
+        # should go to Replacement flow
+        if self.ALLOCATION_ID in prop_diff or self.EIP in prop_diff:
+            instance_id = prop_diff.get(self.INSTANCE_ID)
+            ni_id = prop_diff.get(self.NETWORK_INTERFACE_ID)
+
+            if instance_id or ni_id:
+                raise resource.UpdateReplace(self.name)
+
+        # according to aws doc, when update the instance_id or
+        # network_interface_id, if you also change the EIP or
+        # ALLOCATION_ID, should go to Replacement flow
+        if (self.INSTANCE_ID in prop_diff or
+                self.NETWORK_INTERFACE_ID in prop_diff):
+            eip = prop_diff.get(self.EIP)
+            allocation_id = prop_diff.get(self.ALLOCATION_ID)
+            if eip or allocation_id:
+                raise resource.UpdateReplace(self.name)
+
     def handle_create(self):
         """Add a floating IP address to a server."""
         if self.properties[self.EIP]:
-            server = self.client().servers.get(
-                self.properties[self.INSTANCE_ID])
+            server = self.nova().servers.get(self.properties[self.INSTANCE_ID])
             server.add_floating_ip(self.properties[self.EIP])
             self.resource_id_set(self.properties[self.EIP])
             LOG.debug('ElasticIpAssociation '
@@ -377,8 +391,7 @@ class ElasticIpAssociation(resource.Resource):
             instance_id = self.properties[self.INSTANCE_ID]
             port_id, port_rsrc = self._get_port_info(ni_id, instance_id)
             if not port_id or not port_rsrc:
-                LOG.warning(_LW('Skipping association, resource not '
-                                'specified'))
+                LOG.warn(_LW('Skipping association, resource not specified'))
                 return
 
             float_id = self.properties[self.ALLOCATION_ID]
@@ -406,19 +419,9 @@ class ElasticIpAssociation(resource.Resource):
                                              port_id=None,
                                              ignore_not_found=True)
 
-    def needs_replace_with_prop_diff(self, changed_properties_set,
-                                     after_props, before_props):
-        if (self.ALLOCATION_ID in changed_properties_set or
-                self.EIP in changed_properties_set):
-            instance_id, ni_id = None, None
-            if self.INSTANCE_ID in changed_properties_set:
-                instance_id = after_props.get(self.INSTANCE_ID)
-            if self.NETWORK_INTERFACE_ID in changed_properties_set:
-                ni_id = after_props.get(self.NETWORK_INTERFACE_ID)
-            return bool(instance_id or ni_id)
-
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
+            self._validate_update_properties(prop_diff)
             if self.ALLOCATION_ID in prop_diff or self.EIP in prop_diff:
                 self._handle_update_eipInfo(prop_diff)
             elif (self.INSTANCE_ID in prop_diff or
