@@ -114,7 +114,7 @@ class User(stack_user.StackUser):
         super(User, self).handle_create()
         self.resource_id_set(self._get_user_id())
 
-    def get_reference_id(self):
+    def FnGetRefId(self):
         return self.physical_resource_name_or_FnGetRefId()
 
     def access_allowed(self, resource_name):
@@ -167,13 +167,11 @@ class AccessKey(resource.Resource):
     attributes_schema = {
         USER_NAME: attributes.Schema(
             _('Username associated with the AccessKey.'),
-            cache_mode=attributes.Schema.CACHE_NONE,
-            type=attributes.Schema.STRING
+            cache_mode=attributes.Schema.CACHE_NONE
         ),
         SECRET_ACCESS_KEY: attributes.Schema(
             _('Keypair secret key.'),
-            cache_mode=attributes.Schema.CACHE_NONE,
-            type=attributes.Schema.STRING
+            cache_mode=attributes.Schema.CACHE_NONE
         ),
     }
 
@@ -184,13 +182,12 @@ class AccessKey(resource.Resource):
             self._register_access_key()
 
     def _get_user(self):
-        """Derive the keystone userid, stored in the User resource_id.
-
+        """
         Helper function to derive the keystone userid, which is stored in the
-        resource_id of the User associated with this key. We want to avoid
+        resource_id of the User associated with this key.  We want to avoid
         looking the name up via listing keystone users, as this requires admin
         rights in keystone, so FnGetAtt which calls _secret_accesskey won't
-        work for normal non-admin users.
+        work for normal non-admin users
         """
         # Lookup User resource by intrinsic reference (which is what is passed
         # into the UserName parameter.  Would be cleaner to just make the User
@@ -226,10 +223,9 @@ class AccessKey(resource.Resource):
         user._delete_keypair()
 
     def _secret_accesskey(self):
-        """Return the user's access key.
-
-        Fetching it from keystone if necessary.
-        """
+        '''
+        Return the user's access key, fetching it from keystone if necessary
+        '''
         if self._secret is None:
             if not self.resource_id:
                 LOG.info(_LI('could not get secret for %(username)s '
