@@ -20,7 +20,8 @@ from heat.rpc import api as rpc_api
 
 
 class CloudConfig(software_config.SoftwareConfig):
-    """A configuration resource for representing cloud-init cloud-config.
+    '''
+    A configuration resource for representing cloud-init cloud-config.
 
     This resource allows cloud-config YAML to be defined and stored by the
     config API. Any intrinsic functions called in the config will be resolved
@@ -30,7 +31,7 @@ class CloudConfig(software_config.SoftwareConfig):
     or OS::Heat::MultipartMime parts config. Since cloud-config is boot-only
     configuration, any changes to the definition will result in the
     replacement of all servers which reference it.
-    """
+    '''
 
     support_status = support.SupportStatus(version='2014.1')
 
@@ -49,9 +50,8 @@ class CloudConfig(software_config.SoftwareConfig):
     }
 
     def handle_create(self):
-        cloud_config = template_format.yaml.dump(
-            self.properties[self.CLOUD_CONFIG],
-            Dumper=template_format.yaml_dumper)
+        cloud_config = template_format.yaml.dump(self.properties.get(
+            self.CLOUD_CONFIG), Dumper=template_format.yaml_dumper)
         props = {
             self.NAME: self.physical_resource_name(),
             self.CONFIG: '#cloud-config\n%s' % cloud_config,
