@@ -26,7 +26,9 @@ from heat.rpc import api as rpc_api
 
 
 class MultipartMime(software_config.SoftwareConfig):
-    """Assembles a collection of software configurations as a multi-part mime.
+    '''
+    A resource which assembles a collection of software configurations
+    as a multi-part mime message.
 
     Parts in the message can be populated with inline configuration or
     references to other config resources. If the referenced resource is itself
@@ -40,7 +42,7 @@ class MultipartMime(software_config.SoftwareConfig):
     configuration elements including scripts and cloud-config. Since
     cloud-init is boot-only configuration, any changes to the definition
     will result in the replacement of all servers which reference it.
-    """
+    '''
 
     support_status = support.SupportStatus(version='2014.1')
 
@@ -68,7 +70,7 @@ class MultipartMime(software_config.SoftwareConfig):
                         properties.Schema.STRING,
                         _('Content of part to attach, either inline or by '
                           'referencing the ID of another software config '
-                          'resource.'),
+                          'resource'),
                         required=True
                     ),
                     FILENAME: properties.Schema(
@@ -94,9 +96,9 @@ class MultipartMime(software_config.SoftwareConfig):
 
     def handle_create(self):
         props = {
-            rpc_api.SOFTWARE_CONFIG_NAME: self.physical_resource_name(),
-            rpc_api.SOFTWARE_CONFIG_CONFIG: self.get_message(),
-            rpc_api.SOFTWARE_CONFIG_GROUP: 'Heat::Ungrouped'
+            self.NAME: self.physical_resource_name(),
+            self.CONFIG: self.get_message(),
+            self.GROUP: 'Heat::Ungrouped'
         }
         sc = self.rpc_client().create_software_config(self.context, **props)
         self.resource_id_set(sc[rpc_api.SOFTWARE_CONFIG_ID])
@@ -106,15 +108,18 @@ class MultipartMime(software_config.SoftwareConfig):
             return self.message
 
         subparts = []
-        for item in self.properties[self.PARTS]:
+        for item in self.properties.get(self.PARTS):
             config = item.get(self.CONFIG)
             part_type = item.get(self.TYPE, self.TEXT)
             part = config
 
             if uuidutils.is_uuid_like(config):
-                with self.rpc_client().ignore_error_by_name('NotFound'):
+                try:
                     sc = self.rpc_client().show_software_config(
                         self.context, config)
+                except Exception as ex:
+                    self.rpc_client().ignore_error_named(ex, 'NotFound')
+                else:
                     part = sc[rpc_api.SOFTWARE_CONFIG_CONFIG]
 
             if part_type == self.MULTIPART:
@@ -151,15 +156,8 @@ class MultipartMime(software_config.SoftwareConfig):
 
     @staticmethod
     def _create_message(part, subtype, filename):
-        charset = 'us-ascii'
-        try:
-            part.encode(charset)
-        except UnicodeEncodeError:
-            charset = 'utf-8'
-        msg = (text.MIMEText(part, _subtype=subtype,
-                             _charset=charset)
-               if subtype else text.MIMEText(part, _charset=charset))
-
+        msg = (text.MIMEText(part, _subtype=subtype)
+               if subtype else text.MIMEText(part))
         if filename:
             msg.add_header('Content-Disposition', 'attachment',
                            filename=filename)
