@@ -214,6 +214,8 @@ class ResourceGroup(stack_resource.StackResource):
     def _name_blacklist(self):
         """Resolve the remove_policies to names for removal."""
 
+        nested = self.nested()
+
         # To avoid reusing names after removal, we store a comma-separated
         # blacklist in the resource data
         db_rsrc_names = self.data().get('name_blacklist')
@@ -225,17 +227,18 @@ class ResourceGroup(stack_resource.StackResource):
         # Now we iterate over the removal policies, and update the blacklist
         # with any additional names
         rsrc_names = list(current_blacklist)
-        for r in self.properties[self.REMOVAL_POLICIES]:
-            if self.REMOVAL_RSRC_LIST in r:
-                # Tolerate string or int list values
-                for n in r[self.REMOVAL_RSRC_LIST]:
-                    str_n = six.text_type(n)
-                    if str_n in self.nested() and str_n not in rsrc_names:
-                        rsrc_names.append(str_n)
-                        continue
-                    rsrc = self.nested().resource_by_refid(str_n)
-                    if rsrc and str_n not in rsrc_names:
-                        rsrc_names.append(rsrc.name)
+        if nested:
+            for r in self.properties[self.REMOVAL_POLICIES]:
+                if self.REMOVAL_RSRC_LIST in r:
+                    # Tolerate string or int list values
+                    for n in r[self.REMOVAL_RSRC_LIST]:
+                        str_n = six.text_type(n)
+                        if str_n in nested and str_n not in rsrc_names:
+                            rsrc_names.append(str_n)
+                            continue
+                        rsrc = self.nested().resource_by_refid(str_n)
+                        if rsrc and str_n not in rsrc_names:
+                            rsrc_names.append(rsrc.name)
 
         # If the blacklist has changed, update the resource data
         if rsrc_names != current_blacklist:
