@@ -11,18 +11,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
 
 from heat.common.i18n import _
 from heat.engine import attributes
+from heat.engine import constraints
 from heat.engine import properties
 from heat.engine import resource
-from heat.engine import software_config_io as swc_io
 from heat.engine import support
 from heat.rpc import api as rpc_api
 
+LOG = logging.getLogger(__name__)
+
 
 class SoftwareConfig(resource.Resource):
-    """A resource for describing and storing software configuration.
+    '''
+    A resource for describing and storing software configuration.
 
     The software_configs API which backs this resource creates immutable
     configs, so any change to the template resource definition will result
@@ -40,18 +44,20 @@ class SoftwareConfig(resource.Resource):
     inputs and outputs which the config script is expected to consume and
     produce. Inputs and outputs are optional and will map to concepts which
     are specific to the configuration tool being used.
-    """
+    '''
 
     support_status = support.SupportStatus(version='2014.1')
 
     PROPERTIES = (
-        GROUP, CONFIG,
-        OPTIONS,
-        INPUTS, OUTPUTS
+        GROUP, CONFIG, OPTIONS, INPUTS, OUTPUTS
     ) = (
-        rpc_api.SOFTWARE_CONFIG_GROUP, rpc_api.SOFTWARE_CONFIG_CONFIG,
-        rpc_api.SOFTWARE_CONFIG_OPTIONS,
-        rpc_api.SOFTWARE_CONFIG_INPUTS, rpc_api.SOFTWARE_CONFIG_OUTPUTS,
+        'group', 'config', 'options', 'inputs', 'outputs'
+    )
+
+    IO_PROPERTIES = (
+        NAME, DESCRIPTION, TYPE, DEFAULT, ERROR_OUTPUT
+    ) = (
+        'name', 'description', 'type', 'default', 'error_output'
     )
 
     ATTRIBUTES = (
@@ -60,6 +66,54 @@ class SoftwareConfig(resource.Resource):
         'config',
     )
 
+    input_schema = {
+        NAME: properties.Schema(
+            properties.Schema.STRING,
+            _('Name of the input.'),
+            required=True
+        ),
+        DESCRIPTION: properties.Schema(
+            properties.Schema.STRING,
+            _('Description of the input.')
+        ),
+        TYPE: properties.Schema(
+            properties.Schema.STRING,
+            _('Type of the value of the input.'),
+            default='String',
+            constraints=[constraints.AllowedValues((
+                'String', 'Number', 'CommaDelimitedList', 'Json'))]
+        ),
+        DEFAULT: properties.Schema(
+            properties.Schema.STRING,
+            _('Default value for the input if none is specified.'),
+        ),
+    }
+
+    output_schema = {
+        NAME: properties.Schema(
+            properties.Schema.STRING,
+            _('Name of the output.'),
+            required=True
+        ),
+        DESCRIPTION: properties.Schema(
+            properties.Schema.STRING,
+            _('Description of the output.')
+        ),
+        TYPE: properties.Schema(
+            properties.Schema.STRING,
+            _('Type of the value of the output.'),
+            default='String',
+            constraints=[constraints.AllowedValues((
+                'String', 'Number', 'CommaDelimitedList', 'Json'))]
+        ),
+        ERROR_OUTPUT: properties.Schema(
+            properties.Schema.BOOLEAN,
+            _('Denotes that the deployment is in an error state if this '
+              'output has a value.'),
+            default=False
+        )
+    }
+
     properties_schema = {
         GROUP: properties.Schema(
             properties.Schema.STRING,
@@ -83,27 +137,26 @@ class SoftwareConfig(resource.Resource):
             _('Schema representing the inputs that this software config is '
               'expecting.'),
             schema=properties.Schema(properties.Schema.MAP,
-                                     schema=swc_io.input_config_schema)
+                                     schema=input_schema)
         ),
         OUTPUTS: properties.Schema(
             properties.Schema.LIST,
             _('Schema representing the outputs that this software config '
               'will produce.'),
             schema=properties.Schema(properties.Schema.MAP,
-                                     schema=swc_io.output_config_schema)
+                                     schema=output_schema)
         ),
     }
 
     attributes_schema = {
         CONFIG_ATTR: attributes.Schema(
-            _("The config value of the software config."),
-            type=attributes.Schema.STRING
+            _("The config value of the software config.")
         ),
     }
 
     def handle_create(self):
         props = dict(self.properties)
-        props[rpc_api.SOFTWARE_CONFIG_NAME] = self.physical_resource_name()
+        props[self.NAME] = self.physical_resource_name()
 
         sc = self.rpc_client().create_software_config(self.context, **props)
         self.resource_id_set(sc[rpc_api.SOFTWARE_CONFIG_ID])
@@ -120,11 +173,10 @@ class SoftwareConfig(resource.Resource):
             self.rpc_client().ignore_error_named(ex, 'NotFound')
 
     def _resolve_attribute(self, name):
-        """Retrieve attributes of the SoftwareConfig resource.
-
+        '''
         "config" returns the config value of the software config. If the
          software config does not exist, returns an empty string.
-        """
+        '''
         if name == self.CONFIG_ATTR and self.resource_id:
             try:
                 sc = self.rpc_client().show_software_config(
