@@ -11,8 +11,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import six
-
 from heat.common import exception
 from heat.common.i18n import _
 from heat.engine import attributes
@@ -21,47 +19,30 @@ from heat.engine import properties
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine.resources.openstack.neutron import subnet
 from heat.engine import support
-from heat.engine import translation
 
 
 class Router(neutron.NeutronResource):
-    """A resource that implements Neutron router.
-
-    Router is a physical or virtual network device that passes network traffic
-    between different networks.
-    """
-
-    required_service_extension = 'router'
-
-    entity = 'router'
 
     PROPERTIES = (
         NAME, EXTERNAL_GATEWAY, VALUE_SPECS, ADMIN_STATE_UP,
-        L3_AGENT_ID, L3_AGENT_IDS, DISTRIBUTED, HA, TAGS,
+        L3_AGENT_ID, L3_AGENT_IDS, DISTRIBUTED, HA,
     ) = (
         'name', 'external_gateway_info', 'value_specs', 'admin_state_up',
-        'l3_agent_id', 'l3_agent_ids', 'distributed', 'ha', 'tags',
+        'l3_agent_id', 'l3_agent_ids', 'distributed', 'ha'
     )
 
     _EXTERNAL_GATEWAY_KEYS = (
         EXTERNAL_GATEWAY_NETWORK, EXTERNAL_GATEWAY_ENABLE_SNAT,
-        EXTERNAL_GATEWAY_FIXED_IPS,
     ) = (
-        'network', 'enable_snat', 'external_fixed_ips',
-    )
-
-    _EXTERNAL_GATEWAY_FIXED_IPS_KEYS = (
-        IP_ADDRESS, SUBNET
-    ) = (
-        'ip_address', 'subnet'
+        'network', 'enable_snat',
     )
 
     ATTRIBUTES = (
         STATUS, EXTERNAL_GATEWAY_INFO_ATTR, NAME_ATTR, ADMIN_STATE_UP_ATTR,
-        TENANT_ID,
+        TENANT_ID, SHOW,
     ) = (
         'status', 'external_gateway_info', 'name', 'admin_state_up',
-        'tenant_id',
+        'tenant_id', 'show',
     )
 
     properties_schema = {
@@ -87,32 +68,6 @@ class Router(neutron.NeutronResource):
                       'this property to administrative users only.'),
                     update_allowed=True
                 ),
-                EXTERNAL_GATEWAY_FIXED_IPS: properties.Schema(
-                    properties.Schema.LIST,
-                    _('External fixed IP addresses for the gateway.'),
-                    schema=properties.Schema(
-                        properties.Schema.MAP,
-                        schema={
-                            IP_ADDRESS: properties.Schema(
-                                properties.Schema.STRING,
-                                _('External fixed IP address.'),
-                                constraints=[
-                                    constraints.CustomConstraint('ip_addr'),
-                                ]
-                            ),
-                            SUBNET: properties.Schema(
-                                properties.Schema.STRING,
-                                _('Subnet of external fixed IP address.'),
-                                constraints=[
-                                    constraints.CustomConstraint(
-                                        'neutron.subnet')
-                                ]
-                            ),
-                        }
-                    ),
-                    update_allowed=True,
-                    support_status=support.SupportStatus(version='6.0.0')
-                ),
             },
             update_allowed=True
         ),
@@ -135,15 +90,8 @@ class Router(neutron.NeutronResource):
               'users only.'),
             update_allowed=True,
             support_status=support.SupportStatus(
-                status=support.HIDDEN,
-                version='6.0.0',
-                previous_status=support.SupportStatus(
-                    status=support.DEPRECATED,
-                    version='2015.1',
-                    message=_('Use property %s.') % L3_AGENT_IDS,
-                    previous_status=support.SupportStatus(version='2014.1')
-                )
-            ),
+                support.DEPRECATED,
+                _('Deprecated in Kilo. Use property %s.') % L3_AGENT_IDS),
         ),
         L3_AGENT_IDS: properties.Schema(
             properties.Schema.LIST,
@@ -173,76 +121,35 @@ class Router(neutron.NeutronResource):
               'do not support distributed and ha at the same time.'),
             support_status=support.SupportStatus(version='2015.1')
         ),
-        TAGS: properties.Schema(
-            properties.Schema.LIST,
-            _('The tags to be added to the router.'),
-            schema=properties.Schema(properties.Schema.STRING),
-            update_allowed=True,
-            support_status=support.SupportStatus(version='9.0.0')
-        ),
     }
 
     attributes_schema = {
         STATUS: attributes.Schema(
-            _("The status of the router."),
-            type=attributes.Schema.STRING
+            _("The status of the router.")
         ),
         EXTERNAL_GATEWAY_INFO_ATTR: attributes.Schema(
-            _("Gateway network for the router."),
-            type=attributes.Schema.MAP
+            _("Gateway network for the router.")
         ),
         NAME_ATTR: attributes.Schema(
-            _("Friendly name of the router."),
-            type=attributes.Schema.STRING
+            _("Friendly name of the router.")
         ),
         ADMIN_STATE_UP_ATTR: attributes.Schema(
-            _("Administrative state of the router."),
-            type=attributes.Schema.STRING
+            _("Administrative state of the router.")
         ),
         TENANT_ID: attributes.Schema(
-            _("Tenant owning the router."),
-            type=attributes.Schema.STRING
+            _("Tenant owning the router.")
+        ),
+        SHOW: attributes.Schema(
+            _("All attributes.")
         ),
     }
 
-    def translation_rules(self, props):
-        rules = [
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.RESOLVE,
-                [self.EXTERNAL_GATEWAY, self.EXTERNAL_GATEWAY_NETWORK],
-                client_plugin=self.client_plugin(),
-                finder='find_resourceid_by_name_or_id',
-                entity='network'),
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.RESOLVE,
-                [self.EXTERNAL_GATEWAY, self.EXTERNAL_GATEWAY_FIXED_IPS,
-                 self.SUBNET],
-                client_plugin=self.client_plugin(),
-                finder='find_resourceid_by_name_or_id',
-                entity='subnet')
-            ]
-        if props.get(self.L3_AGENT_ID):
-            rules.extend([
-                translation.TranslationRule(
-                    props,
-                    translation.TranslationRule.ADD,
-                    [self.L3_AGENT_IDS],
-                    [props.get(self.L3_AGENT_ID)]),
-                translation.TranslationRule(
-                    props,
-                    translation.TranslationRule.DELETE,
-                    [self.L3_AGENT_ID]
-                )])
-        return rules
-
     def validate(self):
         super(Router, self).validate()
-        is_distributed = self.properties[self.DISTRIBUTED]
-        l3_agent_id = self.properties[self.L3_AGENT_ID]
-        l3_agent_ids = self.properties[self.L3_AGENT_IDS]
-        is_ha = self.properties[self.HA]
+        is_distributed = self.properties.get(self.DISTRIBUTED)
+        l3_agent_id = self.properties.get(self.L3_AGENT_ID)
+        l3_agent_ids = self.properties.get(self.L3_AGENT_IDS)
+        is_ha = self.properties.get(self.HA)
         if l3_agent_id and l3_agent_ids:
             raise exception.ResourcePropertyConflict(self.L3_AGENT_ID,
                                                      self.L3_AGENT_IDS)
@@ -260,25 +167,25 @@ class Router(neutron.NeutronResource):
 
     def add_dependencies(self, deps):
         super(Router, self).add_dependencies(deps)
-        external_gw = self.properties[self.EXTERNAL_GATEWAY]
+        external_gw = self.properties.get(self.EXTERNAL_GATEWAY)
         if external_gw:
             external_gw_net = external_gw.get(self.EXTERNAL_GATEWAY_NETWORK)
-            for res in six.itervalues(self.stack):
+            for res in self.stack.itervalues():
                 if res.has_interface('OS::Neutron::Subnet'):
-                    subnet_net = res.properties.get(subnet.Subnet.NETWORK)
+                    subnet_net = res.properties.get(
+                        subnet.Subnet.NETWORK) or res.properties.get(
+                            subnet.Subnet.NETWORK_ID)
                     if subnet_net == external_gw_net:
                         deps += (self, res)
 
-    def _resolve_gateway(self, props):
+    def prepare_properties(self, properties, name):
+        props = super(Router, self).prepare_properties(properties, name)
         gateway = props.get(self.EXTERNAL_GATEWAY)
         if gateway:
-            gateway['network_id'] = gateway.pop(self.EXTERNAL_GATEWAY_NETWORK)
+            self.client_plugin().resolve_network(
+                gateway, self.EXTERNAL_GATEWAY_NETWORK, 'network_id')
             if gateway[self.EXTERNAL_GATEWAY_ENABLE_SNAT] is None:
                 del gateway[self.EXTERNAL_GATEWAY_ENABLE_SNAT]
-            if gateway[self.EXTERNAL_GATEWAY_FIXED_IPS] is None:
-                del gateway[self.EXTERNAL_GATEWAY_FIXED_IPS]
-            else:
-                self._resolve_subnet(gateway)
         return props
 
     def _get_l3_agent_list(self, props):
@@ -289,101 +196,64 @@ class Router(neutron.NeutronResource):
 
         return l3_agent_ids
 
-    def _resolve_subnet(self, gateway):
-        external_gw_fixed_ips = gateway[self.EXTERNAL_GATEWAY_FIXED_IPS]
-        for fixed_ip in external_gw_fixed_ips:
-            for key, value in six.iteritems(fixed_ip):
-                if value is None:
-                    fixed_ip.pop(key)
-            if self.SUBNET in fixed_ip:
-                fixed_ip['subnet_id'] = fixed_ip.pop(self.SUBNET)
-
     def handle_create(self):
         props = self.prepare_properties(
             self.properties,
             self.physical_resource_name())
-        self._resolve_gateway(props)
-        l3_agent_ids = self._get_l3_agent_list(props)
-        tags = props.pop(self.TAGS, [])
 
-        router = self.client().create_router({'router': props})['router']
+        l3_agent_ids = self._get_l3_agent_list(props)
+
+        router = self.neutron().create_router({'router': props})['router']
         self.resource_id_set(router['id'])
 
         if l3_agent_ids:
             self._replace_agent(l3_agent_ids)
-        if tags:
-            self.set_tags(tags)
+
+    def _show_resource(self):
+        return self.neutron().show_router(
+            self.resource_id)['router']
 
     def check_create_complete(self, *args):
         attributes = self._show_resource()
         return self.is_built(attributes)
 
     def handle_delete(self):
+        client = self.neutron()
         try:
-            self.client().delete_router(self.resource_id)
+            client.delete_router(self.resource_id)
         except Exception as ex:
             self.client_plugin().ignore_not_found(ex)
         else:
             return True
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        if self.EXTERNAL_GATEWAY in prop_diff:
-            self._resolve_gateway(prop_diff)
-
-        if self.L3_AGENT_IDS in prop_diff or self.L3_AGENT_ID in prop_diff:
-            l3_agent_ids = self._get_l3_agent_list(prop_diff)
+        props = self.prepare_update_properties(json_snippet)
+        l3_agent_ids = self._get_l3_agent_list(props)
+        if l3_agent_ids:
             self._replace_agent(l3_agent_ids)
 
-        if self.TAGS in prop_diff:
-            tags = prop_diff.pop(self.TAGS)
-            self.set_tags(tags)
+        if self.L3_AGENT_IDS in prop_diff:
+            del prop_diff[self.L3_AGENT_IDS]
+        if self.L3_AGENT_ID in prop_diff:
+            del prop_diff[self.L3_AGENT_ID]
 
-        if prop_diff:
-            self.prepare_update_properties(prop_diff)
-            self.client().update_router(
-                self.resource_id, {'router': prop_diff})
+        if len(prop_diff) > 0:
+            self.neutron().update_router(
+                self.resource_id, {'router': props})
 
     def _replace_agent(self, l3_agent_ids=None):
-        ret = self.client().list_l3_agent_hosting_routers(
+        ret = self.neutron().list_l3_agent_hosting_routers(
             self.resource_id)
         for agent in ret['agents']:
-            self.client().remove_router_from_l3_agent(
+            self.neutron().remove_router_from_l3_agent(
                 agent['id'], self.resource_id)
         if l3_agent_ids:
             for l3_agent_id in l3_agent_ids:
-                self.client().add_router_to_l3_agent(
+                self.neutron().add_router_to_l3_agent(
                     l3_agent_id, {'router_id': self.resource_id})
 
-    def parse_live_resource_data(self, resource_properties, resource_data):
-        result = super(Router, self).parse_live_resource_data(
-            resource_properties, resource_data)
-
-        try:
-            ret = self.client().list_l3_agent_hosting_routers(self.resource_id)
-            if ret:
-                result[self.L3_AGENT_IDS] = list(
-                    agent['id'] for agent in ret['agents'])
-        except self.client_plugin().exceptions.Forbidden:
-            # Just pass if forbidden
-            pass
-
-        gateway = resource_data.get(self.EXTERNAL_GATEWAY)
-        if gateway is not None:
-            result[self.EXTERNAL_GATEWAY] = {
-                self.EXTERNAL_GATEWAY_NETWORK: gateway.get('network_id'),
-                self.EXTERNAL_GATEWAY_ENABLE_SNAT: gateway.get('enable_snat')
-            }
-        return result
-
 
 class RouterInterface(neutron.NeutronResource):
-    """A resource for managing Neutron router interfaces.
-
-    Router interfaces associate routers with existing subnets or ports.
-    """
-
-    required_service_extension = 'router'
-
     PROPERTIES = (
         ROUTER, ROUTER_ID, SUBNET_ID, SUBNET, PORT_ID, PORT
     ) = (
@@ -394,7 +264,6 @@ class RouterInterface(neutron.NeutronResource):
         ROUTER: properties.Schema(
             properties.Schema.STRING,
             _('The router.'),
-            required=True,
             constraints=[
                 constraints.CustomConstraint('neutron.router')
             ],
@@ -403,15 +272,8 @@ class RouterInterface(neutron.NeutronResource):
             properties.Schema.STRING,
             _('ID of the router.'),
             support_status=support.SupportStatus(
-                status=support.HIDDEN,
-                version='6.0.0',
-                previous_status=support.SupportStatus(
-                    status=support.DEPRECATED,
-                    message=_('Use property %s.') % ROUTER,
-                    version='2015.1',
-                    previous_status=support.SupportStatus(version='2013.1')
-                )
-            ),
+                support.DEPRECATED,
+                _('Use property %s.') % ROUTER),
             constraints=[
                 constraints.CustomConstraint('neutron.router')
             ],
@@ -419,14 +281,8 @@ class RouterInterface(neutron.NeutronResource):
         SUBNET_ID: properties.Schema(
             properties.Schema.STRING,
             support_status=support.SupportStatus(
-                status=support.HIDDEN,
-                message=_('Use property %s.') % SUBNET,
-                version='5.0.0',
-                previous_status=support.SupportStatus(
-                    status=support.DEPRECATED,
-                    version='2014.2'
-                )
-            ),
+                support.DEPRECATED,
+                _('Use property %s.') % SUBNET),
             constraints=[
                 constraints.CustomConstraint('neutron.subnet')
             ]
@@ -443,15 +299,9 @@ class RouterInterface(neutron.NeutronResource):
             properties.Schema.STRING,
             _('The port id, either subnet or port_id should be specified.'),
             support_status=support.SupportStatus(
-                status=support.HIDDEN,
-                version='6.0.0',
-                previous_status=support.SupportStatus(
-                    status=support.DEPRECATED,
-                    message=_('Use property %s.') % PORT,
-                    version='2015.1',
-                    previous_status=support.SupportStatus(version='2014.1')
-                )
-            ),
+                support.DEPRECATED,
+                _('Deprecated in Kilo. '
+                  'Use property %s.') % PORT),
             constraints=[
                 constraints.CustomConstraint('neutron.port')
             ]
@@ -466,61 +316,30 @@ class RouterInterface(neutron.NeutronResource):
         )
     }
 
-    def translation_rules(self, props):
-        return [
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.REPLACE,
-                [self.PORT],
-                value_path=[self.PORT_ID]
-            ),
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.REPLACE,
-                [self.ROUTER],
-                value_path=[self.ROUTER_ID]
-            ),
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.REPLACE,
-                [self.SUBNET],
-                value_path=[self.SUBNET_ID]
-            ),
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.RESOLVE,
-                [self.PORT],
-                client_plugin=self.client_plugin(),
-                finder='find_resourceid_by_name_or_id',
-                entity='port'
-            ),
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.RESOLVE,
-                [self.ROUTER],
-                client_plugin=self.client_plugin(),
-                finder='find_resourceid_by_name_or_id',
-                entity='router'
-            ),
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.RESOLVE,
-                [self.SUBNET],
-                client_plugin=self.client_plugin(),
-                finder='find_resourceid_by_name_or_id',
-                entity='subnet'
-            )
-
-
-        ]
+    @staticmethod
+    def _validate_deprecated_keys(props, key, deprecated_key):
+        value = props.get(key)
+        deprecated_value = props.get(deprecated_key)
+        if value and deprecated_value:
+            raise exception.ResourcePropertyConflict(key,
+                                                     deprecated_key)
+        if value is None and deprecated_value is None:
+            return False
+        return True
 
     def validate(self):
         """Validate any of the provided params."""
         super(RouterInterface, self).validate()
 
-        prop_subnet_exists = self.properties.get(self.SUBNET) is not None
+        prop_subnet_exists = self._validate_deprecated_keys(
+            self.properties, self.SUBNET, self.SUBNET_ID)
+        if not self._validate_deprecated_keys(
+                self.properties, self.ROUTER, self.ROUTER_ID):
+            raise exception.PropertyUnspecifiedError(self.ROUTER,
+                                                     self.ROUTER_ID)
 
-        prop_port_exists = self.properties.get(self.PORT) is not None
+        prop_port_exists = self._validate_deprecated_keys(
+            self.properties, self.PORT, self.PORT_ID)
 
         if prop_subnet_exists and prop_port_exists:
             raise exception.ResourcePropertyConflict(self.SUBNET,
@@ -531,13 +350,16 @@ class RouterInterface(neutron.NeutronResource):
                                                      self.PORT)
 
     def handle_create(self):
-        router_id = dict(self.properties).get(self.ROUTER)
+        router_id = self.client_plugin().resolve_router(
+            dict(self.properties), self.ROUTER, self.ROUTER_ID)
         key = 'subnet_id'
-        value = dict(self.properties).get(self.SUBNET)
+        value = self.client_plugin().resolve_subnet(
+            dict(self.properties), self.SUBNET, key)
         if not value:
             key = 'port_id'
-            value = dict(self.properties).get(self.PORT)
-        self.client().add_interface_router(
+            value = self.client_plugin().resolve_port(
+                dict(self.properties), self.PORT, key)
+        self.neutron().add_interface_router(
             router_id,
             {key: value})
         self.resource_id_set('%s:%s=%s' % (router_id, key, value))
@@ -545,27 +367,26 @@ class RouterInterface(neutron.NeutronResource):
     def handle_delete(self):
         if not self.resource_id:
             return
+        client = self.neutron()
         tokens = self.resource_id.replace('=', ':').split(':')
         if len(tokens) == 2:    # compatible with old data
             tokens.insert(1, 'subnet_id')
         (router_id, key, value) = tokens
-        with self.client_plugin().ignore_not_found:
-            self.client().remove_interface_router(
+        try:
+            client.remove_interface_router(
                 router_id,
                 {key: value})
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
 
 
 class RouterGateway(neutron.NeutronResource):
 
     support_status = support.SupportStatus(
-        status=support.HIDDEN,
-        message=_('Use the `external_gateway_info` property in '
-                  'the router resource to set up the gateway.'),
-        version='5.0.0',
-        previous_status=support.SupportStatus(
-            status=support.DEPRECATED,
-            version='2014.1'
-        )
+        support.DEPRECATED,
+        _('RouterGateway resource is deprecated and should not be used. '
+          'Instead use the `external_gateway_info` property in the router '
+          'resource to set up the gateway.')
     )
 
     PROPERTIES = (
@@ -586,14 +407,9 @@ class RouterGateway(neutron.NeutronResource):
         NETWORK_ID: properties.Schema(
             properties.Schema.STRING,
             support_status=support.SupportStatus(
-                status=support.HIDDEN,
-                message=_('Use property %s.') % NETWORK,
-                version='9.0.0',
-                previous_status=support.SupportStatus(
-                    status=support.DEPRECATED,
-                    version='2014.2'
-                )
-            ),
+                support.DEPRECATED,
+                _('Use property %s.') % NETWORK),
+            required=False,
             constraints=[
                 constraints.CustomConstraint('neutron.network')
             ],
@@ -601,6 +417,7 @@ class RouterGateway(neutron.NeutronResource):
         NETWORK: properties.Schema(
             properties.Schema.STRING,
             _('external network for the gateway.'),
+            required=False,
             constraints=[
                 constraints.CustomConstraint('neutron.network')
             ],
@@ -608,48 +425,39 @@ class RouterGateway(neutron.NeutronResource):
 
     }
 
-    def translation_rules(self, props):
-        return [
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.REPLACE,
-                [self.NETWORK],
-                value_path=[self.NETWORK_ID]
-            ),
-            translation.TranslationRule(
-                props,
-                translation.TranslationRule.RESOLVE,
-                [self.NETWORK],
-                client_plugin=self.client_plugin(),
-                finder='find_resourceid_by_name_or_id',
-                entity='network'
-            )
-
-        ]
+    def validate(self):
+        super(RouterGateway, self).validate()
+        self._validate_depr_property_required(
+            self.properties, self.NETWORK, self.NETWORK_ID)
 
     def add_dependencies(self, deps):
         super(RouterGateway, self).add_dependencies(deps)
-        for resource in six.itervalues(self.stack):
+        for resource in self.stack.itervalues():
             # depend on any RouterInterface in this template with the same
             # router_id as this router_id
             if resource.has_interface('OS::Neutron::RouterInterface'):
-                dep_router_id = resource.properties[RouterInterface.ROUTER]
-                router_id = self.properties[self.ROUTER_ID]
+                dep_router_id = resource.properties.get(
+                    RouterInterface.ROUTER_ID)
+                router_id = self.properties.get(self.ROUTER_ID)
                 if dep_router_id == router_id:
                     deps += (self, resource)
             # depend on any subnet in this template with the same network_id
             # as this network_id, as the gateway implicitly creates a port
             # on that subnet
             if resource.has_interface('OS::Neutron::Subnet'):
-                dep_network = resource.properties[subnet.Subnet.NETWORK]
-                network = self.properties[self.NETWORK]
+                dep_network = resource.properties.get(
+                    subnet.Subnet.NETWORK) or resource.properties.get(
+                        subnet.Subnet.NETWORK_ID)
+                network = self.properties.get(
+                    self.NETWORK) or self.properties.get(self.NETWORK_ID)
                 if dep_network == network:
                     deps += (self, resource)
 
     def handle_create(self):
-        router_id = self.properties[self.ROUTER_ID]
-        network_id = dict(self.properties).get(self.NETWORK)
-        self.client().add_gateway_router(
+        router_id = self.properties.get(self.ROUTER_ID)
+        network_id = self.client_plugin().resolve_network(
+            dict(self.properties), self.NETWORK, 'network_id')
+        self.neutron().add_gateway_router(
             router_id,
             {'network_id': network_id})
         self.resource_id_set('%s:%s' % (router_id, network_id))
@@ -657,10 +465,12 @@ class RouterGateway(neutron.NeutronResource):
     def handle_delete(self):
         if not self.resource_id:
             return
-
+        client = self.neutron()
         (router_id, network_id) = self.resource_id.split(':')
-        with self.client_plugin().ignore_not_found:
-            self.client().remove_gateway_router(router_id)
+        try:
+            client.remove_gateway_router(router_id)
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
 
 
 def resource_mapping():
