@@ -24,10 +24,6 @@ class ServerGroup(resource.Resource):
 
     support_status = support.SupportStatus(version='2014.2')
 
-    default_client_name = 'nova'
-
-    entity = 'server_groups'
-
     PROPERTIES = (
         NAME, POLICIES
     ) = (
@@ -56,12 +52,19 @@ class ServerGroup(resource.Resource):
     def handle_create(self):
         name = self.physical_resource_name()
         policies = self.properties[self.POLICIES]
-        server_group = self.client().server_groups.create(name=name,
-                                                          policies=policies)
+        server_group = self.nova().server_groups.create(name=name,
+                                                        policies=policies)
         self.resource_id_set(server_group.id)
 
+    def handle_delete(self):
+        if self.resource_id:
+            try:
+                self.nova().server_groups.delete(self.resource_id)
+            except Exception as e:
+                self.client_plugin('nova').ignore_not_found(e)
+
     def physical_resource_name(self):
-        name = self.properties[self.NAME]
+        name = self.properties.get(self.NAME)
         if name:
             return name
         return super(ServerGroup, self).physical_resource_name()
