@@ -27,11 +27,6 @@ LOG = logging.getLogger(__name__)
 
 
 class SwiftContainer(resource.Resource):
-    """A resource for managing Swift containers.
-
-    A container defines a namespace for objects. An object with the same
-    name in two different containers represents two different objects.
-    """
     PROPERTIES = (
         NAME, X_CONTAINER_READ, X_CONTAINER_WRITE, X_CONTAINER_META,
         X_ACCOUNT_META, PURGE_ON_DELETE,
@@ -92,35 +87,29 @@ class SwiftContainer(resource.Resource):
 
     attributes_schema = {
         DOMAIN_NAME: attributes.Schema(
-            _('The host from the container URL.'),
-            type=attributes.Schema.STRING
+            _('The host from the container URL.')
         ),
         WEBSITE_URL: attributes.Schema(
-            _('The URL of the container.'),
-            type=attributes.Schema.STRING
+            _('The URL of the container.')
         ),
         ROOT_URL: attributes.Schema(
-            _('The parent URL of the container.'),
-            type=attributes.Schema.STRING
+            _('The parent URL of the container.')
         ),
         OBJECT_COUNT: attributes.Schema(
-            _('The number of objects stored in the container.'),
-            type=attributes.Schema.INTEGER
+            _('The number of objects stored in the container.')
         ),
         BYTES_USED: attributes.Schema(
-            _('The number of bytes stored in the container.'),
-            type=attributes.Schema.INTEGER
+            _('The number of bytes stored in the container.')
         ),
         HEAD_CONTAINER: attributes.Schema(
-            _('A map containing all headers for the container.'),
-            type=attributes.Schema.MAP
+            _('A map containing all headers for the container.')
         ),
     }
 
     default_client_name = 'swift'
 
     def physical_resource_name(self):
-        name = self.properties[self.NAME]
+        name = self.properties.get(self.NAME)
         if name:
             return name
 
@@ -128,10 +117,10 @@ class SwiftContainer(resource.Resource):
 
     @staticmethod
     def _build_meta_headers(obj_type, meta_props):
-        """Returns a new dict.
-
-        Each key of new dict is prepended with "X-Container-Meta-".
-        """
+        '''
+        Returns a new dict where each key is prepended with:
+        X-Container-Meta-
+        '''
         if meta_props is None:
             return {}
         return dict(
@@ -149,7 +138,7 @@ class SwiftContainer(resource.Resource):
             "account", self.properties[self.X_ACCOUNT_META])
 
         for key in (self.X_CONTAINER_READ, self.X_CONTAINER_WRITE):
-            if self.properties[key] is not None:
+            if self.properties.get(key) is not None:
                 container_headers[key] = self.properties[key]
 
         LOG.debug('SwiftContainer create container %(container)s with '
@@ -159,16 +148,16 @@ class SwiftContainer(resource.Resource):
                      'account_headers': account_headers,
                      'container_headers': container_headers})
 
-        self.client().put_container(container, container_headers)
+        self.swift().put_container(container, container_headers)
 
         if account_headers:
-            self.client().post_account(account_headers)
+            self.swift().post_account(account_headers)
 
         self.resource_id_set(container)
 
     def _get_objects(self):
         try:
-            container, objects = self.client().get_container(self.resource_id)
+            container, objects = self.swift().get_container(self.resource_id)
         except Exception as ex:
             self.client_plugin().ignore_not_found(ex)
             return None
@@ -178,13 +167,14 @@ class SwiftContainer(resource.Resource):
         """Delete the underlying container or an object inside it."""
         args = [self.resource_id]
         if obj:
-            deleter = self.client().delete_object
+            deleter = self.swift().delete_object
             args.append(obj['name'])
         else:
-            deleter = self.client().delete_container
-
-        with self.client_plugin().ignore_not_found:
+            deleter = self.swift().delete_container
+        try:
             deleter(*args)
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
 
     def handle_delete(self):
         if self.resource_id is None:
@@ -201,15 +191,12 @@ class SwiftContainer(resource.Resource):
                             'id': self.resource_id,
                             'prop': self.PURGE_ON_DELETE}
                 raise exception.ResourceActionNotSupported(action=msg)
-        # objects is either None (container is gone already) or (empty) list
-        if objects is not None:
-            objects = len(objects)
         return objects
 
     def check_delete_complete(self, objects):
         if objects is None:  # resource was not created or is gone already
             return True
-        if objects:  # integer >=0 from the first invocation
+        if objects:  # an (empty) list from the first invocation
             objs = self._get_objects()
             if objs is None:
                 return True  # container is gone already
@@ -222,13 +209,13 @@ class SwiftContainer(resource.Resource):
         return True
 
     def handle_check(self):
-        self.client().get_container(self.resource_id)
+        self.swift().get_container(self.resource_id)
 
-    def get_reference_id(self):
+    def FnGetRefId(self):
         return six.text_type(self.resource_id)
 
     def _resolve_attribute(self, key):
-        parsed = list(urlparse.urlparse(self.client().url))
+        parsed = list(urlparse.urlparse(self.swift().url))
         if key == self.DOMAIN_NAME:
             return parsed[1].split(':')[0]
         elif key == self.WEBSITE_URL:
@@ -239,10 +226,10 @@ class SwiftContainer(resource.Resource):
         elif self.resource_id and key in (
                 self.OBJECT_COUNT, self.BYTES_USED, self.HEAD_CONTAINER):
             try:
-                headers = self.client().head_container(self.resource_id)
+                headers = self.swift().head_container(self.resource_id)
             except Exception as ex:
                 if self.client_plugin().is_client_exception(ex):
-                    LOG.warning(_LW("Head container failed: %s"), ex)
+                    LOG.warn(_LW("Head container failed: %s"), ex)
                     return None
                 raise
             else:
@@ -253,43 +240,6 @@ class SwiftContainer(resource.Resource):
                 elif key == self.HEAD_CONTAINER:
                     return headers
 
-    def _show_resource(self):
-        return self.client().head_container(self.resource_id)
-
-    def get_live_resource_data(self):
-        resource_data = super(SwiftContainer, self).get_live_resource_data()
-        account_data = self.client().head_account()
-        resource_data['account_data'] = account_data
-        return resource_data
-
-    def parse_live_resource_data(self, resource_properties, resource_data):
-        swift_reality = {}
-        # swift container name can't be updated
-        swift_reality.update({self.NAME: resource_properties.get(self.NAME)})
-        # PURGE_ON_DELETE property is used only on the heat side and isn't
-        # passed to swift, so update it from existing resource properties
-        swift_reality.update({self.PURGE_ON_DELETE: resource_properties.get(
-            self.PURGE_ON_DELETE)})
-        swift_reality.update({self.X_CONTAINER_META: {}})
-        swift_reality.update({self.X_ACCOUNT_META: {}})
-        for key in [self.X_CONTAINER_READ, self.X_CONTAINER_WRITE]:
-            swift_reality.update({key: resource_data.get(key.lower())})
-        for key in resource_properties.get(self.X_CONTAINER_META):
-            prefixed_key = "%(prefix)s-%(key)s" % {
-                'prefix': self.X_CONTAINER_META.lower(),
-                'key': key.lower()}
-            if prefixed_key in resource_data:
-                swift_reality[self.X_CONTAINER_META].update(
-                    {key: resource_data[prefixed_key]})
-        for key in resource_properties.get(self.X_ACCOUNT_META):
-            prefixed_key = "%(prefix)s-%(key)s" % {
-                'prefix': self.X_ACCOUNT_META.lower(),
-                'key': key.lower()}
-            if prefixed_key in resource_data['account_data']:
-                swift_reality[self.X_ACCOUNT_META].update(
-                    {key: resource_data['account_data'][prefixed_key]})
-        return swift_reality
-
 
 def resource_mapping():
     return {
