@@ -11,7 +11,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import copy
 import hashlib
 import json
 
@@ -80,12 +79,28 @@ class StackResource(resource.Resource):
 
     def _needs_update(self, after, before, after_props, before_props,
                       prev_resource):
-        # Always issue an update to the nested stack and let the individual
+        # Issue an update to the nested stack if the stack resource
+        # is able to update. If return true, let the individual
         # resources in it decide if they need updating.
+
+        # FIXME (ricolin): seems currently can not call super here
+        if self.nested() is None and (
+                self.status == self.FAILED
+                or (self.action == self.INIT
+                    and self.status == self.COMPLETE)):
+            raise resource.UpdateReplace(self)
+
         return True
 
+    def has_nested(self):
+        if self.nested() is not None:
+            return True
+
+        return False
+
     def nested(self, force_reload=False, show_deleted=False):
         '''Return a Stack object representing the nested (child) stack.
+        if we catch NotFound exception when loading, return None.
 
         :param force_reload: Forces reloading from the DB instead of returning
                              the locally cached Stack object
@@ -95,13 +110,13 @@ class StackResource(resource.Resource):
             self._nested = None
 
         if self._nested is None and self.resource_id is not None:
-            self._nested = parser.Stack.load(self.context,
-                                             self.resource_id,
-                                             show_deleted=show_deleted,
-                                             force_reload=force_reload)
-
-            if self._nested is None:
-                raise exception.NotFound(_("Nested stack not found in DB"))
+            try:
+                self._nested = parser.Stack.load(self.context,
+                                                 self.resource_id,
+                                                 show_deleted=show_deleted,
+                                                 force_reload=force_reload)
+            except exception.NotFound:
+                return None
 
         return self._nested
 
@@ -205,8 +220,11 @@ class StackResource(resource.Resource):
         return parsed_template
 
     def _validate_nested_resources(self, templ):
+        if cfg.CONF.max_resources_per_stack == -1:
+            return
+        root_stack_id = self.stack.root_stack_id()
         total_resources = (len(templ[templ.RESOURCES]) +
-                           self.stack.root_stack.total_resources())
+                           self.stack.total_resources(root_stack_id))
 
         if self.nested():
             # It's an update and these resources will be deleted
@@ -266,41 +284,30 @@ class StackResource(resource.Resource):
         self.resource_id_set(result['stack_id'])
 
     def raise_local_exception(self, ex):
-        ex_type = ex.__class__.__name__
-
-        is_remote = ex_type.endswith('_Remote')
-        if is_remote:
-            ex_type = ex_type[:-len('_Remote')]
+        if not ex.__class__.__name__.endswith('_Remote'):
+            raise ex
 
         full_message = six.text_type(ex)
-        if full_message.find('\n') > -1 and is_remote:
+        if full_message.find('\n') > -1:
             message, msg_trace = full_message.split('\n', 1)
         else:
             message = full_message
 
-        if isinstance(ex, exception.HeatException):
-            message = ex.message
-        local_ex = copy.copy(getattr(exception, ex_type))
-        local_ex.msg_fmt = "%(message)s"
-        raise local_ex(message=message)
+        raise exception.ResourceFailure(message, self, action=self.action)
 
     def check_create_complete(self, cookie=None):
         return self._check_status_complete(resource.Resource.CREATE)
 
     def _check_status_complete(self, action, show_deleted=False,
                                cookie=None):
-        try:
-            nested = self.nested(force_reload=True, show_deleted=show_deleted)
-        except exception.NotFound:
+        nested = self.nested(force_reload=True, show_deleted=show_deleted)
+        if nested is None:
             if action == resource.Resource.DELETE:
                 return True
             # It's possible the engine handling the create hasn't persisted
             # the stack to the DB when we first start polling for state
             return False
 
-        if nested is None:
-            return True
-
         if nested.action != action:
             return False
 
@@ -406,10 +413,11 @@ class StackResource(resource.Resource):
         '''
         Delete the nested stack.
         '''
-        stack_identity = identifier.HeatIdentifier(
-            self.context.tenant_id,
-            self.physical_resource_name(),
-            self.resource_id)
+        stack = self.nested()
+        if stack is None:
+            return
+
+        stack_identity = stack.identifier()
 
         try:
             self.rpc_client().delete_stack(self.context, stack_identity)
@@ -464,7 +472,11 @@ class StackResource(resource.Resource):
         return self._check_status_complete(resource.Resource.CHECK)
 
     def prepare_abandon(self):
-        return self.nested().prepare_abandon()
+        nested_stack = self.nested()
+        if nested_stack:
+            return self.nested().prepare_abandon()
+
+        return {}
 
     def get_output(self, op):
         '''
