@@ -14,21 +14,24 @@
 import functools
 import itertools
 import sys
-from time import time as wallclock
+import time
 import types
 
 import eventlet
-from oslo.utils import excutils
+from oslo_log import log as logging
+from oslo_utils import encodeutils
+from oslo_utils import excutils
 import six
 
 from heat.common.i18n import _
-from heat.openstack.common import log as logging
+from heat.common.i18n import _LI
 
 LOG = logging.getLogger(__name__)
 
 
 # Whether TaskRunner._sleep actually does an eventlet sleep when called.
 ENABLE_SLEEP = True
+wallclock = time.time
 
 
 def task_description(task):
@@ -42,7 +45,7 @@ def task_description(task):
             return '%s from %s' % (name, task.__self__)
     elif isinstance(task, types.FunctionType):
         if name is not None:
-            return str(name)
+            return six.text_type(name)
     return repr(task)
 
 
@@ -61,7 +64,7 @@ class Timeout(BaseException):
         """
         Initialise with the TaskRunner and a timeout period in seconds.
         """
-        message = _('%s Timed out') % str(task_runner)
+        message = _('%s Timed out') % six.text_type(task_runner)
         super(Timeout, self).__init__(message)
 
         # Note that we don't attempt to handle leap seconds or large clock
@@ -119,7 +122,7 @@ class ExceptionGroup(Exception):
                         for ex in self.exceptions]).encode('utf-8')
 
     def __unicode__(self):
-        return unicode(map(unicode, self.exceptions))
+        return six.text_type(map(six.text_type, self.exceptions))
 
 
 class TaskRunner(object):
@@ -147,25 +150,32 @@ class TaskRunner(object):
 
     def __str__(self):
         """Return a human-readable string representation of the task."""
-        return 'Task %s' % self.name
+        text = 'Task %s' % self.name
+        return encodeutils.safe_encode(text)
+
+    def __unicode__(self):
+        """Return a human-readable string representation of the task."""
+        text = 'Task %s' % self.name
+        return encodeutils.safe_decode(text)
 
     def _sleep(self, wait_time):
         """Sleep for the specified number of seconds."""
         if ENABLE_SLEEP and wait_time is not None:
-            LOG.debug('%s sleeping' % str(self))
+            LOG.debug('%s sleeping' % six.text_type(self))
             eventlet.sleep(wait_time)
 
     def __call__(self, wait_time=1, timeout=None):
         """
         Start and run the task to completion.
 
-        The task will sleep for `wait_time` seconds between steps. To avoid
-        sleeping, pass `None` for `wait_time`.
+        The task will first sleep for zero seconds, then sleep for `wait_time`
+        seconds between steps. To avoid sleeping, pass `None` for `wait_time`.
         """
         self.start(timeout=timeout)
-        # ensure that wait is applied only if task has not completed.
-        if not self.done():
-            self._sleep(wait_time)
+        # ensure that zero second sleep is applied only if task
+        # has not completed.
+        if not self.done() and wait_time:
+            self._sleep(0)
         self.run_to_completion(wait_time=wait_time)
 
     def start(self, timeout=None):
@@ -179,7 +189,7 @@ class TaskRunner(object):
         assert self._runner is None, "Task already started"
         assert not self._done, "Task already cancelled"
 
-        LOG.debug('%s starting' % str(self))
+        LOG.debug('%s starting' % six.text_type(self))
 
         if timeout is not None:
             self._timeout = Timeout(self, timeout)
@@ -191,7 +201,7 @@ class TaskRunner(object):
         else:
             self._runner = False
             self._done = True
-            LOG.debug('%s done (not resumable)' % str(self))
+            LOG.debug('%s done (not resumable)' % six.text_type(self))
 
     def step(self):
         """
@@ -202,18 +212,18 @@ class TaskRunner(object):
             assert self._runner is not None, "Task not started"
 
             if self._timeout is not None and self._timeout.expired():
-                LOG.info(_('%s timed out') % str(self))
+                LOG.info(_LI('%s timed out'), six.text_type(self))
                 self._done = True
 
                 self._timeout.trigger(self._runner)
             else:
-                LOG.debug('%s running' % str(self))
+                LOG.debug('%s running' % six.text_type(self))
 
                 try:
                     next(self._runner)
                 except StopIteration:
                     self._done = True
-                    LOG.debug('%s complete' % str(self))
+                    LOG.debug('%s complete' % six.text_type(self))
 
         return self._done
 
@@ -233,7 +243,7 @@ class TaskRunner(object):
             return
 
         if not self.started() or grace_period is None:
-            LOG.debug('%s cancelled' % str(self))
+            LOG.debug('%s cancelled' % six.text_type(self))
             self._done = True
             if self.started():
                 self._runner.close()
@@ -272,7 +282,7 @@ def wrappertask(task):
             self.cleanup()
     """
 
-    @functools.wraps(task)
+    @six.wraps(task)
     def wrapper(*args, **kwargs):
         parent = task(*args, **kwargs)
 
@@ -350,12 +360,13 @@ class DependencyTaskGroup(object):
         if name is None:
             name = '(%s) %s' % (getattr(task, '__name__',
                                         task_description(task)),
-                                str(dependencies))
+                                six.text_type(dependencies))
         self.name = name
 
     def __repr__(self):
         """Return a string representation of the task."""
-        return '%s(%s)' % (type(self).__name__, self.name)
+        text = '%s(%s)' % (type(self).__name__, self.name)
+        return encodeutils.safe_encode(text)
 
     def __call__(self):
         """Return a co-routine which runs the task group."""
@@ -462,13 +473,14 @@ class PollingTaskGroup(object):
         Each argument to use should be passed as a list (or iterable) of values
         such that one is passed in the corresponding position for each subtask.
         The number of subtasks spawned depends on the length of the argument
-        lists. For example:
+        lists.
+        For example::
 
             PollingTaskGroup.from_task_with_args(my_task,
                                                  [1, 2, 3],
                                                  alpha=['a', 'b', 'c'])
 
-        will start three TaskRunners that will run:
+        will start three TaskRunners that will run::
 
             my_task(1, alpha='a')
             my_task(2, alpha='b')
@@ -496,7 +508,8 @@ class PollingTaskGroup(object):
 
     def __repr__(self):
         """Return a string representation of the task group."""
-        return '%s(%s)' % (type(self).__name__, self.name)
+        text = '%s(%s)' % (type(self).__name__, self.name)
+        return encodeutils.safe_encode(text)
 
     def __call__(self):
         """Return a co-routine which runs the task group."""
