@@ -535,8 +535,11 @@ class EngineService(service.Service):
                         " Please delete some stacks.") % tenant_limit
             raise exception.RequestLimitExceeded(message=message)
 
+        max_resources = cfg.CONF.max_resources_per_stack
+        if max_resources == -1:
+            return
         num_resources = len(parsed_template[parsed_template.RESOURCES])
-        if num_resources > cfg.CONF.max_resources_per_stack:
+        if num_resources > max_resources:
             message = exception.StackResourceLimitExceeded.msg_fmt
             raise exception.RequestLimitExceeded(message=message)
 
@@ -713,7 +716,8 @@ class EngineService(service.Service):
                 current_stack.env,
                 args.get(rpc_api.PARAM_CLEAR_PARAMETERS, []))
         tmpl = templatem.Template(template, files=files, env=env)
-        if len(tmpl[tmpl.RESOURCES]) > cfg.CONF.max_resources_per_stack:
+        max_resources = cfg.CONF.max_resources_per_stack
+        if max_resources != -1 and len(tmpl[tmpl.RESOURCES]) > max_resources:
             raise exception.RequestLimitExceeded(
                 message=exception.StackResourceLimitExceeded.msg_fmt)
         stack_name = current_stack.name
@@ -1122,6 +1126,9 @@ class EngineService(service.Service):
             LOG.debug("signaling resource %s:%s" % (stack.name, rsrc.name))
             rsrc.signal(details)
 
+            if not rsrc.signal_needs_metadata_updates:
+                return
+
             # Refresh the metadata for all other resources, since signals can
             # update metadata which is used by other resources, e.g
             # when signalling a WaitConditionHandle resource, and other
@@ -1566,12 +1573,24 @@ class EngineService(service.Service):
                 LOG.info(_LI('Service %s was aborted'), service_ref['id'])
                 service_objects.Service.delete(cnxt, service_ref['id'])
 
+    def set_stack_and_resource_to_failed(self, stack):
+        reason = ('Engine went down during stack %s' % stack.action)
+        stack.state_set(stack.action, stack.FAILED, six.text_type(reason))
+        for name, rsrc in six.iteritems(stack.resources):
+            if rsrc.status == rsrc.IN_PROGRESS:
+                status_reason = ('Engine went down '
+                                 'during resource %s' % rsrc.action)
+                rsrc.state_set(rsrc.action,
+                               rsrc.FAILED,
+                               six.text_type(status_reason))
+
     def reset_stack_status(self):
         cnxt = context.get_admin_context()
         filters = {'status': parser.Stack.IN_PROGRESS}
         stacks = stack_object.Stack.get_all(cnxt,
                                             filters=filters,
-                                            tenant_safe=False) or []
+                                            tenant_safe=False,
+                                            show_nested=True) or []
         for s in stacks:
             stk = parser.Stack.load(cnxt, stack=s,
                                     use_stored_context=True)
@@ -1589,9 +1608,8 @@ class EngineService(service.Service):
                          ' was in action %(action)s'),
                      {'engine': engine_id, 'action': stk.action,
                       'stack_id': stk.id})
-            # Set stack status to FAILED.
-            status_reason = ('Engine went down during stack %s' % stk.action)
+
+            # Set stack and resources status to FAILED in sub thread
             self.thread_group_mgr.start_with_acquired_lock(
-                stk, lock, stk.state_set, stk.action,
-                stk.FAILED, six.text_type(status_reason)
+                stk, lock, self.set_stack_and_resource_to_failed, stk
             )
