@@ -15,8 +15,9 @@ from oslo_log import log as logging
 from oslo_utils import timeutils
 
 from heat.common import context
+from heat.common.i18n import _LE
+from heat.common.i18n import _LW
 from heat.engine import stack
-from heat.engine import stk_defn
 from heat.engine import watchrule
 from heat.objects import stack as stack_object
 from heat.objects import watch_rule as watch_rule_object
@@ -60,14 +61,18 @@ class StackWatch(object):
                 sid=stack_id)
 
     def check_stack_watches(self, sid):
-        # Use admin_context for stack_get to defeat tenant
+        # Retrieve the stored credentials & create context
+        # Require tenant_safe=False to the stack_get to defeat tenant
         # scoping otherwise we fail to retrieve the stack
-        LOG.debug("Periodic watcher task for stack %s", sid)
+        LOG.debug("Periodic watcher task for stack %s" % sid)
         admin_context = context.get_admin_context()
         db_stack = stack_object.Stack.get_by_id(admin_context,
-                                                sid)
+                                                sid,
+                                                tenant_safe=False,
+                                                eager_load=True)
         if not db_stack:
-            LOG.error("Unable to retrieve stack %s for periodic task", sid)
+            LOG.error(_LE("Unable to retrieve stack %s for periodic task"),
+                      sid)
             return
         stk = stack.Stack.load(admin_context, stack=db_stack,
                                use_stored_context=True)
@@ -82,16 +87,15 @@ class StackWatch(object):
             wrs = watch_rule_object.WatchRule.get_all_by_stack(admin_context,
                                                                sid)
         except Exception as ex:
-            LOG.warning('periodic_task db error watch rule removed? %s', ex)
+            LOG.warn(_LW('periodic_task db error watch rule removed? %(ex)s'),
+                     ex)
             return
 
         def run_alarm_action(stk, actions, details):
             for action in actions:
                 action(details=details)
-            for res in stk._explicit_dependencies():
+            for res in stk.itervalues():
                 res.metadata_update()
-                stk_defn.update_resource_data(stk.defn, res.name,
-                                              res.node_data())
 
         for wr in wrs:
             rule = watchrule.WatchRule.load(stk.context, watch=wr)
@@ -101,9 +105,9 @@ class StackWatch(object):
                                             actions, rule.get_details())
 
     def periodic_watcher_task(self, sid):
-        """Evaluate all watch-rules defined for stack ID.
-
-        Periodic task, created for each stack, triggers watch-rule evaluation
-        for all rules defined for the stack sid = stack ID.
+        """
+        Periodic task, created for each stack, triggers watch-rule
+        evaluation for all rules defined for the stack
+        sid = stack ID
         """
         self.check_stack_watches(sid)
