@@ -25,6 +25,7 @@ from osprofiler import profiler
 import six
 
 from heat.common import context as common_context
+from heat.common import environment_format as env_fmt
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common.i18n import _LE
@@ -211,14 +212,10 @@ class Stack(collections.Mapping):
         for res in self.values():
             yield res
 
-            get_nested = getattr(res, 'nested', None)
-            if not callable(get_nested) or nested_depth == 0:
-                continue
-
-            nested_stack = get_nested()
-            if nested_stack is None:
+            if not res.has_nested() or nested_depth == 0:
                 continue
 
+            nested_stack = res.nested()
             for nested_res in nested_stack.iter_resources(nested_depth - 1):
                 yield nested_res
 
@@ -247,16 +244,7 @@ class Stack(collections.Mapping):
     def root_stack_id(self):
         if not self.owner_id:
             return self.id
-        return stack_object.Stack.get_root_id(self.context, self.id)
-
-    @property
-    def root_stack(self):
-        '''
-        Return the root stack if this is nested (otherwise return self).
-        '''
-        if (self.parent_resource and self.parent_resource.stack):
-            return self.parent_resource.stack.root_stack
-        return self
+        return stack_object.Stack.get_root_id(self.context, self.owner_id)
 
     def object_path_in_stack(self):
         '''
@@ -284,25 +272,14 @@ class Stack(collections.Mapping):
         return [(stckres.name if stckres else None,
                  stck.name if stck else None) for stckres, stck in opis]
 
-    def total_resources(self):
+    def total_resources(self, stack_id=None):
         '''
         Return the total number of resources in a stack, including nested
         stacks below.
         '''
-        def total_nested(res):
-            get_nested = getattr(res, 'nested', None)
-            if callable(get_nested):
-                try:
-                    nested_stack = get_nested()
-                except exception.NotFound:
-                    # when an delete is underway, a nested stack can
-                    # disapear at any moment.
-                    return 0
-                if nested_stack is not None:
-                    return nested_stack.total_resources()
-            return 0
-
-        return len(self) + sum(total_nested(res) for res in self.itervalues())
+        if not stack_id:
+            stack_id = self.id
+        return stack_object.Stack.count_total_resources(self.context, stack_id)
 
     def _set_param_stackid(self):
         '''
@@ -624,18 +601,20 @@ class Stack(collections.Mapping):
             if result:
                 raise exception.StackValidationFailed(message=result)
 
-        for val in self.outputs.values():
+        for key, val in self.outputs.items():
+            if not isinstance(val, collections.Mapping):
+                message = _('Outputs must contain Output. '
+                            'Found a [%s] instead') % type(val)
+                raise exception.StackValidationFailed(
+                    error='Output validation error',
+                    path=[self.t.OUTPUTS],
+                    message=message)
             try:
-                if not val or not val.get('Value'):
+                if not val or 'Value' not in val:
                     message = _('Each Output must contain '
                                 'a Value key.')
                     raise exception.StackValidationFailed(message=message)
                 function.validate(val.get('Value'))
-            except AttributeError:
-                message = _('Output validation error: '
-                            'Outputs must contain Output. '
-                            'Found a [%s] instead') % type(val)
-                raise exception.StackValidationFailed(message=message)
             except Exception as ex:
                 reason = _('Output validation error: '
                            '%s') % six.text_type(ex)
@@ -810,7 +789,7 @@ class Stack(collections.Mapping):
 
     def supports_check_action(self):
         def is_supported(stack, res):
-            if hasattr(res, 'nested'):
+            if res.has_nested():
                 return res.nested().supports_check_action()
             else:
                 return hasattr(res, 'handle_%s' % self.CHECK.lower())
@@ -921,6 +900,8 @@ class Stack(collections.Mapping):
                              **kwargs)
 
         backup_stack = self._backup_stack()
+        existing_params = environment.Environment({env_fmt.PARAMETERS:
+                                                  self.t.env.params})
         try:
             update_task = update.StackUpdate(
                 self, newstack, backup_stack,
@@ -993,6 +974,12 @@ class Stack(collections.Mapping):
         self.status = stack_status
         self.status_reason = reason
 
+        if self.status == self.FAILED:
+            # Since template was incrementally updated based on existing and
+            # new stack resources, we should have user params of both.
+            existing_params.load(newstack.t.env.user_env_as_dict())
+            self.t.env = existing_params
+            self.t.store(self.context)
         self.store()
         lifecycle_plugin_utils.do_post_ops(self.context, self,
                                            newstack, action,
@@ -1007,6 +994,11 @@ class Stack(collections.Mapping):
             return (child.action == child.CREATE and
                     child.status in (child.FAILED, child.IN_PROGRESS))
 
+        def copy_data(source_res, destination_res):
+            if source_res.data():
+                for key, val in six.iteritems(source_res.data()):
+                    destination_res.data_set(key, val)
+
         for key, backup_res in stack.resources.items():
             # If UpdateReplace is failed, we must restore backup_res
             # to existing_stack in case of it may have dependencies in
@@ -1030,8 +1022,11 @@ class Stack(collections.Mapping):
                     # needed to delete it.
                     self.resources[key].resource_id = backup_res_id
                     self.resources[key].properties = backup_res.properties
+                    copy_data(backup_res, self.resources[key])
+
                     stack.resources[key].resource_id = curr_res_id
                     stack.resources[key].properties = curr_res.properties
+                    copy_data(curr_res, stack.resources[key])
 
         stack.delete(backup=True)
 
@@ -1266,9 +1261,10 @@ class Stack(collections.Mapping):
                            "Can't restore snapshot from other stack")
             return
         self.updated_time = datetime.datetime.utcnow()
-
         env = environment.Environment(snapshot.data['environment'])
-        template = tmpl.Template(snapshot.data['template'], env=env)
+        files = snapshot.data['files']
+        template = tmpl.Template(snapshot.data['template'],
+                                 env=env, files=files)
         newstack = self.__class__(self.context, self.name, template,
                                   timeout_mins=self.timeout_mins,
                                   disable_rollback=self.disable_rollback)
@@ -1356,6 +1352,7 @@ class Stack(collections.Mapping):
             'id': self.id,
             'action': self.action,
             'environment': self.env.user_env_as_dict(),
+            'files': self.t.files,
             'status': self.status,
             'template': self.t.t,
             'resources': dict((res.name, res.prepare_abandon())
