@@ -11,14 +11,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
 import six
 
-from heat.common.i18n import _
-from heat.db import api as db_api
+from heat.common.i18n import _LI
 from heat.engine import dependencies
 from heat.engine import resource
 from heat.engine import scheduler
-from heat.openstack.common import log as logging
+from heat.objects import resource as resource_objects
 
 LOG = logging.getLogger(__name__)
 
@@ -84,8 +84,8 @@ class StackUpdate(object):
 
     @staticmethod
     def _exchange_stacks(existing_res, prev_res):
-        db_api.resource_exchange_stacks(existing_res.stack.context,
-                                        existing_res.id, prev_res.id)
+        resource_objects.Resource.exchange_stacks(existing_res.stack.context,
+                                                  existing_res.id, prev_res.id)
         prev_stack, existing_stack = prev_res.stack, existing_res.stack
         prev_stack.add_resource(existing_res)
         existing_stack.add_resource(prev_res)
@@ -119,13 +119,28 @@ class StackUpdate(object):
             existing_res.state_set(existing_res.UPDATE, existing_res.COMPLETE)
 
         self.existing_stack.add_resource(new_res)
+
+        # Save new resource definition to backup stack if it is not
+        # present in backup stack template already
+        # it allows to resolve all dependencies that existing resource
+        # can have if it was copied to backup stack
+        if (res_name not in
+                self.previous_stack.t[self.previous_stack.t.RESOURCES]):
+            LOG.debug("Backing up new Resource %s" % res_name)
+            definition = new_res.t.reparse(self.previous_stack,
+                                           new_res.stack.t)
+            self.previous_stack.t.add_resource(definition)
+            self.previous_stack.t.store(self.previous_stack.context)
+
         yield new_res.create()
 
     @scheduler.wrappertask
     def _process_new_resource_update(self, new_res):
         res_name = new_res.name
+        res_type = new_res.type()
 
-        if res_name in self.existing_stack:
+        if (res_name in self.existing_stack and
+                res_type == self.existing_stack[res_name].type()):
             existing_res = self.existing_stack[res_name]
             try:
                 yield self._update_in_place(existing_res,
@@ -133,10 +148,18 @@ class StackUpdate(object):
             except resource.UpdateReplace:
                 pass
             else:
-                LOG.info(_("Resource %(res_name)s for stack %(stack_name)s "
-                           "updated")
-                         % {'res_name': res_name,
-                            'stack_name': self.existing_stack.name})
+                # Save updated resource definition to backup stack
+                # cause it allows the backup stack resources to be synchronized
+                LOG.debug("Backing up updated Resource %s" % res_name)
+                definition = existing_res.t.reparse(self.previous_stack,
+                                                    existing_res.stack.t)
+                self.previous_stack.t.add_resource(definition)
+                self.previous_stack.t.store(self.previous_stack.context)
+
+                LOG.info(_LI("Resource %(res_name)s for stack %(stack_name)s "
+                             "updated"),
+                         {'res_name': res_name,
+                          'stack_name': self.existing_stack.name})
                 return
 
         yield self._create_resource(new_res)
