@@ -12,20 +12,19 @@
 # under the License.
 
 
-"""Service object."""
+"""
+Service object
+"""
 
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
-from heat.db.sqlalchemy import api as db_api
-from heat.objects import base as heat_base
+from heat.db import api as db_api
 
 
-class Service(
-        heat_base.HeatObject,
-        base.VersionedObjectDictCompat,
-        base.ComparableVersionedObject,
-):
+class Service(base.VersionedObject,
+              base.VersionedObjectDictCompat,
+              base.ComparableVersionedObject):
     fields = {
         'id': fields.StringField(),
         'engine_id': fields.StringField(),
@@ -49,8 +48,10 @@ class Service(
 
     @classmethod
     def _from_db_objects(cls, context, list_obj):
-        return [cls._from_db_object(context, cls(context), obj)
-                for obj in list_obj]
+        return map(lambda obj: cls._from_db_object(context,
+                                                   cls(context),
+                                                   obj),
+                   list_obj)
 
     @classmethod
     def get_by_id(cls, context, service_id):
@@ -74,7 +75,7 @@ class Service(
 
     @classmethod
     def delete(cls, context, service_id, soft_delete=True):
-        db_api.service_delete(context, service_id, soft_delete)
+        return db_api.service_delete(context, service_id, soft_delete)
 
     @classmethod
     def get_all(cls, context):
