@@ -20,6 +20,7 @@ import six
 
 from heat.common import exception
 from heat.common import template_format
+from heat.engine import resource
 from heat.engine import scheduler
 from heat.tests.autoscaling import inline_templates
 from heat.tests import common
@@ -63,7 +64,8 @@ class TestAutoScalingPolicy(common.HeatTestCase):
         test = {'current': 'not_an_alarm'}
         with mock.patch.object(pol, '_cooldown_inprogress',
                                side_effect=AssertionError()) as dont_call:
-            pol.handle_signal(details=test)
+            self.assertRaises(resource.NoActionRequired,
+                              pol.handle_signal, details=test)
             self.assertEqual([], dont_call.call_args_list)
 
     def test_scaling_policy_cooldown_toosoon(self):
@@ -77,7 +79,8 @@ class TestAutoScalingPolicy(common.HeatTestCase):
                                side_effect=AssertionError) as dont_call:
             with mock.patch.object(pol, '_cooldown_inprogress',
                                    return_value=True) as mock_cip:
-                pol.handle_signal(details=test)
+                self.assertRaises(resource.NoActionRequired,
+                                  pol.handle_signal, details=test)
                 mock_cip.assert_called_once_with()
             self.assertEqual([], dont_call.call_args_list)
 
@@ -109,23 +112,39 @@ class TestCooldownMixin(common.HeatTestCase):
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         return rsrc
 
-    def test_is_in_progress(self):
+    def test_cooldown_is_in_progress_toosoon(self):
         t = template_format.parse(as_template)
         stack = utils.parse_stack(t, params=as_params)
         pol = self.create_scaling_policy(t, stack, 'WebServerScaleUpPolicy')
 
         now = timeutils.utcnow()
-        previous_meta = {timeutils.strtime(now): 'ChangeInCapacity : 1'}
+        previous_meta = {'cooldown': {
+            now.isoformat(): 'ChangeInCapacity : 1'}}
         self.patchobject(pol, 'metadata_get', return_value=previous_meta)
         self.assertTrue(pol._cooldown_inprogress())
 
-    def test_not_in_progress(self):
+    def test_cooldown_is_in_progress_scaling_unfinished(self):
+        t = template_format.parse(as_template)
+        stack = utils.parse_stack(t, params=as_params)
+        pol = self.create_scaling_policy(t, stack, 'WebServerScaleUpPolicy')
+
+        previous_meta = {'scaling_in_progress': True}
+        self.patchobject(pol, 'metadata_get', return_value=previous_meta)
+        self.assertTrue(pol._cooldown_inprogress())
+
+    def test_cooldown_not_in_progress(self):
         t = template_format.parse(as_template)
         stack = utils.parse_stack(t, params=as_params)
         pol = self.create_scaling_policy(t, stack, 'WebServerScaleUpPolicy')
 
         awhile_ago = timeutils.utcnow() - datetime.timedelta(seconds=100)
-        previous_meta = {timeutils.strtime(awhile_ago): 'ChangeInCapacity : 1'}
+
+        previous_meta = {
+            'cooldown': {
+                awhile_ago.isoformat(): 'ChangeInCapacity : 1'
+            },
+            'scaling_in_progress': False
+        }
         self.patchobject(pol, 'metadata_get', return_value=previous_meta)
         self.assertFalse(pol._cooldown_inprogress())
 
@@ -165,12 +184,14 @@ class TestCooldownMixin(common.HeatTestCase):
         stack = utils.parse_stack(t, params=as_params)
         pol = self.create_scaling_policy(t, stack, 'WebServerScaleUpPolicy')
 
-        nowish = timeutils.strtime()
+        nowish = timeutils.utcnow()
         reason = 'cool as'
         meta_set = self.patchobject(pol, 'metadata_set')
-        self.patchobject(timeutils, 'strtime', return_value=nowish)
+        self.patchobject(timeutils, 'utcnow', return_value=nowish)
         pol._cooldown_timestamp(reason)
-        meta_set.assert_called_once_with({nowish: reason})
+        meta_set.assert_called_once_with(
+            {'cooldown': {nowish.isoformat(): reason},
+             'scaling_in_progress': False})
 
 
 class ScalingPolicyAttrTest(common.HeatTestCase):
