@@ -16,22 +16,18 @@ import datetime
 import json
 import uuid
 
-import mock
+from oslo_config import cfg
 from oslo_utils import timeutils
 import six
-from six.moves.urllib import parse
 
 from heat.common import exception
 from heat.common import identifier
 from heat.common import template_format
 from heat.engine import environment
-from heat.engine import node_data
+from heat.engine import parser
 from heat.engine.resources.aws.cfn import wait_condition_handle as aws_wch
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.engine import stack as parser
-from heat.engine import stk_defn
-from heat.engine import template as tmpl
 from heat.objects import resource as resource_objects
 from heat.tests import common
 from heat.tests import utils
@@ -80,13 +76,18 @@ test_template_wc_count = '''
 
 class WaitConditionTest(common.HeatTestCase):
 
+    def setUp(self):
+        super(WaitConditionTest, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+
     def create_stack(self, stack_id=None,
                      template=test_template_waitcondition, params=None,
                      stub=True, stub_status=True):
         params = params or {}
         temp = template_format.parse(template)
-        template = tmpl.Template(temp,
-                                 env=environment.Environment(params))
+        template = parser.Template(temp,
+                                   env=environment.Environment(params))
         ctx = utils.dummy_context(tenant_id='test_tenant')
         stack = parser.Stack(ctx, 'test_stack', template,
                              disable_rollback=True)
@@ -127,7 +128,7 @@ class WaitConditionTest(common.HeatTestCase):
                          rsrc.state)
 
         r = resource_objects.Resource.get_by_name_and_stack(
-            self.stack.context, 'WaitHandle', self.stack.id)
+            None, 'WaitHandle', self.stack.id)
         self.assertEqual('WaitHandle', r.name)
         self.m.VerifyAll()
 
@@ -147,7 +148,7 @@ class WaitConditionTest(common.HeatTestCase):
         self.assertTrue(reason.startswith('WaitConditionFailure:'))
 
         r = resource_objects.Resource.get_by_name_and_stack(
-            self.stack.context, 'WaitHandle', self.stack.id)
+            None, 'WaitHandle', self.stack.id)
         self.assertEqual('WaitHandle', r.name)
         self.m.VerifyAll()
 
@@ -170,7 +171,7 @@ class WaitConditionTest(common.HeatTestCase):
                          rsrc.state)
 
         r = resource_objects.Resource.get_by_name_and_stack(
-            self.stack.context, 'WaitHandle', self.stack.id)
+            None, 'WaitHandle', self.stack.id)
         self.assertEqual('WaitHandle', r.name)
         self.m.VerifyAll()
 
@@ -191,7 +192,7 @@ class WaitConditionTest(common.HeatTestCase):
         self.assertTrue(reason.startswith('WaitConditionFailure:'))
 
         r = resource_objects.Resource.get_by_name_and_stack(
-            self.stack.context, 'WaitHandle', self.stack.id)
+            None, 'WaitHandle', self.stack.id)
         self.assertEqual('WaitHandle', r.name)
         self.m.VerifyAll()
 
@@ -203,9 +204,8 @@ class WaitConditionTest(common.HeatTestCase):
         self.stack.timeout_secs().MultipleTimes().AndReturn(None)
 
         now = timeutils.utcnow()
-        periods = [0, 0.001, 0.1, 4.1, 5.1]
-        periods.extend(range(10, 100, 5))
-        fake_clock = [now + datetime.timedelta(0, t) for t in periods]
+        fake_clock = [now + datetime.timedelta(0, t)
+                      for t in (0, 0.001, 0.1, 4.1, 5.1)]
         timeutils.set_time_override(fake_clock)
         self.addCleanup(timeutils.clear_time_override)
 
@@ -251,40 +251,10 @@ class WaitConditionTest(common.HeatTestCase):
                          'Status': 'SUCCESS', 'UniqueId': '456'}
         ret = handle.handle_signal(test_metadata)
         wc_att = rsrc.FnGetAtt('Data')
-        self.assertIsInstance(wc_att, six.string_types)
-        self.assertEqual({"123": "foo", "456": "dog"}, json.loads(wc_att))
+        self.assertEqual(u'{"123": "foo", "456": "dog"}', wc_att)
         self.assertEqual('status:SUCCESS reason:cat', ret)
         self.m.VerifyAll()
 
-    def test_FnGetRefId_resource_name(self):
-        self.stack = self.create_stack()
-        rsrc = self.stack['WaitHandle']
-        self.assertEqual('WaitHandle', rsrc.FnGetRefId())
-
-    @mock.patch.object(aws_wch.WaitConditionHandle, '_get_ec2_signed_url')
-    def test_FnGetRefId_signed_url(self, mock_get_signed_url):
-        self.stack = self.create_stack()
-        rsrc = self.stack['WaitHandle']
-        rsrc.resource_id = '123'
-        mock_get_signed_url.return_value = 'http://signed_url'
-        self.assertEqual('http://signed_url', rsrc.FnGetRefId())
-
-    def test_FnGetRefId_convergence_cache_data(self):
-        t = template_format.parse(test_template_waitcondition)
-        template = tmpl.Template(t)
-        stack = parser.Stack(utils.dummy_context(), 'test', template,
-                             cache_data={
-                                 'WaitHandle': node_data.NodeData.from_dict({
-                                     'uuid': mock.ANY,
-                                     'id': mock.ANY,
-                                     'action': 'CREATE',
-                                     'status': 'COMPLETE',
-                                     'reference_id': 'http://convg_signed_url'
-                                 })})
-
-        rsrc = stack.defn['WaitHandle']
-        self.assertEqual('http://convg_signed_url', rsrc.FnGetRefId())
-
     def test_validate_handle_url_bad_stackid(self):
         self.m.ReplayAll()
 
@@ -378,9 +348,14 @@ class WaitConditionTest(common.HeatTestCase):
 
 
 class WaitConditionHandleTest(common.HeatTestCase):
+    def setUp(self):
+        super(WaitConditionHandleTest, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+
     def create_stack(self, stack_name=None, stack_id=None):
         temp = template_format.parse(test_template_waitcondition)
-        template = tmpl.Template(temp)
+        template = parser.Template(temp)
         ctx = utils.dummy_context(tenant_id='test_tenant')
         if stack_name is None:
             stack_name = utils.random_name()
@@ -402,8 +377,10 @@ class WaitConditionHandleTest(common.HeatTestCase):
                                            stack.id, '', 'WaitHandle')
         self.m.StubOutWithMock(aws_wch.WaitConditionHandle, 'identifier')
         aws_wch.WaitConditionHandle.identifier().MultipleTimes().AndReturn(id)
+
         self.m.ReplayAll()
         stack.create()
+
         return stack
 
     def test_handle(self):
@@ -413,26 +390,18 @@ class WaitConditionHandleTest(common.HeatTestCase):
         self.stack = self.create_stack(stack_id=stack_id,
                                        stack_name=stack_name)
 
-        self.m.StubOutWithMock(self.stack.clients.client_plugin('heat'),
-                               'get_heat_cfn_url')
-        self.stack.clients.client_plugin('heat').get_heat_cfn_url().AndReturn(
-            'http://server.test:8000/v1')
-        self.m.ReplayAll()
         rsrc = self.stack['WaitHandle']
-        self.assertEqual(rsrc.resource_id, rsrc.data().get('user_id'))
         # clear the url
         rsrc.data_set('ec2_signed_url', None, False)
 
         rsrc.created_time = created_time
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
-        connection_url = "".join([
+
+        expected_url = "".join([
             'http://server.test:8000/v1/waitcondition/',
             'arn%3Aopenstack%3Aheat%3A%3Atest_tenant%3Astacks%2F',
             'test_stack2%2F', stack_id, '%2Fresources%2F',
-            'WaitHandle?'])
-
-        expected_url = "".join([
-            connection_url,
+            'WaitHandle?',
             'Timestamp=2012-11-29T13%3A49%3A37Z&',
             'SignatureMethod=HmacSHA256&',
             'AWSAccessKeyId=4567&',
@@ -440,19 +409,14 @@ class WaitConditionHandleTest(common.HeatTestCase):
             'Signature=',
             'fHyt3XFnHq8%2FSwYaVcHdJka1hz6jdK5mHtgbo8OOKbQ%3D'])
 
-        actual_url = rsrc.FnGetRefId()
-        expected_params = parse.parse_qs(expected_url.split("?", 1)[1])
-        actual_params = parse.parse_qs(actual_url.split("?", 1)[1])
-        self.assertEqual(expected_params, actual_params)
-        self.assertTrue(connection_url.startswith(connection_url))
-
+        self.assertEqual(six.text_type(expected_url), rsrc.FnGetRefId())
         self.m.VerifyAll()
 
     def test_handle_signal(self):
         self.stack = self.create_stack()
         rsrc = self.stack['WaitHandle']
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
-        self.assertEqual(rsrc.resource_id, rsrc.data().get('user_id'))
+
         test_metadata = {'Data': 'foo', 'Reason': 'bar',
                          'Status': 'SUCCESS', 'UniqueId': '123'}
         rsrc.handle_signal(test_metadata)
@@ -466,7 +430,7 @@ class WaitConditionHandleTest(common.HeatTestCase):
         self.stack = self.create_stack()
         rsrc = self.stack['WaitHandle']
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
-        self.assertEqual(rsrc.resource_id, rsrc.data().get('user_id'))
+
         # handle_signal should raise a ValueError if the metadata
         # is missing any of the expected keys
         err_metadata = {'Data': 'foo', 'Status': 'SUCCESS', 'UniqueId': '123'}
@@ -509,7 +473,7 @@ class WaitConditionHandleTest(common.HeatTestCase):
         self.stack = self.create_stack()
         rsrc = self.stack['WaitHandle']
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
-        self.assertEqual(rsrc.resource_id, rsrc.data().get('user_id'))
+
         # UnsetStubs, don't want get_status stubbed anymore..
         self.m.VerifyAll()
         self.m.UnsetStubs()
@@ -542,8 +506,7 @@ class WaitConditionHandleTest(common.HeatTestCase):
         test_metadata = {'Data': 'dog', 'Reason': 'cat',
                          'Status': 'SUCCESS', 'UniqueId': '456'}
         ret = rsrc.handle_signal(test_metadata)
-        self.assertEqual(
-            ['bar', 'cat'], sorted(rsrc.get_status_reason('SUCCESS')))
+        self.assertEqual(['bar', 'cat'], rsrc.get_status_reason('SUCCESS'))
         self.assertEqual('status:SUCCESS reason:cat', ret)
 
         test_metadata = {'Data': 'boo', 'Reason': 'hoo',
@@ -555,11 +518,16 @@ class WaitConditionHandleTest(common.HeatTestCase):
 
 
 class WaitConditionUpdateTest(common.HeatTestCase):
-    def create_stack(self, temp=None):
-        if temp is None:
-            temp = test_template_wc_count
-        temp_fmt = template_format.parse(temp)
-        template = tmpl.Template(temp_fmt)
+    def setUp(self):
+        super(WaitConditionUpdateTest, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+
+    def create_stack(self, tmpl=None):
+        if tmpl is None:
+            tmpl = test_template_wc_count
+        temp = template_format.parse(tmpl)
+        template = parser.Template(temp)
         ctx = utils.dummy_context(tenant_id='test_tenant')
         stack = parser.Stack(ctx, 'test_stack', template,
                              disable_rollback=True)
@@ -642,8 +610,6 @@ class WaitConditionUpdateTest(common.HeatTestCase):
                                                       rsrc.type(),
                                                       uprops)
 
-        stk_defn.update_resource_data(self.stack.defn, 'WaitHandle',
-                                      self.stack['WaitHandle'].node_data())
         updater = scheduler.TaskRunner(rsrc.update, update_snippet)
         updater()
 
