@@ -30,37 +30,49 @@ from heat.tests import utils
 
 
 gw_template_deprecated = '''
-heat_template_version: 2015-04-30
-description: Template to test Network Gateway resource
-resources:
-  NetworkGateway:
-    type: OS::Neutron::NetworkGateway
-    properties:
-      name: NetworkGateway
-      devices:
-        - id: e52148ca-7db9-4ec3-abe6-2c7c0ff316eb
-          interface_name: breth1
-      connections:
-        - network_id: 6af055d3-26f6-48dd-a597-7611d7e58d35
-          segmentation_type: vlan
-          segmentation_id: 10
+{
+  'AWSTemplateFormatVersion': '2010-09-09',
+  'Description': 'Template to test Network Gateway resource',
+  'Parameters': {},
+  'Resources': {
+    'NetworkGateway': {
+      'Type': 'OS::Neutron::NetworkGateway',
+      'Properties': {
+        'name': 'NetworkGateway',
+        'devices': [{
+          'id': 'e52148ca-7db9-4ec3-abe6-2c7c0ff316eb',
+          'interface_name': 'breth1'}],
+        'connections': [{
+          'network_id': '6af055d3-26f6-48dd-a597-7611d7e58d35',
+          'segmentation_type': 'vlan',
+          'segmentation_id': 10}]
+      }
+    }
+  }
+}
 '''
 
 gw_template = '''
-heat_template_version: 2015-04-30
-description: Template to test Network Gateway resource
-resources:
-  NetworkGateway:
-    type: OS::Neutron::NetworkGateway
-    properties:
-      name: NetworkGateway
-      devices:
-        - id: e52148ca-7db9-4ec3-abe6-2c7c0ff316eb
-          interface_name: breth1
-      connections:
-        - network: 6af055d3-26f6-48dd-a597-7611d7e58d35
-          segmentation_type: vlan
-          segmentation_id: 10
+{
+  'AWSTemplateFormatVersion': '2010-09-09',
+  'Description': 'Template to test Network Gateway resource',
+  'Parameters': {},
+  'Resources': {
+    'NetworkGateway': {
+      'Type': 'OS::Neutron::NetworkGateway',
+      'Properties': {
+        'name': 'NetworkGateway',
+        'devices': [{
+          'id': 'e52148ca-7db9-4ec3-abe6-2c7c0ff316eb',
+          'interface_name': 'breth1'}],
+        'connections': [{
+          'network': '6af055d3-26f6-48dd-a597-7611d7e58d35',
+          'segmentation_type': 'vlan',
+          'segmentation_id': 10}]
+      }
+    }
+  }
+}
 '''
 
 sng = {
@@ -189,12 +201,13 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
 
     def _test_network_gateway_create(self, resolve_neutron=True):
         rsrc = self.prepare_create_network_gateway(resolve_neutron)
-        neutronV20.find_resourceid_by_name_or_id(
-            mox.IsA(neutronclient.Client),
-            'network',
-            '6af055d3-26f6-48dd-a597-7611d7e58d35'
-        ).MultipleTimes().AndReturn(
-            '6af055d3-26f6-48dd-a597-7611d7e58d35')
+        if resolve_neutron:
+            neutronV20.find_resourceid_by_name_or_id(
+                mox.IsA(neutronclient.Client),
+                'network',
+                '6af055d3-26f6-48dd-a597-7611d7e58d35'
+            ).MultipleTimes().AndReturn(
+                '6af055d3-26f6-48dd-a597-7611d7e58d35')
 
         neutronclient.Client.disconnect_network_gateway(
             'ed4c03b9-8251-4c09-acc4-e59ee9e6aa37', {
@@ -339,15 +352,15 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
         neutronclient.Client.disconnect_network_gateway(
             u'ed4c03b9-8251-4c09-acc4-e59ee9e6aa37', {
                 'network_id': u'6af055d3-26f6-48dd-a597-7611d7e58d35',
-                'segmentation_id': 0,
-                'segmentation_type': u'flat'
+                'segmentation_id': 10,
+                'segmentation_type': u'vlan'
             }
         ).AndRaise(qe.NeutronClientException(status_code=404))
 
         neutronclient.Client.connect_network_gateway(
             u'ed4c03b9-8251-4c09-acc4-e59ee9e6aa37', {
                 'network_id': u'6af055d3-26f6-48dd-a597-7611d7e58d35',
-                'segmentation_id': 1,
+                'segmentation_id': 0,
                 'segmentation_type': u'flat'
             }
         ).AndReturn({
@@ -361,8 +374,8 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
         neutronclient.Client.disconnect_network_gateway(
             u'ed4c03b9-8251-4c09-acc4-e59ee9e6aa37', {
                 'network_id': u'6af055d3-26f6-48dd-a597-7611d7e58d35',
-                'segmentation_id': 1,
-                'segmentation_type': u'flat'
+                'segmentation_id': 10,
+                'segmentation_type': u'vlan'
             }
         ).AndReturn(None)
 
@@ -372,7 +385,7 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
 
         neutronclient.Client.create_network_gateway({
             'network_gateway': {
-                'name': u'NetworkGatewayUpdate',
+                'name': u'NetworkGateway',
                 'devices': [{'id': u'e52148ca-7db9-4ec3-abe6-2c7c0ff316eb',
                              'interface_name': u'breth2'}]
             }
@@ -393,8 +406,8 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
         neutronclient.Client.connect_network_gateway(
             u'ed4c03b9-8251-4c09-acc4-e59ee9e6aa37', {
                 'network_id': u'6af055d3-26f6-48dd-a597-7611d7e58d35',
-                'segmentation_id': 1,
-                'segmentation_type': u'flat'
+                'segmentation_id': 10,
+                'segmentation_type': u'vlan'
             }
         ).AndReturn({
             'connection_info': {
@@ -411,7 +424,7 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
 
         # update name
-        snippet_for_update1 = rsrc_defn.ResourceDefinition(
+        snippet_for_update = rsrc_defn.ResourceDefinition(
             rsrc.name,
             rsrc.type(),
             {
@@ -424,14 +437,17 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
                     'segmentation_type': 'vlan',
                     'segmentation_id': 10}]
             })
-        scheduler.TaskRunner(rsrc.update, snippet_for_update1)()
+        prop_diff = {'name': u'NetworkGatewayUpdate'}
+        self.assertIsNone(rsrc.handle_update(snippet_for_update,
+                                             mox.IgnoreArg(),
+                                             prop_diff))
 
         # update connections
-        snippet_for_update2 = rsrc_defn.ResourceDefinition(
+        snippet_for_update = rsrc_defn.ResourceDefinition(
             rsrc.name,
             rsrc.type(),
             {
-                'name': u'NetworkGatewayUpdate',
+                'name': u'NetworkGateway',
                 'devices': [{
                     'id': u'e52148ca-7db9-4ec3-abe6-2c7c0ff316eb',
                     'interface_name': u'breth1'}],
@@ -440,32 +456,27 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
                     'segmentation_type': u'flat',
                     'segmentation_id': 0}]
             })
-        scheduler.TaskRunner(rsrc.update, snippet_for_update2,
-                             snippet_for_update1)()
+        prop_diff = {
+            'connections': [{
+                'network': u'6af055d3-26f6-48dd-a597-7611d7e58d35',
+                'segmentation_type': u'flat',
+                'segmentation_id': 0}]
+        }
+        self.assertIsNone(rsrc.handle_update(snippet_for_update,
+                                             mox.IgnoreArg(),
+                                             prop_diff))
 
         # update connections once more
-        snippet_for_update3 = rsrc_defn.ResourceDefinition(
-            rsrc.name,
-            rsrc.type(),
-            {
-                'name': u'NetworkGatewayUpdate',
-                'devices': [{
-                    'id': u'e52148ca-7db9-4ec3-abe6-2c7c0ff316eb',
-                    'interface_name': u'breth1'}],
-                'connections': [{
-                    'network': u'6af055d3-26f6-48dd-a597-7611d7e58d35',
-                    'segmentation_type': u'flat',
-                    'segmentation_id': 1}]
-            })
-        scheduler.TaskRunner(rsrc.update, snippet_for_update3,
-                             snippet_for_update2)()
+        self.assertIsNone(rsrc.handle_update(snippet_for_update,
+                                             mox.IgnoreArg(),
+                                             prop_diff))
 
         # update devices
-        snippet_for_update4 = rsrc_defn.ResourceDefinition(
+        snippet_for_update = rsrc_defn.ResourceDefinition(
             rsrc.name,
             rsrc.type(),
             {
-                'name': u'NetworkGatewayUpdate',
+                'name': u'NetworkGateway',
                 'devices': [{
                     'id': u'e52148ca-7db9-4ec3-abe6-2c7c0ff316eb',
                     'interface_name': u'breth2'}],
@@ -474,8 +485,14 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
                     'segmentation_type': u'vlan',
                     'segmentation_id': 10}]
             })
-        scheduler.TaskRunner(rsrc.update, snippet_for_update4,
-                             snippet_for_update3)()
+        prop_diff = {
+            'devices': [{
+                'id': u'e52148ca-7db9-4ec3-abe6-2c7c0ff316eb',
+                'interface_name': u'breth2'}]
+        }
+        self.assertIsNone(rsrc.handle_update(snippet_for_update,
+                                             mox.IgnoreArg(),
+                                             prop_diff))
 
         self.m.VerifyAll()
 
@@ -512,7 +529,7 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
 
     def test_gateway_validate_failed_with_vlan(self):
         t = template_format.parse(gw_template)
-        del t['resources']['NetworkGateway']['properties'][
+        del t['Resources']['NetworkGateway']['Properties'][
             'connections'][0]['segmentation_id']
         stack = utils.parse_stack(t)
         resource_defns = stack.t.resource_definitions(stack)
@@ -534,7 +551,7 @@ class NeutronNetworkGatewayTest(common.HeatTestCase):
 
     def test_gateway_validate_failed_with_flat(self):
         t = template_format.parse(gw_template)
-        t['resources']['NetworkGateway']['properties'][
+        t['Resources']['NetworkGateway']['Properties'][
             'connections'][0]['segmentation_type'] = 'flat'
         stack = utils.parse_stack(t)
         resource_defns = stack.t.resource_definitions(stack)
