@@ -27,84 +27,128 @@ from heat.tests import common
 from heat.tests import utils
 
 
-vpnservice_template = '''
-heat_template_version: 2015-04-30
-description: Template to test vpnservice Neutron resource
-resources:
-  VPNService:
-    type: OS::Neutron::VPNService
-    properties:
-      name: VPNService
-      description: My new VPN service
-      admin_state_up: true
-      router_id: rou123
-      subnet: sub123
+vpnservice_template_deprecated = '''
+{
+  "AWSTemplateFormatVersion" : "2010-09-09",
+  "Description" : "Template to test VPN service resource",
+  "Parameters" : {},
+  "Resources" : {
+    "VPNService" : {
+      "Type" : "OS::Neutron::VPNService",
+      "Properties" : {
+        "name" : "VPNService",
+        "description" : "My new VPN service",
+        "admin_state_up" : true,
+        "router_id" : "rou123",
+        "subnet_id" : "sub123"
+      }
+    }
+  }
+}
 '''
 
-vpnservice_template_deprecated = vpnservice_template.replace(
-    'subnet', 'subnet_id')
+vpnservice_template = '''
+{
+  "AWSTemplateFormatVersion" : "2010-09-09",
+  "Description" : "Template to test VPN service resource",
+  "Parameters" : {},
+  "Resources" : {
+    "VPNService" : {
+      "Type" : "OS::Neutron::VPNService",
+      "Properties" : {
+        "name" : "VPNService",
+        "description" : "My new VPN service",
+        "admin_state_up" : true,
+        "router_id" : "rou123",
+        "subnet" : "sub123"
+      }
+    }
+  }
+}
+'''
 
 ipsec_site_connection_template = '''
-heat_template_version: 2015-04-30
-description: Template to test IPsec policy resource
-resources:
-  IPsecSiteConnection:
-    type: OS::Neutron::IPsecSiteConnection,
-    properties:
-      name: IPsecSiteConnection
-      description: My new VPN connection
-      peer_address: 172.24.4.233
-      peer_id: 172.24.4.233
-      peer_cidrs: [ 10.2.0.0/24 ]
-      mtu: 1500
-      dpd:
-        actions: hold
-        interval: 30
-        timeout: 120
-      psk: secret
-      initiator: bi-directional
-      admin_state_up: true
-      ikepolicy_id: ike123
-      ipsecpolicy_id: ips123
-      vpnservice_id: vpn123
+{
+  "AWSTemplateFormatVersion" : "2010-09-09",
+  "Description" : "Template to test IPsec policy resource",
+  "Parameters" : {},
+  "Resources" : {
+    "IPsecSiteConnection" : {
+      "Type" : "OS::Neutron::IPsecSiteConnection",
+      "Properties" : {
+        "name" : "IPsecSiteConnection",
+        "description" : "My new VPN connection",
+        "peer_address" : "172.24.4.233",
+        "peer_id" : "172.24.4.233",
+        "peer_cidrs" : [ "10.2.0.0/24" ],
+        "mtu" : 1500,
+        "dpd" : {
+            "actions" : "hold",
+            "interval" : 30,
+            "timeout" : 120
+        },
+        "psk" : "secret",
+        "initiator" : "bi-directional",
+        "admin_state_up" : true,
+        "ikepolicy_id" : "ike123",
+        "ipsecpolicy_id" : "ips123",
+        "vpnservice_id" : "vpn123"
+      }
+    }
+  }
+}
 '''
 
 ikepolicy_template = '''
-heat_template_version: 2015-04-30
-description: Template to test IKE policy resource
-resources:
-  IKEPolicy:
-    type: OS::Neutron::IKEPolicy
-    properties:
-      name: IKEPolicy
-      description: My new IKE policy
-      auth_algorithm: sha1
-      encryption_algorithm: 3des
-      phase1_negotiation_mode: main
-      lifetime:
-        units: seconds
-        value: 3600
-      pfs: group5
-      ike_version: v1
+{
+  "AWSTemplateFormatVersion" : "2010-09-09",
+  "Description" : "Template to test IKE policy resource",
+  "Parameters" : {},
+  "Resources" : {
+    "IKEPolicy" : {
+      "Type" : "OS::Neutron::IKEPolicy",
+      "Properties" : {
+        "name" : "IKEPolicy",
+        "description" : "My new IKE policy",
+        "auth_algorithm" : "sha1",
+        "encryption_algorithm" : "3des",
+        "phase1_negotiation_mode" : "main",
+        "lifetime" : {
+            "units" : "seconds",
+            "value" : 3600
+        },
+        "pfs" : "group5",
+        "ike_version" : "v1"
+      }
+    }
+  }
+}
 '''
 
 ipsecpolicy_template = '''
-heat_template_version: 2015-04-30
-description: Template to test IPsec policy resource
-resources:
-  IPsecPolicy:
-    type: OS::Neutron::IPsecPolicy
-    properties:
-      name: IPsecPolicy
-      description: My new IPsec policy
-      transform_protocol: esp
-      encapsulation_mode: tunnel
-      auth_algorithm: sha1
-      encryption_algorithm: 3des
-      lifetime:
-        units: seconds
-        value: 3600
-      pfs : group5
+{
+  "AWSTemplateFormatVersion" : "2010-09-09",
+  "Description" : "Template to test IPsec policy resource",
+  "Parameters" : {},
+  "Resources" : {
+    "IPsecPolicy" : {
+      "Type" : "OS::Neutron::IPsecPolicy",
+      "Properties" : {
+        "name" : "IPsecPolicy",
+        "description" : "My new IPsec policy",
+        "transform_protocol": "esp",
+        "encapsulation_mode" : "tunnel",
+        "auth_algorithm" : "sha1",
+        "encryption_algorithm" : "3des",
+        "lifetime" : {
+            "units" : "seconds",
+            "value" : 3600
+        },
+        "pfs" : "group5"
+      }
+    }
+  }
+}
 '''
 
 
@@ -131,12 +175,12 @@ class VPNServiceTest(common.HeatTestCase):
     def create_vpnservice(self, resolve_neutron=True, resolve_router=True):
         self.stub_SubnetConstraint_validate()
         self.stub_RouterConstraint_validate()
-        neutronV20.find_resourceid_by_name_or_id(
-            mox.IsA(neutronclient.Client),
-            'subnet',
-            'sub123'
-        ).AndReturn('sub123')
         if resolve_neutron:
+            neutronV20.find_resourceid_by_name_or_id(
+                mox.IsA(neutronclient.Client),
+                'subnet',
+                'sub123'
+            ).AndReturn('sub123')
             snippet = template_format.parse(vpnservice_template)
         else:
             snippet = template_format.parse(vpnservice_template_deprecated)
@@ -146,7 +190,7 @@ class VPNServiceTest(common.HeatTestCase):
                 'router',
                 'rou123'
             ).AndReturn('rou123')
-            props = snippet['resources']['VPNService']['properties']
+            props = snippet['Resources']['VPNService']['Properties']
             props['router'] = 'rou123'
             del props['router_id']
         neutronclient.Client.create_vpnservice(
@@ -164,6 +208,9 @@ class VPNServiceTest(common.HeatTestCase):
     def test_create(self):
         self._test_create()
 
+    def test_create_router_id(self):
+        self._test_create(resolve_router=False)
+
     def _test_create(self, resolve_neutron=True, resolve_router=True):
         rsrc = self.create_vpnservice(resolve_neutron, resolve_router)
         self.m.ReplayAll()
@@ -172,11 +219,6 @@ class VPNServiceTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_create_failed(self):
-        neutronV20.find_resourceid_by_name_or_id(
-            mox.IsA(neutronclient.Client),
-            'router',
-            'rou123'
-        ).MultipleTimes().AndReturn('rou123')
         neutronV20.find_resourceid_by_name_or_id(
             mox.IsA(neutronclient.Client),
             'subnet',
