@@ -14,18 +14,16 @@
 import datetime
 import uuid
 
-from oslo_serialization import jsonutils as json
 from oslo_utils import timeutils
 import six
 
-from heat.common import exception
 from heat.common import identifier
 from heat.common import template_format
 from heat.engine.clients.os import heat_plugin
 from heat.engine import environment
+from heat.engine import parser
+from heat.engine import resource
 from heat.engine.resources.openstack.heat import wait_condition_handle as h_wch
-from heat.engine import stack as parser
-from heat.engine import template as tmpl
 from heat.objects import resource as resource_objects
 from heat.tests import common
 from heat.tests import utils
@@ -81,8 +79,8 @@ class HeatWaitConditionTest(common.HeatTestCase):
                      params={},
                      stub=True, stub_status=True):
         temp = template_format.parse(template)
-        template = tmpl.Template(temp,
-                                 env=environment.Environment(params))
+        template = parser.Template(temp,
+                                   env=environment.Environment(params))
         ctx = utils.dummy_context(tenant_id=self.tenant_id)
         stack = parser.Stack(ctx, 'test_stack', template,
                              disable_rollback=True)
@@ -214,8 +212,7 @@ class HeatWaitConditionTest(common.HeatTestCase):
                          'status': 'SUCCESS', 'id': '456'}
         ret = handle.handle_signal(details=test_metadata)
         wc_att = rsrc.FnGetAtt('data')
-        self.assertEqual(json.loads(u'{"123": "foo", "456": "dog"}'),
-                         json.loads(wc_att))
+        self.assertEqual(u'{"123": "foo", "456": "dog"}', wc_att)
         self.assertEqual('status:SUCCESS reason:cat', ret)
         self.m.VerifyAll()
 
@@ -232,8 +229,7 @@ class HeatWaitConditionTest(common.HeatTestCase):
                          'status': 'SUCCESS'}
         ret = handle.handle_signal(details=test_metadata)
         wc_att = rsrc.FnGetAtt('data')
-        self.assertEqual(json.loads(u'{"1": "foo", "2": "dog"}'),
-                         json.loads(wc_att))
+        self.assertEqual(u'{"1": "foo", "2": "dog"}', wc_att)
         self.assertEqual('status:SUCCESS reason:cat', ret)
         self.m.VerifyAll()
 
@@ -247,8 +243,7 @@ class HeatWaitConditionTest(common.HeatTestCase):
 
         handle.handle_signal()
         wc_att = rsrc.FnGetAtt('data')
-        self.assertEqual(json.loads(u'{"1": null, "2": null}'),
-                         json.loads(wc_att))
+        self.assertEqual(u'{"1": null, "2": null}', wc_att)
         self.m.VerifyAll()
 
     def test_data_partial_complete(self):
@@ -265,8 +260,7 @@ class HeatWaitConditionTest(common.HeatTestCase):
         expected = 'status:SUCCESS reason:Signal 2 received'
         self.assertEqual(expected, ret)
         wc_att = rsrc.FnGetAtt('data')
-        self.assertEqual(json.loads(u'{"1": null, "2": null}'),
-                         json.loads(wc_att))
+        self.assertEqual(u'{"1": null, "2": null}', wc_att)
         self.m.VerifyAll()
 
     def _create_heat_handle(self):
@@ -278,8 +272,6 @@ class HeatWaitConditionTest(common.HeatTestCase):
 
         handle = self.stack['wait_handle']
         self.assertEqual((handle.CREATE, handle.COMPLETE), handle.state)
-        self.assertIsNotNone(handle.password)
-        self.assertEqual(handle.resource_id, handle.data().get('user_id'))
         return handle
 
     def test_get_status_none_complete(self):
@@ -360,4 +352,4 @@ class HeatWaitConditionTest(common.HeatTestCase):
         handle = self.stack['update_wait_handle']
         self.assertEqual((handle.CREATE, handle.COMPLETE), handle.state)
         self.assertRaises(
-            exception.UpdateReplace, handle.update, None, None)
+            resource.UpdateReplace, handle.update, None, None)
