@@ -15,23 +15,24 @@ import json
 import os
 
 import mock
-from oslo.config import cfg
+from oslo_config import cfg
+import six
 
 from heat.api.aws import exception
 import heat.api.cfn.v1.stacks as stacks
 from heat.common import exception as heat_exception
 from heat.common import identifier
 from heat.common import policy
-from heat.common.wsgi import Request
+from heat.common import wsgi
 from heat.rpc import api as rpc_api
 from heat.rpc import client as rpc_client
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 from heat.tests import utils
 
 policy_path = os.path.dirname(os.path.realpath(__file__)) + "/policy/"
 
 
-class CfnStackControllerTest(HeatTestCase):
+class CfnStackControllerTest(common.HeatTestCase):
     '''
     Tests the API class which acts as the WSGI controller,
     the endpoint processing API requests after they are routed
@@ -66,7 +67,7 @@ class CfnStackControllerTest(HeatTestCase):
         params = params or {}
         qs = "&".join(["=".join([k, str(params[k])]) for k in params])
         environ = {'REQUEST_METHOD': 'GET', 'QUERY_STRING': qs}
-        req = Request(environ)
+        req = wsgi.Request(environ)
         req.context = utils.dummy_context()
         return req
 
@@ -263,20 +264,20 @@ class CfnStackControllerTest(HeatTestCase):
                                         u'DBRootPassword': u'admin',
                                         u'DBPassword': u'admin',
                                         u'DBName': u'wordpress'},
-                       u'outputs':
-                       [{u'output_key': u'WebsiteURL',
-                         u'description': u'URL for Wordpress wiki',
-                         u'output_value': u'http://10.0.0.8/wordpress'}],
-                       u'stack_status_reason': u'Stack successfully created',
-                       u'creation_time': u'2012-07-09T09:12:45Z',
-                       u'stack_name': u'wordpress',
-                       u'notification_topics': [],
-                       u'stack_action': u'CREATE',
-                       u'stack_status': u'COMPLETE',
-                       u'description': u'blah',
-                       u'disable_rollback': 'true',
-                       u'timeout_mins':60,
-                       u'capabilities':[]}]
+                        u'outputs':
+                        [{u'output_key': u'WebsiteURL',
+                          u'description': u'URL for Wordpress wiki',
+                          u'output_value': u'http://10.0.0.8/wordpress'}],
+                        u'stack_status_reason': u'Stack successfully created',
+                        u'creation_time': u'2012-07-09T09:12:45Z',
+                        u'stack_name': u'wordpress',
+                        u'notification_topics': [],
+                        u'stack_action': u'CREATE',
+                        u'stack_status': u'COMPLETE',
+                        u'description': u'blah',
+                        u'disable_rollback': 'true',
+                        u'timeout_mins':60,
+                        u'capabilities':[]}]
 
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
@@ -482,19 +483,36 @@ class CfnStackControllerTest(HeatTestCase):
 
     # TODO(shardy) : test the _get_template TemplateUrl case
 
-    def test_create(self):
-        # Format a dummy request
-        stack_name = "wordpress"
-        json_template = json.dumps(self.template)
-        params = {'Action': 'CreateStack', 'StackName': stack_name,
-                  'TemplateBody': '%s' % json_template,
-                  'TimeoutInMinutes': 30,
-                  'DisableRollback': 'true',
-                  'Parameters.member.1.ParameterKey': 'InstanceType',
-                  'Parameters.member.1.ParameterValue': 'm1.xlarge'}
-        engine_parms = {u'InstanceType': u'm1.xlarge'}
-        engine_args = {'timeout_mins': u'30', 'disable_rollback': 'true'}
-        dummy_req = self._dummy_GET_request(params)
+    def _stub_rpc_create_stack_call_failure(self, req_context, stack_name,
+                                            engine_parms, engine_args,
+                                            failure, need_stub=True):
+        if need_stub:
+            self.m.StubOutWithMock(policy.Enforcer, 'enforce')
+            self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
+        policy.Enforcer.enforce(req_context,
+                                'CreateStack').AndReturn(True)
+
+        # Insert an engine RPC error and ensure we map correctly to the
+        # heat exception type
+        rpc_client.EngineClient.call(
+            req_context,
+            ('create_stack',
+             {'stack_name': stack_name,
+              'template': self.template,
+              'params': engine_parms,
+              'files': {},
+              'args': engine_args,
+              'owner_id': None,
+              'nested_depth': 0,
+              'user_creds_id': None,
+              'parent_resource_name': None,
+              'stack_user_project_id': None}),
+            version='1.7'
+        ).AndRaise(failure)
+
+    def _stub_rpc_create_stack_call_success(self, stack_name, engine_parms,
+                                            engine_args, parameters):
+        dummy_req = self._dummy_GET_request(parameters)
         self._stub_enforce(dummy_req, 'CreateStack')
 
         # Stub out the RPC call to the engine with a pre-canned response
@@ -512,11 +530,33 @@ class CfnStackControllerTest(HeatTestCase):
               'params': engine_parms,
               'files': {},
               'args': engine_args,
-              'owner_id': None})
+              'owner_id': None,
+              'nested_depth': 0,
+              'user_creds_id': None,
+              'parent_resource_name': None,
+              'stack_user_project_id': None}),
+            version='1.7'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
+        return dummy_req
 
+    def test_create(self):
+        # Format a dummy request
+        stack_name = "wordpress"
+        json_template = json.dumps(self.template)
+        params = {'Action': 'CreateStack', 'StackName': stack_name,
+                  'TemplateBody': '%s' % json_template,
+                  'TimeoutInMinutes': 30,
+                  'DisableRollback': 'true',
+                  'Parameters.member.1.ParameterKey': 'InstanceType',
+                  'Parameters.member.1.ParameterValue': 'm1.xlarge'}
+        engine_parms = {u'InstanceType': u'm1.xlarge'}
+        engine_args = {'timeout_mins': u'30', 'disable_rollback': 'true'}
+        dummy_req = self._stub_rpc_create_stack_call_success(stack_name,
+                                                             engine_parms,
+                                                             engine_args,
+                                                             params)
         response = self.controller.create(dummy_req)
 
         expected = {
@@ -541,28 +581,10 @@ class CfnStackControllerTest(HeatTestCase):
                   'Parameters.member.1.ParameterValue': 'm1.xlarge'}
         engine_parms = {u'InstanceType': u'm1.xlarge'}
         engine_args = {'timeout_mins': u'30', 'disable_rollback': 'false'}
-        dummy_req = self._dummy_GET_request(params)
-        self._stub_enforce(dummy_req, 'CreateStack')
-
-        # Stub out the RPC call to the engine with a pre-canned response
-        engine_resp = {u'tenant': u't',
-                       u'stack_name': u'wordpress',
-                       u'stack_id': u'1',
-                       u'path': u''}
-
-        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndReturn(engine_resp)
-
-        self.m.ReplayAll()
+        dummy_req = self._stub_rpc_create_stack_call_success(stack_name,
+                                                             engine_parms,
+                                                             engine_args,
+                                                             params)
 
         response = self.controller.create(dummy_req)
 
@@ -588,28 +610,10 @@ class CfnStackControllerTest(HeatTestCase):
                   'Parameters.member.1.ParameterValue': 'm1.xlarge'}
         engine_parms = {u'InstanceType': u'm1.xlarge'}
         engine_args = {'timeout_mins': u'30', 'disable_rollback': 'true'}
-        dummy_req = self._dummy_GET_request(params)
-        self._stub_enforce(dummy_req, 'CreateStack')
-
-        # Stub out the RPC call to the engine with a pre-canned response
-        engine_resp = {u'tenant': u't',
-                       u'stack_name': u'wordpress',
-                       u'stack_id': u'1',
-                       u'path': u''}
-
-        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndReturn(engine_resp)
-
-        self.m.ReplayAll()
+        dummy_req = self._stub_rpc_create_stack_call_success(stack_name,
+                                                             engine_parms,
+                                                             engine_args,
+                                                             params)
 
         response = self.controller.create(dummy_req)
 
@@ -635,28 +639,10 @@ class CfnStackControllerTest(HeatTestCase):
                   'Parameters.member.1.ParameterValue': 'm1.xlarge'}
         engine_parms = {u'InstanceType': u'm1.xlarge'}
         engine_args = {'timeout_mins': u'30', 'disable_rollback': 'false'}
-        dummy_req = self._dummy_GET_request(params)
-        self._stub_enforce(dummy_req, 'CreateStack')
-
-        # Stub out the RPC call to the engine with a pre-canned response
-        engine_resp = {u'tenant': u't',
-                       u'stack_name': u'wordpress',
-                       u'stack_id': u'1',
-                       u'path': u''}
-
-        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndReturn(engine_resp)
-
-        self.m.ReplayAll()
+        dummy_req = self._stub_rpc_create_stack_call_success(stack_name,
+                                                             engine_parms,
+                                                             engine_args,
+                                                             params)
 
         response = self.controller.create(dummy_req)
 
@@ -682,28 +668,10 @@ class CfnStackControllerTest(HeatTestCase):
                   'Parameters.member.1.ParameterValue': 'm1.xlarge'}
         engine_parms = {u'InstanceType': u'm1.xlarge'}
         engine_args = {'timeout_mins': u'30', 'disable_rollback': 'false'}
-        dummy_req = self._dummy_GET_request(params)
-        self._stub_enforce(dummy_req, 'CreateStack')
-
-        # Stub out the RPC call to the engine with a pre-canned response
-        engine_resp = {u'tenant': u't',
-                       u'stack_name': u'wordpress',
-                       u'stack_id': u'1',
-                       u'path': u''}
-
-        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndReturn(engine_resp)
-
-        self.m.ReplayAll()
+        dummy_req = self._stub_rpc_create_stack_call_success(stack_name,
+                                                             engine_parms,
+                                                             engine_args,
+                                                             params)
 
         response = self.controller.create(dummy_req)
 
@@ -768,52 +736,25 @@ class CfnStackControllerTest(HeatTestCase):
         engine_parms = {u'InstanceType': u'm1.xlarge'}
         engine_args = {'timeout_mins': u'30'}
         dummy_req = self._dummy_GET_request(params)
-
-        self.m.StubOutWithMock(policy.Enforcer, 'enforce')
-
-        # Insert an engine RPC error and ensure we map correctly to the
-        # heat exception type
-        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
-
-        policy.Enforcer.enforce(dummy_req.context, 'CreateStack'
-                                ).AndReturn(True)
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndRaise(AttributeError())
-
-        policy.Enforcer.enforce(dummy_req.context, 'CreateStack'
-                                ).AndReturn(True)
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndRaise(heat_exception.UnknownUserParameter(key='test'))
-
-        policy.Enforcer.enforce(dummy_req.context, 'CreateStack'
-                                ).AndReturn(True)
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndRaise(heat_exception.UserParameterMissing(key='test'))
-
+        self._stub_rpc_create_stack_call_failure(dummy_req.context,
+                                                 stack_name,
+                                                 engine_parms,
+                                                 engine_args,
+                                                 AttributeError())
+        failure = heat_exception.UnknownUserParameter(key='test')
+        self._stub_rpc_create_stack_call_failure(dummy_req.context,
+                                                 stack_name,
+                                                 engine_parms,
+                                                 engine_args,
+                                                 failure,
+                                                 False)
+        failure = heat_exception.UserParameterMissing(key='test')
+        self._stub_rpc_create_stack_call_failure(dummy_req.context,
+                                                 stack_name,
+                                                 engine_parms,
+                                                 engine_args,
+                                                 failure,
+                                                 False)
         self.m.ReplayAll()
 
         result = self.controller.create(dummy_req)
@@ -836,26 +777,15 @@ class CfnStackControllerTest(HeatTestCase):
                   'Parameters.member.1.ParameterValue': 'm1.xlarge'}
         engine_parms = {u'InstanceType': u'm1.xlarge'}
         engine_args = {'timeout_mins': u'30'}
+        failure = heat_exception.StackExists(stack_name='test')
         dummy_req = self._dummy_GET_request(params)
-        self._stub_enforce(dummy_req, 'CreateStack')
-
-        # Insert an engine RPC error and ensure we map correctly to the
-        # heat exception type
-        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
-
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndRaise(heat_exception.StackExists(stack_name='test'))
+        self._stub_rpc_create_stack_call_failure(dummy_req.context,
+                                                 stack_name,
+                                                 engine_parms,
+                                                 engine_args,
+                                                 failure)
 
         self.m.ReplayAll()
-
         result = self.controller.create(dummy_req)
 
         self.assertIsInstance(result, exception.AlreadyExistsError)
@@ -871,26 +801,15 @@ class CfnStackControllerTest(HeatTestCase):
                   'Parameters.member.1.ParameterValue': 'm1.xlarge'}
         engine_parms = {u'InstanceType': u'm1.xlarge'}
         engine_args = {'timeout_mins': u'30'}
+        failure = heat_exception.StackValidationFailed(
+            message='Something went wrong')
         dummy_req = self._dummy_GET_request(params)
-        self._stub_enforce(dummy_req, 'CreateStack')
-
-        # Stub out the RPC call to the engine with a pre-canned response
-        self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
-
-        rpc_client.EngineClient.call(
-            dummy_req.context,
-            ('create_stack',
-             {'stack_name': stack_name,
-              'template': self.template,
-              'params': engine_parms,
-              'files': {},
-              'args': engine_args,
-              'owner_id': None})
-        ).AndRaise(heat_exception.StackValidationFailed(
-            message='Something went wrong'))
-
+        self._stub_rpc_create_stack_call_failure(dummy_req.context,
+                                                 stack_name,
+                                                 engine_parms,
+                                                 engine_args,
+                                                 failure)
         self.m.ReplayAll()
-
         result = self.controller.create(dummy_req)
 
         self.assertIsInstance(result, exception.HeatInvalidParameterValueError)
@@ -1274,7 +1193,7 @@ class CfnStackControllerTest(HeatTestCase):
         expected = {'DescribeStackEventsResponse':
                     {'DescribeStackEventsResult':
                      {'StackEvents':
-                      [{'EventId': unicode(event_id),
+                      [{'EventId': six.text_type(event_id),
                         'StackId': u'arn:openstack:heat::t:stacks/wordpress/6',
                         'ResourceStatus': u'TEST_IN_PROGRESS',
                         'ResourceType': u'AWS::EC2::Instance',
@@ -1369,9 +1288,10 @@ class CfnStackControllerTest(HeatTestCase):
         args = {
             'stack_identity': identity,
             'resource_name': dummy_req.params.get('LogicalResourceId'),
+            'with_attr': None,
         }
         rpc_client.EngineClient.call(
-            dummy_req.context, ('describe_stack_resource', args)
+            dummy_req.context, ('describe_stack_resource', args), version='1.2'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -1433,9 +1353,10 @@ class CfnStackControllerTest(HeatTestCase):
         args = {
             'stack_identity': identity,
             'resource_name': dummy_req.params.get('LogicalResourceId'),
+            'with_attr': None,
         }
         rpc_client.EngineClient.call(
-            dummy_req.context, ('describe_stack_resource', args)
+            dummy_req.context, ('describe_stack_resource', args), version='1.2'
         ).AndRaise(heat_exception.ResourceNotFound(
             resource_name='test', stack_name='test'))
 
