@@ -13,19 +13,19 @@
 
 
 import json
-import six
 
-from oslo.config import cfg
-from oslo.utils import importutils
+from oslo_config import cfg
+from oslo_utils import importutils
 import requests
+import six
 
 from heat.api.aws import ec2token
 from heat.api.aws import exception
-from heat.common.wsgi import Request
-from heat.tests.common import HeatTestCase
+from heat.common import wsgi
+from heat.tests import common
 
 
-class Ec2TokenTest(HeatTestCase):
+class Ec2TokenTest(common.HeatTestCase):
     '''
     Tests the Ec2Token middleware
     '''
@@ -40,7 +40,7 @@ class Ec2TokenTest(HeatTestCase):
         environ = environ or {}
         qs = "&".join(["=".join([k, str(params[k])]) for k in params])
         environ.update({'REQUEST_METHOD': 'GET', 'QUERY_STRING': qs})
-        req = Request(environ)
+        req = wsgi.Request(environ)
         return req
 
     def test_conf_get_paste(self):
@@ -60,6 +60,34 @@ class Ec2TokenTest(HeatTestCase):
             'http://192.0.2.9/v2.0/ec2tokens',
             ec2._conf_get_keystone_ec2_uri('http://192.0.2.9/v2.0/'))
 
+    def test_conf_get_ssl_default_options(self):
+        ec2 = ec2token.EC2Token(app=None, conf={})
+        self.assertTrue(ec2.ssl_options['verify'],
+                        "SSL verify should be True by default")
+        self.assertIsNone(ec2.ssl_options['cert'],
+                          "SSL client cert should be None by default")
+
+    def test_conf_ssl_insecure_option(self):
+        ec2 = ec2token.EC2Token(app=None, conf={})
+        cfg.CONF.set_default('insecure', 'True', group='ec2authtoken')
+        cfg.CONF.set_default('ca_file', None, group='ec2authtoken')
+        self.assertFalse(ec2.ssl_options['verify'])
+
+    def test_conf_get_ssl_opts(self):
+        cfg.CONF.set_default('auth_uri', 'https://192.0.2.9/v2.0/',
+                             group='ec2authtoken')
+        cfg.CONF.set_default('ca_file', '/home/user/cacert.pem',
+                             group='ec2authtoken')
+        cfg.CONF.set_default('insecure', 'false', group='ec2authtoken')
+        cfg.CONF.set_default('cert_file', '/home/user/mycert',
+                             group='ec2authtoken')
+        cfg.CONF.set_default('key_file', '/home/user/mykey',
+                             group='ec2authtoken')
+        ec2 = ec2token.EC2Token(app=None, conf={})
+        self.assertEqual('/home/user/cacert.pem', ec2.ssl_options['verify'])
+        self.assertEqual(('/home/user/mycert', '/home/user/mykey'),
+                         ec2.ssl_options['cert'])
+
     def test_get_signature_param_old(self):
         params = {'Signature': 'foo'}
         dummy_req = self._dummy_GET_request(params)
@@ -183,13 +211,15 @@ class Ec2TokenTest(HeatTestCase):
         self.assertEqual('xyz', ec2.__call__(dummy_req))
 
     def _stub_http_connection(self, headers=None, params=None, response=None,
-                              req_url='http://123:5000/v2.0/ec2tokens'):
+                              req_url='http://123:5000/v3/ec2tokens',
+                              verify=True, cert=None):
 
         headers = headers or {}
         params = params or {}
 
         class DummyHTTPResponse(object):
             text = response
+            headers = {'X-Subject-Token': 123}
 
             def json(self):
                 return json.loads(self.text)
@@ -206,7 +236,7 @@ class Ec2TokenTest(HeatTestCase):
                                  "path": "/v1",
                                  "body_hash": body_hash}})
         req_headers = {'Content-Type': 'application/json'}
-        requests.post(req_url, data=req_creds,
+        requests.post(req_url, data=req_creds, verify=verify, cert=cert,
                       headers=req_headers).AndReturn(DummyHTTPResponse())
 
     def test_call_ok(self):
@@ -222,9 +252,8 @@ class Ec2TokenTest(HeatTestCase):
                    'HTTP_AUTHORIZATION': auth_str}
         dummy_req = self._dummy_GET_request(environ=req_env)
 
-        ok_resp = json.dumps({'access': {'token': {
-            'id': 123,
-            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        ok_resp = json.dumps({'token': {
+            'project': {'name': 'tenant', 'id': 'abcd1234'}}})
         self._stub_http_connection(headers={'Authorization': auth_str},
                                    response=ok_resp)
         self.m.ReplayAll()
@@ -247,12 +276,12 @@ class Ec2TokenTest(HeatTestCase):
                    'HTTP_AUTHORIZATION': auth_str}
         dummy_req = self._dummy_GET_request(environ=req_env)
 
-        ok_resp = json.dumps({'access': {
+        ok_resp = json.dumps({
             'token': {
                 'id': 123,
-                'tenant': {'name': 'tenant', 'id': 'abcd1234'}
-            },
-            'metadata': {'roles': ['aa', 'bb', 'cc']}}})
+                'project': {'name': 'tenant', 'id': 'abcd1234'},
+                'roles': [{'name': 'aa'}, {'name': 'bb'}, {'name': 'cc'}]}
+        })
         self._stub_http_connection(headers={'Authorization': auth_str},
                                    response=ok_resp)
         self.m.ReplayAll()
@@ -338,9 +367,8 @@ class Ec2TokenTest(HeatTestCase):
                    'PATH_INFO': '/v1'}
         dummy_req = self._dummy_GET_request(params, req_env)
 
-        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
-            'id': 123,
-            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        ok_resp = json.dumps({'token': {
+            'project': {'name': 'tenant', 'id': 'abcd1234'}}})
         self._stub_http_connection(response=ok_resp,
                                    params={'AWSAccessKeyId': 'foo'})
         self.m.ReplayAll()
@@ -361,9 +389,8 @@ class Ec2TokenTest(HeatTestCase):
                    'PATH_INFO': '/v1'}
         dummy_req = self._dummy_GET_request(params, req_env)
 
-        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
-            'id': 123,
-            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        ok_resp = json.dumps({'token': {
+            'project': {'name': 'tenant', 'id': 'abcd1234'}}})
         err_msg = "EC2 access key not found."
         err_resp = json.dumps({'error': {'message': err_msg}})
 
@@ -465,9 +492,8 @@ class Ec2TokenTest(HeatTestCase):
                    'PATH_INFO': '/v1'}
         dummy_req = self._dummy_GET_request(params, req_env)
 
-        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
-            'id': 123,
-            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        ok_resp = json.dumps({'token': {
+            'project': {'name': 'tenant', 'id': 'abcd1234'}}})
         self._stub_http_connection(response=ok_resp,
                                    params={'AWSAccessKeyId': 'foo'})
         self.m.ReplayAll()
@@ -487,9 +513,8 @@ class Ec2TokenTest(HeatTestCase):
                    'PATH_INFO': '/v1'}
         dummy_req = self._dummy_GET_request(params, req_env)
 
-        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
-            'id': 123,
-            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        ok_resp = json.dumps({'token': {
+            'project': {'name': 'tenant', 'id': 'abcd1234'}}})
         self._stub_http_connection(response=ok_resp,
                                    params={'AWSAccessKeyId': 'foo'})
         self.m.ReplayAll()
@@ -510,12 +535,21 @@ class Ec2TokenTest(HeatTestCase):
                    'PATH_INFO': '/v1'}
         dummy_req = self._dummy_GET_request(params, req_env)
 
-        ok_resp = json.dumps({'access': {'metadata': {}, 'token': {
-            'id': 123,
-            'tenant': {'name': 'tenant', 'id': 'abcd1234'}}}})
+        ok_resp = json.dumps({'token': {
+            'project': {'name': 'tenant', 'id': 'abcd1234'}}})
         self._stub_http_connection(response=ok_resp,
                                    params={'AWSAccessKeyId': 'foo'})
         self.m.ReplayAll()
         self.assertEqual('woot', ec2.__call__(dummy_req))
 
         self.m.VerifyAll()
+
+    def test_filter_factory(self):
+        ec2_filter = ec2token.EC2Token_filter_factory(global_conf={})
+
+        self.assertEqual('xyz', ec2_filter('xyz').application)
+
+    def test_filter_factory_none_app(self):
+        ec2_filter = ec2token.EC2Token_filter_factory(global_conf={})
+
+        self.assertEqual(None, ec2_filter(None).application)
