@@ -14,26 +14,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from keystoneclient.exceptions import Unauthorized
-from keystoneclient.v2_0 import client as keystone_client
+from keystoneclient.auth.identity import v3 as ks_v3_auth
+from keystoneclient import exceptions as keystone_exc
+from keystoneclient import session as ks_session
+import mox
+from oslo_config import cfg
 import webob
 
-from heat.common.auth_password import KeystonePasswordAuthProtocol
-from heat.tests.common import HeatTestCase
+from heat.common import auth_password
+from heat.tests import common
 
-EXPECTED_V2_DEFAULT_ENV_RESPONSE = {
+
+cfg.CONF.import_opt('keystone_backend', 'heat.common.heat_keystoneclient')
+
+
+EXPECTED_ENV_RESPONSE = {
     'HTTP_X_IDENTITY_STATUS': 'Confirmed',
+    'HTTP_X_PROJECT_ID': 'tenant_id1',
+    'HTTP_X_PROJECT_NAME': 'tenant_name1',
     'HTTP_X_TENANT_ID': 'tenant_id1',
     'HTTP_X_TENANT_NAME': 'tenant_name1',
     'HTTP_X_USER_ID': 'user_id1',
     'HTTP_X_USER_NAME': 'user_name1',
     'HTTP_X_ROLES': 'role1,role2',
+    'HTTP_X_AUTH_TOKEN': 'lalalalalala',
     'HTTP_X_USER': 'user_name1',  # deprecated (diablo-compat)
     'HTTP_X_TENANT': 'tenant_name1',  # deprecated (diablo-compat)
     'HTTP_X_ROLE': 'role1,role2',  # deprecated (diablo-compat)
 }
 
-TOKEN_RESPONSE = {
+
+TOKEN_V2_RESPONSE = {
     'token': {
         'id': 'lalalalalala',
         'expires': '2020-01-01T00:00:10.000123Z',
@@ -54,12 +65,39 @@ TOKEN_RESPONSE = {
 }
 
 
+TOKEN_V3_RESPONSE = {
+    'version': 'v3',
+    'project': {
+        'id': 'tenant_id1',
+        'name': 'tenant_name1',
+    },
+    'token': {
+        'id': 'lalalalalala',
+        'expires': '2020-01-01T00:00:10.000123Z',
+        'tenant': {
+            'id': 'tenant_id1',
+            'name': 'tenant_name1',
+        },
+        'methods': ['password'],
+    },
+    'user': {
+        'id': 'user_id1',
+        'name': 'user_name1',
+    },
+    'roles': [
+        {'name': 'role1'},
+        {'name': 'role2'},
+    ],
+    'auth_token': 'lalalalalala'
+}
+
+
 class FakeApp(object):
     """This represents a WSGI app protected by our auth middleware."""
 
     def __init__(self, expected_env=None):
         expected_env = expected_env or {}
-        self.expected_env = dict(EXPECTED_V2_DEFAULT_ENV_RESPONSE)
+        self.expected_env = dict(EXPECTED_ENV_RESPONSE)
         self.expected_env.update(expected_env)
 
     def __call__(self, env, start_response):
@@ -71,27 +109,58 @@ class FakeApp(object):
         return resp(env, start_response)
 
 
-class KeystonePasswordAuthProtocolTest(HeatTestCase):
+class KeystonePasswordAuthProtocolTest(common.HeatTestCase):
 
     def setUp(self):
         super(KeystonePasswordAuthProtocolTest, self).setUp()
         self.config = {'auth_uri': 'http://keystone.test.com:5000'}
-        self.app = FakeApp(
-            expected_env={'HTTP_X_AUTH_URL': self.config['auth_uri']})
-        self.middleware = KeystonePasswordAuthProtocol(self.app, self.config)
+        self.app = FakeApp()
+        self.middleware = auth_password.KeystonePasswordAuthProtocol(
+            self.app, self.config)
 
     def _start_fake_response(self, status, headers):
         self.response_status = int(status.split(' ', 1)[0])
         self.response_headers = dict(headers)
 
-    def test_valid_request(self):
-        mock_client = self.m.CreateMock(keystone_client.Client)
-        self.m.StubOutWithMock(keystone_client, 'Client')
-        keystone_client.Client(
-            username='user_name1', password='goodpassword',
-            tenant_id='tenant_id1',
-            auth_url=self.config['auth_uri']).AndReturn(mock_client)
-        mock_client.auth_ref = TOKEN_RESPONSE
+    def test_valid_v2_request(self):
+        mock_auth = self.m.CreateMock(ks_v3_auth.Password)
+        self.m.StubOutWithMock(ks_v3_auth, 'Password')
+
+        ks_v3_auth.Password(
+            auth_url=self.config['auth_uri'],
+            password='goodpassword',
+            project_id='tenant_id1',
+            user_domain_id='default',
+            username='user_name1').AndReturn(mock_auth)
+
+        m = mock_auth.get_access(mox.IsA(ks_session.Session))
+        m.AndReturn(TOKEN_V2_RESPONSE)
+
+        self.app.expected_env['keystone.token_info'] = TOKEN_V2_RESPONSE
+        self.m.ReplayAll()
+        req = webob.Request.blank('/tenant_id1/')
+        req.headers['X_AUTH_USER'] = 'user_name1'
+        req.headers['X_AUTH_KEY'] = 'goodpassword'
+        req.headers['X_AUTH_URL'] = self.config['auth_uri']
+        self.middleware(req.environ, self._start_fake_response)
+        self.m.VerifyAll()
+
+    def test_valid_v3_request(self):
+        mock_auth = self.m.CreateMock(ks_v3_auth.Password)
+        self.m.StubOutWithMock(ks_v3_auth, 'Password')
+
+        ks_v3_auth.Password(auth_url=self.config['auth_uri'],
+                            password='goodpassword',
+                            project_id='tenant_id1',
+                            user_domain_id='default',
+                            username='user_name1').AndReturn(mock_auth)
+
+        m = mock_auth.get_access(mox.IsA(ks_session.Session))
+        m.AndReturn(TOKEN_V3_RESPONSE)
+
+        self.app.expected_env['keystone.token_info'] = {
+            'token': TOKEN_V3_RESPONSE
+        }
         self.m.ReplayAll()
         req = webob.Request.blank('/tenant_id1/')
         req.headers['X_AUTH_USER'] = 'user_name1'
@@ -101,12 +170,15 @@ class KeystonePasswordAuthProtocolTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_request_with_bad_credentials(self):
-        self.m.StubOutWithMock(
-            keystone_client, 'Client', use_mock_anything=True)
-        mock_client = keystone_client.Client(
-            username='user_name1', password='badpassword',
-            tenant_id='tenant_id1', auth_url=self.config['auth_uri'])
-        mock_client.AndRaise(Unauthorized(401))
+        self.m.StubOutWithMock(ks_v3_auth, 'Password')
+
+        m = ks_v3_auth.Password(auth_url=self.config['auth_uri'],
+                                password='badpassword',
+                                project_id='tenant_id1',
+                                user_domain_id='default',
+                                username='user_name1')
+        m.AndRaise(keystone_exc.Unauthorized(401))
+
         self.m.ReplayAll()
         req = webob.Request.blank('/tenant_id1/')
         req.headers['X_AUTH_USER'] = 'user_name1'
