@@ -15,20 +15,21 @@ import copy
 import json
 
 from ceilometerclient import exc as ceilometerclient_exc
+import mock
 import mox
-from oslo.config import cfg
+from oslo_config import cfg
 import six
 
 from heat.common import exception
 from heat.common import template_format
 from heat.engine.clients.os import ceilometer
 from heat.engine import parser
-from heat.engine.properties import schemata
+from heat.engine import properties as props
 from heat.engine import resource
-from heat.engine.resources.ceilometer import alarm
+from heat.engine.resources.openstack.ceilometer import alarm
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 from heat.tests import generic_resource
 from heat.tests import utils
 
@@ -110,22 +111,7 @@ class FakeCeilometerAlarm(object):
     alarm_id = 'foo'
 
 
-class FakeCeilometerAlarms(object):
-    def create(self, **kwargs):
-        pass
-
-    def update(self, **kwargs):
-        pass
-
-    def delete(self, alarm_id):
-        pass
-
-
-class FakeCeilometerClient(object):
-    alarms = FakeCeilometerAlarms()
-
-
-class CeilometerAlarmTest(HeatTestCase):
+class CeilometerAlarmTest(common.HeatTestCase):
     def setUp(self):
         super(CeilometerAlarmTest, self).setUp()
 
@@ -135,8 +121,7 @@ class CeilometerAlarmTest(HeatTestCase):
         cfg.CONF.set_default('heat_waitcondition_server_url',
                              'http://server.test:8000/v1/waitcondition')
 
-        self.stub_keystoneclient()
-        self.fa = FakeCeilometerClient()
+        self.fa = mock.Mock()
 
     def create_stack(self, template=None):
         if template is None:
@@ -161,15 +146,31 @@ class CeilometerAlarmTest(HeatTestCase):
         al['ok_actions'] = None
         al['repeat_actions'] = True
         al['enabled'] = True
-        al['evaluation_periods'] = 1
-        al['period'] = 60
-        al['threshold'] = 50
-        if 'matching_metadata' in al:
-            al['matching_metadata'] = dict(
-                ('metadata.metering.%s' % k, v)
-                for k, v in al['matching_metadata'].items())
+        rule = dict(
+            period=60,
+            evaluation_periods=1,
+            threshold=50)
+        for field in ['period', 'evaluation_periods', 'threshold']:
+            del al[field]
+        for field in ['statistic', 'comparison_operator', 'meter_name']:
+            rule[field] = al[field]
+            del al[field]
+        if 'query' in al and al['query']:
+            query = al['query']
         else:
-            al['matching_metadata'] = {}
+            query = []
+        if 'query' in al:
+            del al['query']
+        if 'matching_metadata' in al and al['matching_metadata']:
+            for k, v in al['matching_metadata'].items():
+                key = 'metadata.metering.' + k
+                query.append(dict(field=key, op='eq', value=six.text_type(v)))
+        if 'matching_metadata' in al:
+            del al['matching_metadata']
+        if query:
+            rule['query'] = query
+        al['threshold_rule'] = rule
+        al['type'] = 'threshold'
         self.m.StubOutWithMock(self.fa.alarms, 'create')
         self.fa.alarms.create(**al).AndReturn(FakeCeilometerAlarm())
         return stack
@@ -179,28 +180,43 @@ class CeilometerAlarmTest(HeatTestCase):
         Make sure that we can change the update-able properties
         without replacing the Alarm rsrc.
         '''
-        #short circuit the alarm's references
+        # short circuit the alarm's references
         t = template_format.parse(alarm_template)
         properties = t['Resources']['MEMAlarmHigh']['Properties']
         properties['alarm_actions'] = ['signal_handler']
         properties['matching_metadata'] = {'a': 'v'}
+        properties['query'] = [dict(field='b', op='eq', value='w')]
 
         self.stack = self.create_stack(template=json.dumps(t))
         self.m.StubOutWithMock(self.fa.alarms, 'update')
-        schema = schemata(alarm.CeilometerAlarm.properties_schema)
+        schema = props.schemata(alarm.CeilometerAlarm.properties_schema)
+        exns = ['period', 'evaluation_periods', 'threshold',
+                'statistic', 'comparison_operator', 'meter_name',
+                'matching_metadata', 'query']
         al2 = dict((k, mox.IgnoreArg())
-                   for k, s in schema.items() if s.update_allowed)
+                   for k, s in schema.items()
+                   if s.update_allowed and k not in exns)
         al2['alarm_id'] = mox.IgnoreArg()
-        del al2['enabled']
-        del al2['repeat_actions']
+        al2['type'] = 'threshold'
+        al2['threshold_rule'] = dict(
+            meter_name=properties['meter_name'],
+            period=90,
+            evaluation_periods=2,
+            threshold=39,
+            statistic='max',
+            comparison_operator='lt',
+            query=[
+                dict(field='c', op='ne', value='z'),
+                dict(field='metadata.metering.x', op='eq', value='y')
+            ])
         self.fa.alarms.update(**al2).AndReturn(None)
 
         self.m.ReplayAll()
         self.stack.create()
         rsrc = self.stack['MEMAlarmHigh']
 
-        props = copy.copy(rsrc.properties.data)
-        props.update({
+        properties = copy.copy(rsrc.properties.data)
+        properties.update({
             'comparison_operator': 'lt',
             'description': 'fruity',
             'evaluation_periods': '2',
@@ -212,10 +228,12 @@ class CeilometerAlarmTest(HeatTestCase):
             'insufficient_data_actions': [],
             'alarm_actions': [],
             'ok_actions': ['signal_handler'],
+            'matching_metadata': {'x': 'y'},
+            'query': [dict(field='c', op='ne', value='z')]
         })
         snippet = rsrc_defn.ResourceDefinition(rsrc.name,
                                                rsrc.type(),
-                                               props)
+                                               properties)
 
         scheduler.TaskRunner(rsrc.update, snippet)()
 
@@ -237,11 +255,11 @@ class CeilometerAlarmTest(HeatTestCase):
         self.stack.create()
         rsrc = self.stack['MEMAlarmHigh']
 
-        props = copy.copy(rsrc.properties.data)
-        props['meter_name'] = 'temp'
+        properties = copy.copy(rsrc.properties.data)
+        properties['meter_name'] = 'temp'
         snippet = rsrc_defn.ResourceDefinition(rsrc.name,
                                                rsrc.type(),
-                                               props)
+                                               properties)
 
         updater = scheduler.TaskRunner(rsrc.update, snippet)
         self.assertRaises(resource.UpdateReplace, updater)
@@ -288,6 +306,46 @@ class CeilometerAlarmTest(HeatTestCase):
 
         self.m.VerifyAll()
 
+    def test_alarm_metadata_prefix(self):
+        t = template_format.parse(alarm_template)
+        properties = t['Resources']['MEMAlarmHigh']['Properties']
+        # Test for bug/1383521, where meter_name is in NOVA_METERS
+        properties[alarm.CeilometerAlarm.METER_NAME] = 'memory.usage'
+        properties['matching_metadata'] = {'metadata.user_metadata.groupname':
+                                           'foo'}
+
+        self.stack = self.create_stack(template=json.dumps(t))
+
+        rsrc = self.stack['MEMAlarmHigh']
+        rsrc.properties.data = rsrc.cfn_to_ceilometer(self.stack, properties)
+        self.assertIsNone(rsrc.properties.data.get('matching_metadata'))
+        query = rsrc.properties.data['threshold_rule']['query']
+        expected_query = [{'field': u'metadata.user_metadata.groupname',
+                           'value': u'foo', 'op': 'eq'}]
+        self.assertEqual(expected_query, query)
+
+    def test_mem_alarm_high_correct_matching_metadata(self):
+        t = template_format.parse(alarm_template)
+        properties = t['Resources']['MEMAlarmHigh']['Properties']
+        properties['matching_metadata'] = {'fro': 'bar',
+                                           'bro': True,
+                                           'dro': 1234,
+                                           'pro': '{"Mem": {"Ala": {"Hig"}}}',
+                                           'tro': [1, 2, 3, 4]}
+
+        self.stack = self.create_stack(template=json.dumps(t))
+
+        self.m.ReplayAll()
+        self.stack.create()
+        rsrc = self.stack['MEMAlarmHigh']
+        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        rsrc.properties.data = rsrc.cfn_to_ceilometer(self.stack, properties)
+        self.assertIsNone(rsrc.properties.data.get('matching_metadata'))
+        for key in rsrc.properties.data['threshold_rule']['query']:
+            self.assertIsInstance(key['value'], six.text_type)
+
+        self.m.VerifyAll()
+
     def test_no_matching_metadata(self):
         """Make sure that we can pass in an empty matching_metadata."""
 
@@ -318,8 +376,8 @@ class CeilometerAlarmTest(HeatTestCase):
             error = self.assertRaises(exception.StackValidationFailed,
                                       rsrc.validate)
             self.assertEqual(
-                "Property error : MEMAlarmHigh: %s Value '60a' is not an "
-                "integer" % p, six.text_type(error))
+                "Property error: Resources.MEMAlarmHigh.Properties.%s: "
+                "Value '60a' is not an integer" % p, six.text_type(error))
 
     def test_mem_alarm_high_not_integer_parameters(self):
         snippet = template_format.parse(not_string_alarm_template)
@@ -333,8 +391,9 @@ class CeilometerAlarmTest(HeatTestCase):
             error = self.assertRaises(exception.StackValidationFailed,
                                       rsrc.validate)
             self.assertEqual(
-                "Property error : MEMAlarmHigh: %s int() argument must be "
-                "a string or a number, not 'list'" % p, six.text_type(error))
+                "Property error: Resources.MEMAlarmHigh.Properties.%s: "
+                "int() argument must be a string or a number, not "
+                "'list'" % p, six.text_type(error))
 
     def test_mem_alarm_high_check_not_required_parameters(self):
         snippet = template_format.parse(not_string_alarm_template)
@@ -347,7 +406,8 @@ class CeilometerAlarmTest(HeatTestCase):
         error = self.assertRaises(exception.StackValidationFailed,
                                   rsrc.validate)
         self.assertEqual(
-            "Property error : MEMAlarmHigh: Property meter_name not assigned",
+            "Property error: Resources.MEMAlarmHigh.Properties: "
+            "Property meter_name not assigned",
             six.text_type(error))
 
         for p in ('period', 'evaluation_periods', 'statistic',
@@ -378,12 +438,48 @@ class CeilometerAlarmTest(HeatTestCase):
 
         self.m.VerifyAll()
 
+    def _prepare_check_resource(self):
+        snippet = template_format.parse(not_string_alarm_template)
+        self.stack = utils.parse_stack(snippet)
+        res = self.stack['MEMAlarmHigh']
+        res.ceilometer = mock.Mock()
+        mock_alarm = mock.Mock(enabled=True, state='ok')
+        res.ceilometer().alarms.get.return_value = mock_alarm
+        return res
 
-class CombinationAlarmTest(HeatTestCase):
+    @mock.patch.object(alarm.watchrule.WatchRule, 'load')
+    def test_check(self, mock_load):
+        res = self._prepare_check_resource()
+        scheduler.TaskRunner(res.check)()
+        self.assertEqual((res.CHECK, res.COMPLETE), res.state)
+
+    @mock.patch.object(alarm.watchrule.WatchRule, 'load')
+    def test_check_watchrule_failure(self, mock_load):
+        res = self._prepare_check_resource()
+        exc = alarm.exception.WatchRuleNotFound(watch_name='Boom')
+        mock_load.side_effect = exc
+
+        self.assertRaises(exception.ResourceFailure,
+                          scheduler.TaskRunner(res.check))
+        self.assertEqual((res.CHECK, res.FAILED), res.state)
+        self.assertIn('Boom', res.status_reason)
+
+    @mock.patch.object(alarm.watchrule.WatchRule, 'load')
+    def test_check_alarm_failure(self, mock_load):
+        res = self._prepare_check_resource()
+        res.ceilometer().alarms.get.side_effect = Exception('Boom')
+
+        self.assertRaises(exception.ResourceFailure,
+                          scheduler.TaskRunner(res.check))
+        self.assertEqual((res.CHECK, res.FAILED), res.state)
+        self.assertIn('Boom', res.status_reason)
+
+
+class CombinationAlarmTest(common.HeatTestCase):
 
     def setUp(self):
         super(CombinationAlarmTest, self).setUp()
-        self.fc = FakeCeilometerClient()
+        self.fc = mock.Mock()
         self.m.StubOutWithMock(ceilometer.CeilometerClientPlugin, '_create')
 
     def create_alarm(self):
@@ -402,10 +498,10 @@ class CombinationAlarmTest(HeatTestCase):
                               'operator': u'and'}
         ).AndReturn(FakeCeilometerAlarm())
         snippet = template_format.parse(combination_alarm_template)
-        stack = utils.parse_stack(snippet)
-        resource_defns = stack.t.resource_definitions(stack)
+        self.stack = utils.parse_stack(snippet)
+        resource_defns = self.stack.t.resource_definitions(self.stack)
         return alarm.CombinationAlarm(
-            'CombinAlarm', resource_defns['CombinAlarm'], stack)
+            'CombinAlarm', resource_defns['CombinAlarm'], self.stack)
 
     def test_create(self):
         rsrc = self.create_alarm()
@@ -426,8 +522,9 @@ class CombinationAlarmTest(HeatTestCase):
         error = self.assertRaises(exception.StackValidationFailed,
                                   rsrc.validate)
         self.assertEqual(
-            "Property error : CombinAlarm: alarm_ids length (0) is out of "
-            "range (min: 1, max: None)", six.text_type(error))
+            "Property error: Resources.CombinAlarm.Properties.alarm_ids: "
+            "length (0) is out of range (min: 1, max: None)",
+            six.text_type(error))
 
     def test_update(self):
         rsrc = self.create_alarm()
@@ -495,3 +592,26 @@ class CombinationAlarmTest(HeatTestCase):
         self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
 
         self.m.VerifyAll()
+
+    def _prepare_check_resource(self):
+        snippet = template_format.parse(combination_alarm_template)
+        self.stack = utils.parse_stack(snippet)
+        res = self.stack['CombinAlarm']
+        res.ceilometer = mock.Mock()
+        mock_alarm = mock.Mock(enabled=True, state='ok')
+        res.ceilometer().alarms.get.return_value = mock_alarm
+        return res
+
+    def test_check(self):
+        res = self._prepare_check_resource()
+        scheduler.TaskRunner(res.check)()
+        self.assertEqual((res.CHECK, res.COMPLETE), res.state)
+
+    def test_check_failure(self):
+        res = self._prepare_check_resource()
+        res.ceilometer().alarms.get.side_effect = Exception('Boom')
+
+        self.assertRaises(exception.ResourceFailure,
+                          scheduler.TaskRunner(res.check))
+        self.assertEqual((res.CHECK, res.FAILED), res.state)
+        self.assertIn('Boom', res.status_reason)
