@@ -11,16 +11,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import uuid
+
 import mock
 
 from heat.engine import parser
-from heat.engine.resources.software_config import cloud_config as cc
+from heat.engine.resources.openstack.heat import cloud_config
 from heat.engine import template
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 from heat.tests import utils
 
 
-class CloudConfigTest(HeatTestCase):
+class CloudConfigTest(common.HeatTestCase):
 
     def setUp(self):
         super(CloudConfigTest, self).setUp()
@@ -38,23 +40,27 @@ class CloudConfigTest(HeatTestCase):
                         'Properties': self.properties
                     }}}))
         self.config = self.stack['config_mysql']
-        heat = mock.MagicMock()
-        self.config.heat = heat
-        self.software_configs = heat.return_value.software_configs
+        self.rpc_client = mock.MagicMock()
+        self.config._rpc_client = self.rpc_client
 
     def test_resource_mapping(self):
-        mapping = cc.resource_mapping()
+        mapping = cloud_config.resource_mapping()
         self.assertEqual(1, len(mapping))
-        self.assertEqual(cc.CloudConfig,
+        self.assertEqual(cloud_config.CloudConfig,
                          mapping['OS::Heat::CloudConfig'])
-        self.assertIsInstance(self.config, cc.CloudConfig)
+        self.assertIsInstance(self.config, cloud_config.CloudConfig)
 
     def test_handle_create(self):
-        sc = mock.MagicMock()
         config_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
-        sc.id = config_id
-        self.software_configs.create.return_value = sc
+        value = {'id': config_id}
+        self.rpc_client.create_software_config.return_value = value
+        self.config.id = 5
+        self.config.uuid = uuid.uuid4().hex
         self.config.handle_create()
         self.assertEqual(config_id, self.config.resource_id)
-        kwargs = self.software_configs.create.call_args[1]
-        self.assertEqual('#cloud-config\n{foo: bar}\n', kwargs['config'])
+        kwargs = self.rpc_client.create_software_config.call_args[1]
+        self.assertEqual({
+            'name': self.config.physical_resource_name(),
+            'config': '#cloud-config\n{foo: bar}\n',
+            'group': 'Heat::Ungrouped'
+        }, kwargs)
