@@ -11,21 +11,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
 import os
-from oslo.config import cfg
+
+import mock
+from oslo_config import cfg
+from oslo_middleware import request_id
 import webob
 
 from heat.common import context
 from heat.common import exception
-from heat.openstack.common.middleware import request_id
-from heat.openstack.common import policy as base_policy
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 
 policy_path = os.path.dirname(os.path.realpath(__file__)) + "/policy/"
 
 
-class TestRequestContext(HeatTestCase):
+class TestRequestContext(common.HeatTestCase):
 
     def setUp(self):
         self.ctx = {'username': 'mick',
@@ -40,9 +40,11 @@ class TestRequestContext(HeatTestCase):
                     'roles': ['arole', 'notadmin'],
                     'tenant_id': '456tenant',
                     'user_id': 'fooUser',
-                    'tenant': 'atenant',
+                    'tenant': u'\u5218\u80dc',
                     'auth_url': 'http://xyz',
-                    'aws_creds': 'blah'}
+                    'aws_creds': 'blah',
+                    'region_name': 'RegionOne',
+                    'user_identity': 'fooUser 456tenant'}
 
         super(TestRequestContext, self).setUp()
 
@@ -59,7 +61,11 @@ class TestRequestContext(HeatTestCase):
             roles=self.ctx.get('roles'),
             show_deleted=self.ctx.get('show_deleted'),
             is_admin=self.ctx.get('is_admin'),
-            auth_token_info=self.ctx.get('auth_token_info'))
+            auth_token_info=self.ctx.get('auth_token_info'),
+            trustor_user_id=self.ctx.get('trustor_user_id'),
+            trust_id=self.ctx.get('trust_id'),
+            user=self.ctx.get('user'),
+            region_name=self.ctx.get('region_name'))
         ctx_dict = ctx.to_dict()
         del(ctx_dict['request_id'])
         self.assertEqual(self.ctx, ctx_dict)
@@ -74,6 +80,8 @@ class TestRequestContext(HeatTestCase):
         ctx = context.RequestContext.from_dict(self.ctx)
 
         for k in self.ctx:
+            if k == 'user_identity':
+                continue
             self.assertEqual(self.ctx.get(k), ctx.to_dict().get(k))
             override = '%s_override' % k
             setattr(ctx, k, override)
@@ -104,7 +112,7 @@ class TestRequestContext(HeatTestCase):
             self.assertFalse(ctx.is_admin)
 
 
-class RequestContextMiddlewareTest(HeatTestCase):
+class RequestContextMiddlewareTest(common.HeatTestCase):
 
     scenarios = [(
         'empty_headers',
@@ -239,10 +247,7 @@ class RequestContextMiddlewareTest(HeatTestCase):
             cfg.StrOpt('project', default='heat'),
         ]
         cfg.CONF.register_opts(opts)
-        pf = policy_path + 'check_admin.json'
-        self.m.StubOutWithMock(base_policy.Enforcer, '_get_policy_path')
-        base_policy.Enforcer._get_policy_path().MultipleTimes().AndReturn(pf)
-        self.m.ReplayAll()
+        cfg.CONF.set_override('policy_file', 'check_admin.json')
 
     def test_context_middleware(self):
 
