@@ -11,27 +11,31 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from datetime import datetime
+import datetime as dt
+import json
 import uuid
 
-import json
 import mock
+from oslo_utils import timeutils
 import six
 
-from heat.common.identifier import EventIdentifier
+from heat.common import exception
+from heat.common import identifier
 from heat.common import template_format
 from heat.engine import api
-from heat.engine.event import Event
+from heat.engine import event
 from heat.engine import parameters
 from heat.engine import parser
 from heat.engine import resource
 from heat.rpc import api as rpc_api
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
 
+datetime = dt.datetime
 
-class FormatTest(HeatTestCase):
+
+class FormatTest(common.HeatTestCase):
     def setUp(self):
         super(FormatTest, self).setUp()
 
@@ -41,11 +45,14 @@ class FormatTest(HeatTestCase):
                 'generic1': {'Type': 'GenericResourceType'},
                 'generic2': {
                     'Type': 'GenericResourceType',
-                    'DependsOn': 'generic1'}
+                    'DependsOn': 'generic1'},
+                'generic4': {'Type': 'StackResourceType'}
             }
         })
         resource._register_class('GenericResourceType',
                                  generic_rsrc.GenericResource)
+        resource._register_class('StackResourceType',
+                                 generic_rsrc.StackResourceType)
         resource._register_class('ResWithComplexPropsAndAttrs',
                                  generic_rsrc.ResWithComplexPropsAndAttrs)
         self.stack = parser.Stack(utils.dummy_context(), 'test_stack',
@@ -53,11 +60,12 @@ class FormatTest(HeatTestCase):
 
     def _dummy_event(self, event_id):
         resource = self.stack['generic1']
-        return Event(utils.dummy_context(), self.stack, 'CREATE', 'COMPLETE',
-                     'state changed', 'z3455xyc-9f88-404d-a85b-5315293e67de',
-                     resource.properties, resource.name, resource.type(),
-                     uuid='abc123yc-9f88-404d-a85b-531529456xyz',
-                     id=event_id)
+        return event.Event(utils.dummy_context(), self.stack, 'CREATE',
+                           'COMPLETE', 'state changed',
+                           'z3455xyc-9f88-404d-a85b-5315293e67de',
+                           resource.properties, resource.name, resource.type(),
+                           uuid='abc123yc-9f88-404d-a85b-531529456xyz',
+                           id=event_id)
 
     def test_format_stack_resource(self):
         res = self.stack['generic1']
@@ -73,10 +81,14 @@ class FormatTest(HeatTestCase):
             rpc_api.RES_ID,
             rpc_api.RES_STACK_ID,
             rpc_api.RES_STACK_NAME,
-            rpc_api.RES_REQUIRED_BY))
+            rpc_api.RES_REQUIRED_BY,
+        ))
 
-        resource_details_keys = resource_keys.union(set(
-            (rpc_api.RES_DESCRIPTION, rpc_api.RES_METADATA)))
+        resource_details_keys = resource_keys.union(set((
+            rpc_api.RES_DESCRIPTION,
+            rpc_api.RES_METADATA,
+            rpc_api.RES_SCHEMA_ATTRIBUTES,
+        )))
 
         formatted = api.format_stack_resource(res, True)
         self.assertEqual(resource_details_keys, set(formatted.keys()))
@@ -93,6 +105,49 @@ class FormatTest(HeatTestCase):
         formatted_props = formatted[rpc_api.RES_SCHEMA_PROPERTIES]
         self.assertEqual('formatted_res_props', formatted_props)
 
+    @mock.patch.object(api, 'format_resource_attributes')
+    def test_format_stack_resource_with_attributes(self, mock_format_attrs):
+        mock_format_attrs.return_value = 'formatted_resource_attrs'
+        res = self.stack['generic1']
+
+        formatted = api.format_stack_resource(res, True, with_attr=['a', 'b'])
+        formatted_attrs = formatted[rpc_api.RES_SCHEMA_ATTRIBUTES]
+        self.assertEqual('formatted_resource_attrs', formatted_attrs)
+
+    def test_format_resource_attributes(self):
+        res = self.stack['generic1']
+        formatted_attributes = api.format_resource_attributes(res)
+        self.assertEqual(2, len(formatted_attributes))
+        self.assertIn('foo', formatted_attributes)
+        self.assertIn('Foo', formatted_attributes)
+
+    def test_format_resource_attributes_show_attribute(self):
+        res = mock.Mock()
+        res.attributes = {'a': 'a_value', 'show': {'b': 'b_value'}}
+
+        formatted_attributes = api.format_resource_attributes(res)
+        self.assertIn('b', formatted_attributes)
+        self.assertNotIn('a', formatted_attributes)
+
+    def test_format_resource_attributes_show_attribute_fail(self):
+        res = mock.Mock()
+        res.attributes = {'a': 'a_value', 'show': ''}
+
+        formatted_attributes = api.format_resource_attributes(res)
+        self.assertIn('a', formatted_attributes)
+        self.assertIn('show', formatted_attributes)
+
+    def test_format_resource_attributes_force_attributes(self):
+        res = self.stack['generic1']
+        force_attrs = ['a1', 'a2']
+
+        formatted_attributes = api.format_resource_attributes(res, force_attrs)
+        self.assertEqual(4, len(formatted_attributes))
+        self.assertIn('foo', formatted_attributes)
+        self.assertIn('Foo', formatted_attributes)
+        self.assertIn('a1', formatted_attributes)
+        self.assertIn('a2', formatted_attributes)
+
     def _get_formatted_resource_properties(self, res_name):
         tmpl = parser.Template(template_format.parse('''
             heat_template_version: 2013-05-23
@@ -128,7 +183,7 @@ class FormatTest(HeatTestCase):
         self.assertEqual('', props['a_string'])
 
     def test_format_stack_resource_with_nested_stack(self):
-        res = self.stack['generic1']
+        res = self.stack['generic4']
         nested_id = {'foo': 'bar'}
         res.nested = mock.Mock()
         res.nested.return_value.identifier.return_value = nested_id
@@ -137,7 +192,7 @@ class FormatTest(HeatTestCase):
         self.assertEqual(nested_id, formatted[rpc_api.RES_NESTED_STACK_ID])
 
     def test_format_stack_resource_with_nested_stack_none(self):
-        res = self.stack['generic1']
+        res = self.stack['generic4']
         res.nested = mock.Mock()
         res.nested.return_value = None
 
@@ -155,7 +210,41 @@ class FormatTest(HeatTestCase):
             rpc_api.RES_REQUIRED_BY))
 
         formatted = api.format_stack_resource(res, False)
-        self.assertEqual(resource_keys, set(formatted.keys()))
+        self.assertEqual(resource_keys, set(six.iterkeys(formatted)))
+
+    def test_format_stack_resource_with_nested_stack_not_found(self):
+        res = self.stack['generic4']
+        self.patchobject(parser.Stack, 'load',
+                         side_effect=exception.NotFound())
+
+        resource_keys = set((
+            rpc_api.RES_UPDATED_TIME,
+            rpc_api.RES_NAME,
+            rpc_api.RES_PHYSICAL_ID,
+            rpc_api.RES_ACTION,
+            rpc_api.RES_STATUS,
+            rpc_api.RES_STATUS_DATA,
+            rpc_api.RES_TYPE,
+            rpc_api.RES_ID,
+            rpc_api.RES_STACK_ID,
+            rpc_api.RES_STACK_NAME,
+            rpc_api.RES_REQUIRED_BY))
+
+        formatted = api.format_stack_resource(res, False)
+        # 'nested_stack_id' is not in formatted
+        self.assertEqual(resource_keys, set(six.iterkeys(formatted)))
+
+    def test_format_stack_resource_with_nested_stack_empty(self):
+        res = self.stack['generic4']
+        nested_id = {'foo': 'bar'}
+
+        res.nested = mock.MagicMock()
+        res.nested.return_value.identifier.return_value = nested_id
+        res.nested.return_value.__len__.return_value = 0
+
+        formatted = api.format_stack_resource(res, False)
+        res.nested.return_value.identifier.assert_called_once_with()
+        self.assertEqual(nested_id, formatted[rpc_api.RES_NESTED_STACK_ID])
 
     def test_format_stack_resource_required_by(self):
         res1 = api.format_stack_resource(self.stack['generic1'])
@@ -165,8 +254,7 @@ class FormatTest(HeatTestCase):
 
     def test_format_stack_resource_with_parent_stack(self):
         res = self.stack['generic1']
-        res.stack.parent_resource = mock.Mock()
-        res.stack.parent_resource.name = 'foobar'
+        res.stack.parent_resource_name = 'foobar'
 
         formatted = api.format_stack_resource(res, False)
         self.assertEqual('foobar', formatted[rpc_api.RES_PARENT_RESOURCE])
@@ -194,10 +282,11 @@ class FormatTest(HeatTestCase):
         self.assertEqual(event_keys, set(formatted.keys()))
 
         event_id_formatted = formatted[rpc_api.EVENT_ID]
-        event_identifier = EventIdentifier(event_id_formatted['tenant'],
-                                           event_id_formatted['stack_name'],
-                                           event_id_formatted['stack_id'],
-                                           event_id_formatted['path'])
+        event_identifier = identifier.EventIdentifier(
+            event_id_formatted['tenant'],
+            event_id_formatted['stack_name'],
+            event_id_formatted['stack_id'],
+            event_id_formatted['path'])
         self.assertEqual(event_id, event_identifier.event_id)
 
     @mock.patch.object(api, 'format_stack_resource')
@@ -311,7 +400,7 @@ class FormatTest(HeatTestCase):
         self.assertEqual(expected, info)
 
 
-class FormatValidateParameterTest(HeatTestCase):
+class FormatValidateParameterTest(common.HeatTestCase):
 
     base_template = '''
     {
@@ -853,13 +942,15 @@ class FormatValidateParameterTest(HeatTestCase):
         self.assertEqual(self.expected, param_formated)
 
 
-class FormatSoftwareConfigDeploymentTest(HeatTestCase):
+class FormatSoftwareConfigDeploymentTest(common.HeatTestCase):
 
     def _dummy_software_config(self):
         config = mock.Mock()
+        self.now = timeutils.utcnow()
         config.name = 'config_mysql'
         config.group = 'Heat::Shell'
         config.id = str(uuid.uuid4())
+        config.created_at = self.now
         config.config = {
             'inputs': [{'name': 'bar'}],
             'outputs': [{'name': 'result'}],
@@ -879,6 +970,8 @@ class FormatSoftwareConfigDeploymentTest(HeatTestCase):
         deployment.action = 'INIT'
         deployment.status = 'COMPLETE'
         deployment.status_reason = 'Because'
+        deployment.created_at = config.created_at
+        deployment.updated_at = config.created_at
         return deployment
 
     def test_format_software_config(self):
@@ -887,7 +980,10 @@ class FormatSoftwareConfigDeploymentTest(HeatTestCase):
         self.assertIsNotNone(result)
         self.assertEqual([{'name': 'bar'}], result['inputs'])
         self.assertEqual([{'name': 'result'}], result['outputs'])
+        self.assertEqual([{'name': 'result'}], result['outputs'])
         self.assertEqual({}, result['options'])
+        self.assertEqual(timeutils.isotime(self.now),
+                         result['creation_time'])
 
     def test_format_software_config_none(self):
         self.assertIsNone(api.format_software_config(None))
@@ -904,12 +1000,16 @@ class FormatSoftwareConfigDeploymentTest(HeatTestCase):
         self.assertEqual(deployment.action, result['action'])
         self.assertEqual(deployment.status, result['status'])
         self.assertEqual(deployment.status_reason, result['status_reason'])
+        self.assertEqual(timeutils.isotime(self.now),
+                         result['creation_time'])
+        self.assertEqual(timeutils.isotime(self.now),
+                         result['updated_time'])
 
     def test_format_software_deployment_none(self):
         self.assertIsNone(api.format_software_deployment(None))
 
 
-class TestExtractArgs(HeatTestCase):
+class TestExtractArgs(common.HeatTestCase):
     def test_timeout_extract(self):
         p = {'timeout_mins': '5'}
         args = api.extract_args(p)
@@ -945,11 +1045,9 @@ class TestExtractArgs(HeatTestCase):
         self.assertTrue(args.get('adopt_stack_data'))
 
     def test_invalid_adopt_stack_data(self):
-        p = {'adopt_stack_data': json.dumps("foo")}
-        error = self.assertRaises(ValueError, api.extract_args, p)
-        self.assertEqual(
-            'Unexpected adopt data "foo". Adopt data must be a dict.',
-            six.text_type(error))
+        params = {'adopt_stack_data': json.dumps("foo")}
+        exc = self.assertRaises(ValueError, api.extract_args, params)
+        self.assertIn('Invalid adopt data', six.text_type(exc))
 
     def test_adopt_stack_data_extract_not_present(self):
         args = api.extract_args({})
