@@ -12,13 +12,14 @@
 #    under the License.
 
 import json
+
 import mock
 
 from heat.common import template_format
 from heat.engine import scheduler
-from heat.tests.common import HeatTestCase
+from heat.tests import common
+from heat.tests.nova import fakes as fakes_nova
 from heat.tests import utils
-from heat.tests.v1_1 import fakes
 
 sg_template = {
     "heat_template_version": "2013-05-23",
@@ -40,14 +41,14 @@ class FakeGroup(object):
         self.name = name
 
 
-class NovaServerGroupTest(HeatTestCase):
+class NovaServerGroupTest(common.HeatTestCase):
     def setUp(self):
         super(NovaServerGroupTest, self).setUp()
 
     def _init_template(self, sg_template):
         template = template_format.parse(json.dumps(sg_template))
-        stack = utils.parse_stack(template)
-        self.sg = stack['ServerGroup']
+        self.stack = utils.parse_stack(template)
+        self.sg = self.stack['ServerGroup']
         # create mock clients and objects
         nova = mock.MagicMock()
         self.sg.nova = mock.MagicMock(return_value=nova)
@@ -55,24 +56,24 @@ class NovaServerGroupTest(HeatTestCase):
 
     def _create_sg(self, name):
         if name:
-                sg = sg_template['resources']['ServerGroup']
-                sg['properties']['name'] = name
-                self._init_template(sg_template)
-                self.sg_mgr.create.return_value = FakeGroup(name)
+            sg = sg_template['resources']['ServerGroup']
+            sg['properties']['name'] = name
+            self._init_template(sg_template)
+            self.sg_mgr.create.return_value = FakeGroup(name)
         else:
-                try:
-                        sg = sg_template['resources']['ServerGroup']
-                        del sg['properties']['name']
-                except Exception:
-                        pass
-                self._init_template(sg_template)
-                name = 'test'
-                n = name
+            try:
+                sg = sg_template['resources']['ServerGroup']
+                del sg['properties']['name']
+            except Exception:
+                pass
+            self._init_template(sg_template)
+            name = 'test'
+            n = name
 
-                def fake_create(name, policies):
-                        self.assertTrue(len(name) > 1)
-                        return FakeGroup(n)
-                self.sg_mgr.create = fake_create
+            def fake_create(name, policies):
+                self.assertTrue(len(name) > 1)
+                return FakeGroup(n)
+            self.sg_mgr.create = fake_create
         scheduler.TaskRunner(self.sg.create)()
         self.assertEqual((self.sg.CREATE, self.sg.COMPLETE),
                          self.sg.state)
@@ -99,7 +100,7 @@ class NovaServerGroupTest(HeatTestCase):
 
     def test_sg_delete_not_found(self):
         self._create_sg('test')
-        self.sg_mgr.delete.side_effect = fakes.fake_exception()
+        self.sg_mgr.delete.side_effect = fakes_nova.fake_exception()
         scheduler.TaskRunner(self.sg.delete)()
         self.sg_mgr.delete.assert_called_once_with('test')
         self.assertEqual((self.sg.DELETE, self.sg.COMPLETE),
