@@ -11,8 +11,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_serialization import jsonutils
 import six
-import testtools
 
 from heat.common import exception
 from heat.engine.cfn import functions as cfn_funcs
@@ -22,9 +22,10 @@ from heat.engine import parameters
 from heat.engine import properties
 from heat.engine import resources
 from heat.engine import support
+from heat.tests import common
 
 
-class PropertySchemaTest(testtools.TestCase):
+class PropertySchemaTest(common.HeatTestCase):
     def test_schema_all(self):
         d = {
             'type': 'string',
@@ -468,6 +469,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertTrue(schema.required)
         self.assertIsNone(schema.default)
         self.assertEqual(0, len(schema.constraints))
+        self.assertFalse(schema.allow_conversion)
 
     def test_from_number_param_min(self):
         default = "42"
@@ -546,6 +548,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
+        self.assertFalse(schema.allow_conversion)
 
         allowed_constraint = schema.constraints[0]
 
@@ -563,6 +566,7 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertEqual(properties.Schema.LIST, schema.type)
         self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
+        self.assertFalse(schema.allow_conversion)
 
     def test_from_json_param(self):
         param = parameters.Schema.from_dict('name', {
@@ -575,9 +579,10 @@ class PropertySchemaTest(testtools.TestCase):
         self.assertEqual(properties.Schema.MAP, schema.type)
         self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
+        self.assertTrue(schema.allow_conversion)
 
 
-class PropertyTest(testtools.TestCase):
+class PropertyTest(common.HeatTestCase):
     def test_required_default(self):
         p = properties.Property({'Type': 'String'})
         self.assertFalse(p.required())
@@ -862,6 +867,12 @@ class PropertyTest(testtools.TestCase):
         p = properties.Property({'Type': 'Map'})
         self.assertRaises(TypeError, p.get_value, ['foo'])
 
+    def test_map_allow_conversion(self):
+        p = properties.Property({'Type': 'Map'})
+        p.schema.allow_conversion = True
+        self.assertEqual('foo', p.get_value('foo'))
+        self.assertEqual(jsonutils.dumps(['foo']), p.get_value(['foo']))
+
     def test_map_schema_good(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
         p = properties.Property({'Type': 'Map', 'Schema': map_schema})
@@ -872,8 +883,8 @@ class PropertyTest(testtools.TestCase):
         p = properties.Property({'Type': 'Map', 'Schema': map_schema})
         ex = self.assertRaises(exception.StackValidationFailed,
                                p.get_value, {'valid': 'fish'}, True)
-        self.assertEqual('Property error : valid "fish" is not '
-                         'a valid boolean', six.text_type(ex))
+        self.assertEqual('Property error: valid: "fish" is not a '
+                         'valid boolean', six.text_type(ex))
 
     def test_map_schema_missing_data(self):
         map_schema = {'valid': {'Type': 'Boolean'}}
@@ -885,7 +896,7 @@ class PropertyTest(testtools.TestCase):
         p = properties.Property({'Type': 'Map', 'Schema': map_schema})
         ex = self.assertRaises(exception.StackValidationFailed,
                                p.get_value, {}, True)
-        self.assertEqual('Property error : Property valid not assigned',
+        self.assertEqual('Property error: Property valid not assigned',
                          six.text_type(ex))
 
     def test_list_schema_good(self):
@@ -904,8 +915,8 @@ class PropertyTest(testtools.TestCase):
         ex = self.assertRaises(exception.StackValidationFailed,
                                p.get_value,
                                [{'valid': 'True'}, {'valid': 'fish'}], True)
-        self.assertEqual('Property error : 1 Property error : 1: valid '
-                         '"fish" is not a valid boolean', six.text_type(ex))
+        self.assertEqual('Property error: [1].valid: "fish" is not '
+                         'a valid boolean', six.text_type(ex))
 
     def test_list_schema_int_good(self):
         list_schema = {'Type': 'Integer'}
@@ -917,11 +928,11 @@ class PropertyTest(testtools.TestCase):
         p = properties.Property({'Type': 'List', 'Schema': list_schema})
         ex = self.assertRaises(exception.StackValidationFailed,
                                p.get_value, [42, 'fish'], True)
-        self.assertEqual('Property error : 1 Value \'fish\' is not '
-                         'an integer', six.text_type(ex))
+        self.assertEqual("Property error: [1]: Value 'fish' is not "
+                         "an integer", six.text_type(ex))
 
 
-class PropertiesTest(testtools.TestCase):
+class PropertiesTest(common.HeatTestCase):
     def setUp(self):
         super(PropertiesTest, self).setUp()
         schema = {
@@ -1046,16 +1057,6 @@ class PropertiesTest(testtools.TestCase):
         props = properties.Properties(schema, {'foo': None})
         self.assertEqual(['one', 'two'], props['foo'])
 
-    def test_bad_resolver(self):
-        schema = {'foo': {'Type': 'String', 'Default': 'bar'}}
-
-        def bad_resolver(prop):
-            raise Exception('resolution failed!')
-
-        props = properties.Properties(schema, {'foo': 'baz'}, bad_resolver)
-        err = self.assertRaises(ValueError, props.get, 'foo')
-        self.assertEqual('foo resolution failed!', six.text_type(err))
-
     def test_resolve_returns_none(self):
         schema = {'foo': {'Type': 'String', "MinLength": "5"}}
 
@@ -1099,7 +1100,10 @@ class PropertiesTest(testtools.TestCase):
         class rsrc(object):
             action = INIT = "INIT"
 
-        stack = {'another_res': rsrc()}
+        class DummyStack(dict):
+            pass
+
+        stack = DummyStack(another_res=rsrc())
 
         # define properties with function and constraint
         props = properties.Properties(
@@ -1498,7 +1502,7 @@ class PropertiesTest(testtools.TestCase):
         self.assertTrue(props_a != props_b)
 
 
-class PropertiesValidationTest(testtools.TestCase):
+class PropertiesValidationTest(common.HeatTestCase):
     def test_required(self):
         schema = {'foo': {'Type': 'String', 'Required': True}}
         props = properties.Properties(schema, {'foo': 'bar'})
@@ -1533,14 +1537,14 @@ class PropertiesValidationTest(testtools.TestCase):
         schema = {'foo': {'Type': 'String'}}
         props = properties.Properties(schema, {'foo': ['foo', 'bar']})
         ex = self.assertRaises(exception.StackValidationFailed, props.validate)
-        self.assertEqual('Property error : foo Value must be a string',
+        self.assertEqual('Property error: foo: Value must be a string',
                          six.text_type(ex))
 
     def test_dict_instead_string(self):
         schema = {'foo': {'Type': 'String'}}
         props = properties.Properties(schema, {'foo': {'foo': 'bar'}})
         ex = self.assertRaises(exception.StackValidationFailed, props.validate)
-        self.assertEqual('Property error : foo Value must be a string',
+        self.assertEqual('Property error: foo: Value must be a string',
                          six.text_type(ex))
 
     def test_none_string(self):
@@ -1613,8 +1617,9 @@ class PropertiesValidationTest(testtools.TestCase):
 
         prop_expected = {'foo': {'Ref': 'foo'}}
         param_expected = {'foo': {'Type': 'Json'}}
-        (parameters, props) = \
-            properties.Properties.schema_to_parameters_and_properties(schema)
+        (parameters,
+         props) = properties.Properties.schema_to_parameters_and_properties(
+             schema)
         self.assertEqual(param_expected, parameters)
         self.assertEqual(prop_expected, props)
 
@@ -1628,8 +1633,9 @@ class PropertiesValidationTest(testtools.TestCase):
 
         prop_expected = {'foo': {'Fn::Split': [",", {'Ref': 'foo'}]}}
         param_expected = {'foo': {'Type': 'CommaDelimitedList'}}
-        (parameters, props) = \
-            properties.Properties.schema_to_parameters_and_properties(schema)
+        (parameters,
+         props) = properties.Properties.schema_to_parameters_and_properties(
+             schema)
         self.assertEqual(param_expected, parameters)
         self.assertEqual(prop_expected, props)
 
@@ -1648,8 +1654,9 @@ class PropertiesValidationTest(testtools.TestCase):
 
         prop_expected = {'foo': {'Fn::Split': [",", {'Ref': 'foo'}]}}
         param_expected = {'foo': {'Type': 'CommaDelimitedList'}}
-        (parameters, props) = \
-            properties.Properties.schema_to_parameters_and_properties(schema)
+        (parameters,
+         props) = properties.Properties.schema_to_parameters_and_properties(
+             schema)
         self.assertEqual(param_expected, parameters)
         self.assertEqual(prop_expected, props)
 
@@ -1661,8 +1668,9 @@ class PropertiesValidationTest(testtools.TestCase):
         prop_expected = {'foo': {'Ref': 'foo'}}
         param_expected = {'foo': {'Type': 'String'}}
 
-        (parameters, props) = \
-            properties.Properties.schema_to_parameters_and_properties(schema)
+        (parameters,
+         props) = properties.Properties.schema_to_parameters_and_properties(
+             schema)
         self.assertEqual(param_expected, parameters)
         self.assertEqual(prop_expected, props)
 
@@ -1681,12 +1689,12 @@ class PropertiesValidationTest(testtools.TestCase):
             )
         }
         props = properties.Properties(schema, {})
-        self.assertEqual(props.props['foo_sup'].support_status().status,
-                         support.SUPPORTED)
-        self.assertEqual(props.props['bar_dep'].support_status().status,
-                         support.DEPRECATED)
-        self.assertEqual(props.props['bar_dep'].support_status().message,
-                         'Do not use this ever')
+        self.assertEqual(support.SUPPORTED,
+                         props.props['foo_sup'].support_status().status)
+        self.assertEqual(support.DEPRECATED,
+                         props.props['bar_dep'].support_status().status)
+        self.assertEqual('Do not use this ever',
+                         props.props['bar_dep'].support_status().message)
 
     def test_nested_properties_schema_invalid_property_in_list(self):
         child_schema = {'Key': {'Type': 'String',
@@ -1705,8 +1713,8 @@ class PropertiesValidationTest(testtools.TestCase):
         props = properties.Properties(schema, invalid_data)
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
-        self.assertEqual('Property error : foo Property error : foo: 0 '
-                         'Unknown Property bar', six.text_type(ex))
+        self.assertEqual('Property error: foo[0]: Unknown Property bar',
+                         six.text_type(ex))
 
     def test_nested_properties_schema_invalid_property_in_map(self):
         child_schema = {'Key': {'Type': 'String',
@@ -1725,8 +1733,8 @@ class PropertiesValidationTest(testtools.TestCase):
         props = properties.Properties(schema, invalid_data)
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
-        self.assertEqual('Property error : foo Property error : foo: boo '
-                         'Unknown Property bar', six.text_type(ex))
+        self.assertEqual('Property error: foo.boo: Unknown Property bar',
+                         six.text_type(ex))
 
     def test_more_nested_properties_schema_invalid_property_in_list(self):
         nested_child_schema = {'Key': {'Type': 'String',
@@ -1743,8 +1751,7 @@ class PropertiesValidationTest(testtools.TestCase):
         props = properties.Properties(schema, invalid_data)
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
-        self.assertEqual('Property error : foo Property error : foo: 0 '
-                         'Property error : 0: doo Unknown Property bar',
+        self.assertEqual('Property error: foo[0].doo: Unknown Property bar',
                          six.text_type(ex))
 
     def test_more_nested_properties_schema_invalid_property_in_map(self):
@@ -1762,15 +1769,15 @@ class PropertiesValidationTest(testtools.TestCase):
         props = properties.Properties(schema, invalid_data)
         ex = self.assertRaises(exception.StackValidationFailed,
                                props.validate)
-        self.assertEqual('Property error : foo Property error : foo: boo '
-                         'Property error : boo: doo Unknown Property bar',
+        self.assertEqual('Property error: foo.boo.doo: Unknown Property bar',
                          six.text_type(ex))
 
     def test_schema_to_template_empty_schema(self):
         schema = {}
 
-        (parameters, props) = \
-            properties.Properties.schema_to_parameters_and_properties(schema)
+        (parameters,
+         props) = properties.Properties.schema_to_parameters_and_properties(
+             schema)
         self.assertEqual({}, parameters)
         self.assertEqual({}, props)
 
