@@ -381,6 +381,47 @@ class ProviderTemplateTest(common.HeatTestCase):
                          "DummyResource (Map) and provider (String)",
                          six.text_type(ex))
 
+    def test_properties_list_with_none(self):
+        provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'Foo': {'Type': "CommaDelimitedList"},
+            },
+        }
+        files = {'test_resource.template': json.dumps(provider)}
+
+        class DummyResource(generic_rsrc.GenericResource):
+            support_status = support.SupportStatus()
+            properties_schema = {"Foo":
+                                 properties.Schema(properties.Schema.LIST)}
+            attributes_schema = {}
+
+        env = environment.Environment()
+        resource._register_class('DummyResource', DummyResource)
+        env.load({'resource_registry':
+                  {'DummyResource': 'test_resource.template'}})
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             parser.Template(
+                                 {'HeatTemplateFormatVersion': '2012-12-12'},
+                                 files=files, env=env),
+                             stack_id=str(uuid.uuid4()))
+
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Foo": [None,
+                                                           'test', None]})
+        temp_res = template_resource.TemplateResource('test_t_res',
+                                                      definition, stack)
+        self.assertIsNone(temp_res.validate())
+
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Foo": [None,
+                                                           None, None]})
+        temp_res = template_resource.TemplateResource('test_t_res',
+                                                      definition, stack)
+        self.assertIsNone(temp_res.validate())
+
     def test_properties_type_match(self):
         provider = {
             'HeatTemplateFormatVersion': '2012-12-12',
@@ -924,9 +965,13 @@ class TemplateResourceCrudTest(common.HeatTestCase):
         self.res.uuid = six.text_type(uuid.uuid4())
         self.res.resource_id = six.text_type(uuid.uuid4())
         self.res.action = self.res.CREATE
+        self.res.nested = mock.MagicMock()
         ident = identifier.HeatIdentifier(self.ctx.tenant_id,
                                           self.res.physical_resource_name(),
                                           self.res.resource_id)
+        self.res.nested().identifier.return_value = ident
         self.res.handle_delete()
         rpcc = self.res.rpc_client.return_value
-        rpcc.delete_stack.assert_called_once_with(self.ctx, ident)
+        rpcc.delete_stack.assert_called_once_with(
+            self.ctx,
+            self.res.nested().identifier())
