@@ -12,17 +12,15 @@
 #    under the License.
 
 import copy
+
 import mock
 import six
-import uuid
 
 from heat.common import exception
 from heat.engine import properties
 from heat.engine import resource
-from heat.engine.resources import resource_group
-from heat.engine import scheduler
+from heat.engine.resources.openstack.heat import resource_group
 from heat.engine import stack as stackm
-from heat.engine import template as templatem
 from heat.tests import common
 from heat.tests import generic_resource
 from heat.tests import utils
@@ -143,41 +141,7 @@ class ResourceGroupTest(common.HeatTestCase):
         AttributeResource = generic_resource.ResourceWithComplexAttributes
         resource._register_class("dummyattr.resource",
                                  AttributeResource)
-
-    def test_build_resource_definition(self):
-        stack = utils.parse_stack(template)
-        snip = stack.t.resource_definitions(stack)['group1']
-        resg = resource_group.ResourceGroup('test', snip, stack)
-        expect = {
-            "type": "dummy.resource",
-            "properties": {
-                "Foo": "Bar"
-            },
-        }
-        self.assertEqual(
-            expect, resg._build_resource_definition())
-        self.assertEqual(
-            expect, resg._build_resource_definition(include_all=True))
-
-    def test_build_resource_definition_include(self):
-        templ = copy.deepcopy(template)
-        res_def = templ["resources"]["group1"]["properties"]['resource_def']
-        res_def['properties']['Foo'] = None
-        stack = utils.parse_stack(templ)
-        snip = stack.t.resource_definitions(stack)['group1']
-        resg = resource_group.ResourceGroup('test', snip, stack)
-        expect = {
-            "type": "dummy.resource",
-            "properties": {}
-        }
-        self.assertEqual(
-            expect, resg._build_resource_definition())
-        expect = {
-            "type": "dummy.resource",
-            "properties": {"Foo": None}
-        }
-        self.assertEqual(
-            expect, resg._build_resource_definition(include_all=True))
+        self.m.StubOutWithMock(stackm.Stack, 'validate')
 
     def test_assemble_nested(self):
         """
@@ -354,86 +318,197 @@ class ResourceGroupTest(common.HeatTestCase):
         resgrp = resource_group.ResourceGroup('test', snip, stack)
         self.assertIsNone(resgrp.validate())
 
-    def test_zero_resources(self):
-        noresources = copy.deepcopy(template)
-        noresources['resources']['group1']['properties']['count'] = 0
-        resg = self._create_dummy_stack(noresources, expect_count=0)
-        self.assertEqual((resg.CREATE, resg.COMPLETE), resg.state)
+    def test_invalid_removal_policies_nolist(self):
+        """Test that error raised for malformed removal_policies."""
+        tmp = copy.deepcopy(template)
+        grp_props = tmp['resources']['group1']['properties']
+        grp_props['removal_policies'] = 'notallowed'
+        stack = utils.parse_stack(tmp)
+        snip = stack.t.resource_definitions(stack)['group1']
+        resg = resource_group.ResourceGroup('test', snip, stack)
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                resg.validate)
+        errstr = "removal_policies: \"'notallowed'\" is not a list"
+        self.assertIn(errstr, six.text_type(exc))
 
-    def test_delete(self):
-        """Test basic delete."""
-        resg = self._create_dummy_stack()
-        self.assertIsNotNone(resg.nested())
-        scheduler.TaskRunner(resg.delete)()
-        self.assertEqual((resg.DELETE, resg.COMPLETE), resg.nested().state)
-        self.assertEqual((resg.DELETE, resg.COMPLETE), resg.state)
+    def test_invalid_removal_policies_nomap(self):
+        """Test that error raised for malformed removal_policies."""
+        tmp = copy.deepcopy(template)
+        grp_props = tmp['resources']['group1']['properties']
+        grp_props['removal_policies'] = ['notallowed']
+        stack = utils.parse_stack(tmp)
+        snip = stack.t.resource_definitions(stack)['group1']
+        resg = resource_group.ResourceGroup('test', snip, stack)
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                resg.validate)
+        errstr = '"notallowed" is not a map'
+        self.assertIn(errstr, six.text_type(exc))
 
-    def test_update(self):
-        """Test basic update."""
-        resg = self._create_dummy_stack()
-        new_snip = copy.deepcopy(resg.t)
-        new_snip['Properties']['count'] = 3
-        scheduler.TaskRunner(resg.update, new_snip)()
-        self.stack = resg.nested()
-        self.assertEqual((resg.UPDATE, resg.COMPLETE), resg.state)
-        self.assertEqual((resg.UPDATE, resg.COMPLETE), resg.nested().state)
-        self.assertEqual(3, len(resg.nested()))
+    def test_child_template(self):
+        stack = utils.parse_stack(template2)
+        snip = stack.t.resource_definitions(stack)['group1']
+        resgrp = resource_group.ResourceGroup('test', snip, stack)
+        resgrp._assemble_nested = mock.Mock(return_value='tmpl')
+        resgrp.properties.data[resgrp.COUNT] = 2
 
-    def test_props_update(self):
-        """Test update of resource_def properties."""
-        resg = self._create_dummy_stack()
-        self.assertEqual(2, len(resg.nested()))
-        resource_names = [r.name for r in resg.nested().iter_resources()]
-        self.assertEqual(['0', '1'], sorted(resource_names))
-        new_snip = copy.deepcopy(resg.t)
-        new_snip['Properties']['resource_def']['properties']['Foo'] = 'xyz'
-        preupdate_resgid = resg.id
-        preupdate_nestedid = resg.nested().id
-        scheduler.TaskRunner(resg.update, new_snip)()
-        self.stack = resg.nested()
-        self.assertEqual((resg.UPDATE, resg.COMPLETE), resg.state)
-        self.assertEqual((resg.UPDATE, resg.COMPLETE), resg.nested().state)
-        self.assertEqual(2, len(resg.nested()))
-        resource_names = [r.name for r in resg.nested().iter_resources()]
-        self.assertEqual(['0', '1'], sorted(resource_names))
-        # resource_def update should recurse and update (not replace) nested
-        self.assertEqual(preupdate_resgid, resg.id)
-        self.assertEqual(preupdate_nestedid, resg.nested().id)
+        self.assertEqual('tmpl', resgrp.child_template())
+        resgrp._assemble_nested.assert_called_once_with(['0', '1'])
 
-    def test_update_nochange(self):
-        """Test update with no properties change."""
-        resg = self._create_dummy_stack()
-        self.assertEqual(2, len(resg.nested()))
-        resource_names = [r.name for r in resg.nested().iter_resources()]
-        self.assertEqual(['0', '1'], sorted(resource_names))
-        new_snip = copy.deepcopy(resg.t)
-        scheduler.TaskRunner(resg.update, new_snip)()
-        self.stack = resg.nested()
-        self.assertEqual((resg.UPDATE, resg.COMPLETE), resg.state)
-        self.assertEqual((resg.UPDATE, resg.COMPLETE), resg.nested().state)
-        self.assertEqual(2, len(resg.nested()))
-        resource_names = [r.name for r in resg.nested().iter_resources()]
-        self.assertEqual(['0', '1'], sorted(resource_names))
+    def test_child_params(self):
+        stack = utils.parse_stack(template2)
+        snip = stack.t.resource_definitions(stack)['group1']
+        resgrp = resource_group.ResourceGroup('test', snip, stack)
+        self.assertEqual({}, resgrp.child_params())
 
-    def test_update_nochange_resource_needs_update(self):
-        """Test update when the resource definition has changed."""
-        # Test the scenario when the ResourceGroup update happens without
-        # any changed properties, this can happen if the definition of
-        # a contained provider resource changes (files map changes), then
-        # the group and underlying nested stack should end up updated.
-        resg = self._create_dummy_stack()
-        self.assertEqual(2, len(resg.nested()))
-        resource_names = [r.name for r in resg.nested().iter_resources()]
-        self.assertEqual(['0', '1'], sorted(resource_names))
-        new_snip = copy.deepcopy(resg.t)
-        resg._needs_update = mock.Mock(return_value=True)
-        scheduler.TaskRunner(resg.update, new_snip)()
-        self.stack = resg.nested()
-        self.assertEqual((resg.UPDATE, resg.COMPLETE), resg.state)
-        self.assertEqual((resg.UPDATE, resg.COMPLETE), resg.nested().state)
-        self.assertEqual(2, len(resg.nested()))
-        resource_names = [r.name for r in resg.nested().iter_resources()]
-        self.assertEqual(['0', '1'], sorted(resource_names))
+
+class ResourceGroupBlackList(common.HeatTestCase):
+    """This class tests ResourceGroup._name_blacklist()."""
+
+    # 1) no resource_list, empty blacklist
+    # 2) no resource_list, existing blacklist
+    # 3) resource_list not in nested()
+    # 4) resource_list (refid) not in nested()
+    # 5) resource_list in nested() -> saved
+    # 6) resource_list (refid) in nested() -> saved
+    scenarios = [
+        ('1', dict(data_in=None, rm_list=[],
+                   nested_rsrcs={}, expected=[],
+                   saved=False)),
+        ('2', dict(data_in='0,1,2', rm_list=[],
+                   nested_rsrcs={}, expected=['0', '1', '2'],
+                   saved=False)),
+        ('3', dict(data_in='1,3', rm_list=['6'],
+                   nested_rsrcs=['0', '1', '3'],
+                   expected=['1', '3'],
+                   saved=False)),
+        ('4', dict(data_in='0,1', rm_list=['id-7'],
+                   nested_rsrcs=['0', '1', '3'],
+                   expected=['0', '1'],
+                   saved=False)),
+        ('5', dict(data_in='0,1', rm_list=['3'],
+                   nested_rsrcs=['0', '1', '3'],
+                   expected=['0', '1', '3'],
+                   saved=True)),
+        ('6', dict(data_in='0,1', rm_list=['id-3'],
+                   nested_rsrcs=['0', '1', '3'],
+                   expected=['0', '1', '3'],
+                   saved=True)),
+    ]
+
+    def test_blacklist(self):
+        stack = utils.parse_stack(template)
+        resg = stack['group1']
+
+        # mock properties
+        resg.properties = mock.MagicMock()
+        resg.properties.__getitem__.return_value = [
+            {'resource_list': self.rm_list}]
+
+        # mock data get/set
+        resg.data = mock.Mock()
+        resg.data.return_value.get.return_value = self.data_in
+        resg.data_set = mock.Mock()
+
+        # mock nested access
+        def stack_contains(name):
+            return name in self.nested_rsrcs
+
+        def by_refid(name):
+            rid = name.replace('id-', '')
+            if rid not in self.nested_rsrcs:
+                return None
+            res = mock.Mock()
+            res.name = rid
+            return res
+
+        nested = mock.MagicMock()
+        nested.__contains__.side_effect = stack_contains
+        nested.resource_by_refid.side_effect = by_refid
+        resg.nested = mock.Mock(return_value=nested)
+
+        self.assertEqual(self.expected, resg._name_blacklist())
+        if self.saved:
+            resg.data_set.assert_called_once_with('name_blacklist',
+                                                  ','.join(self.expected))
+
+
+class ResourceGroupEmptyParams(common.HeatTestCase):
+    """This class tests ResourceGroup._build_resource_definition()."""
+
+    scenarios = [
+        ('non_empty', dict(value='Bar', expected={'Foo': 'Bar'},
+                           expected_include={'Foo': 'Bar'})),
+        ('empty_None', dict(value=None, expected={},
+                            expected_include={'Foo': None})),
+        ('empty_boolean', dict(value=False, expected={'Foo': False},
+                               expected_include={'Foo': False})),
+        ('empty_string', dict(value='', expected={'Foo': ''},
+                              expected_include={'Foo': ''})),
+        ('empty_number', dict(value=0, expected={'Foo': 0},
+                              expected_include={'Foo': 0})),
+        ('empty_json', dict(value={}, expected={'Foo': {}},
+                            expected_include={'Foo': {}})),
+        ('empty_list', dict(value=[], expected={'Foo': []},
+                            expected_include={'Foo': []}))
+    ]
+
+    def test_definition(self):
+        templ = copy.deepcopy(template)
+        res_def = templ["resources"]["group1"]["properties"]['resource_def']
+        res_def['properties']['Foo'] = self.value
+        stack = utils.parse_stack(templ)
+        snip = stack.t.resource_definitions(stack)['group1']
+        resg = resource_group.ResourceGroup('test', snip, stack)
+        exp1 = {
+            "type": "dummy.resource",
+            "properties": self.expected,
+        }
+        exp2 = {
+            "type": "dummy.resource",
+            "properties": self.expected_include,
+        }
+        self.assertEqual(exp1, resg._build_resource_definition())
+        self.assertEqual(
+            exp2, resg._build_resource_definition(include_all=True))
+
+
+class ResourceGroupNameListTest(common.HeatTestCase):
+    """This class tests ResourceGroup._resource_names()."""
+
+    # 1) no blacklist, 0 count
+    # 2) no blacklist, x count
+    # 3) blacklist (not effecting)
+    # 4) blacklist with pruning
+    scenarios = [
+        ('1', dict(blacklist=[], count=0,
+                   expected=[])),
+        ('2', dict(blacklist=[], count=4,
+                   expected=['0', '1', '2', '3'])),
+        ('3', dict(blacklist=['5', '6'], count=3,
+                   expected=['0', '1', '2'])),
+        ('4', dict(blacklist=['2', '4'], count=4,
+                   expected=['0', '1', '3', '5'])),
+    ]
+
+    def test_names(self):
+        stack = utils.parse_stack(template)
+        resg = stack['group1']
+
+        resg.properties = mock.MagicMock()
+        resg.properties.get.return_value = self.count
+        resg._name_blacklist = mock.MagicMock(return_value=self.blacklist)
+        self.assertEqual(self.expected, resg._resource_names())
+
+
+class ResourceGroupAttrTest(common.HeatTestCase):
+
+    def setUp(self):
+        super(ResourceGroupAttrTest, self).setUp()
+        resource._register_class("dummy.resource",
+                                 ResourceWithPropsAndId)
+        AttributeResource = generic_resource.ResourceWithComplexAttributes
+        resource._register_class("dummyattr.resource",
+                                 AttributeResource)
 
     def test_aggregate_attribs(self):
         """
@@ -468,7 +543,8 @@ class ResourceGroupTest(common.HeatTestCase):
         Test attribute aggregation and that we mimic the nested resource's
         attributes.
         """
-        resg = self._create_dummy_stack(template_attr)
+        resg = self._create_dummy_stack(template_attr,
+                                        expect_attrs={'0': 2, '1': 2})
         self.assertEqual(2, resg.FnGetAtt('resource.0',
                                           'nested_dict', 'dict', 'b'))
         self.assertEqual(2, resg.FnGetAtt('resource.1',
@@ -479,19 +555,11 @@ class ResourceGroupTest(common.HeatTestCase):
         Test attribute aggregation and that we mimic the nested resource's
         attributes.
         """
-        resg = self._create_dummy_stack(template_attr)
+        resg = self._create_dummy_stack(template_attr,
+                                        expect_attrs={'0': 3, '1': 3})
         expected = [3, 3]
         self.assertEqual(expected, resg.FnGetAtt('nested_dict', 'list', 2))
 
-    def test_get_attr_path(self):
-        """
-        Test attribute aggregation and that we mimic the nested resource's
-        attributes.
-        """
-        resg = self._create_dummy_stack(template_attr)
-        expected = ['abc', 'abc']
-        self.assertEqual(expected, resg.stack.output('nested_strings'))
-
     def test_aggregate_refs(self):
         """
         Test resource id aggregation
@@ -500,12 +568,22 @@ class ResourceGroupTest(common.HeatTestCase):
         expected = ['ID-0', 'ID-1']
         self.assertEqual(expected, resg.FnGetAtt("refs"))
 
+    def test_aggregate_refs_with_index(self):
+        """
+        Test resource id aggregation with index
+        """
+        resg = self._create_dummy_stack()
+        expected = ['ID-0', 'ID-1']
+        self.assertEqual(expected[0], resg.FnGetAtt("refs", 0))
+        self.assertEqual(expected[1], resg.FnGetAtt("refs", 1))
+        self.assertIsNone(resg.FnGetAtt("refs", 2))
+
     def test_aggregate_outputs(self):
         """
         Test outputs aggregation
         """
-        resg = self._create_dummy_stack(template_attr)
         expected = {'0': ['foo', 'bar'], '1': ['foo', 'bar']}
+        resg = self._create_dummy_stack(template_attr, expect_attrs=expected)
         self.assertEqual(expected, resg.FnGetAtt('attributes', 'list'))
 
     def test_aggregate_outputs_no_path(self):
@@ -524,78 +602,23 @@ class ResourceGroupTest(common.HeatTestCase):
         self.assertRaises(exception.InvalidTemplateAttribute, resg.FnGetAtt,
                           'resource.2')
 
-    def _create_dummy_stack(self, template_data=template, expect_count=2):
+    def _create_dummy_stack(self, template_data=template, expect_count=2,
+                            expect_attrs=None):
         stack = utils.parse_stack(template_data)
         resg = stack['group1']
-        scheduler.TaskRunner(resg.create)()
-        self.stack = resg.nested()
-        self.assertEqual(expect_count, len(resg.nested()))
-        self.assertEqual((resg.CREATE, resg.COMPLETE), resg.state)
+        fake_res = {}
+        if expect_attrs is None:
+            expect_attrs = {}
+        for resc in range(expect_count):
+            res = str(resc)
+            fake_res[res] = mock.Mock()
+            fake_res[res].FnGetRefId.return_value = 'ID-%s' % res
+            if res in expect_attrs:
+                fake_res[res].FnGetAtt.return_value = expect_attrs[res]
+            else:
+                fake_res[res].FnGetAtt.return_value = res
+        resg.nested = mock.Mock(return_value=fake_res)
+
+        names = [str(name) for name in range(expect_count)]
+        resg._resource_names = mock.Mock(return_value=names)
         return resg
-
-    def test_child_template(self):
-        stack = utils.parse_stack(template2)
-        snip = stack.t.resource_definitions(stack)['group1']
-        resgrp = resource_group.ResourceGroup('test', snip, stack)
-        resgrp._assemble_nested = mock.Mock(return_value='tmpl')
-        resgrp.properties.data[resgrp.COUNT] = 2
-
-        self.assertEqual('tmpl', resgrp.child_template())
-        resgrp._assemble_nested.assert_called_once_with(['0', '1'])
-
-    def test_child_params(self):
-        stack = utils.parse_stack(template2)
-        snip = stack.t.resource_definitions(stack)['group1']
-        resgrp = resource_group.ResourceGroup('test', snip, stack)
-        self.assertEqual({}, resgrp.child_params())
-
-    def test_adopt(self):
-        tmpl = templatem.Template(template)
-        stack = stackm.Stack(utils.dummy_context(),
-                             'test_stack',
-                             tmpl,
-                             stack_id=str(uuid.uuid4()))
-
-        resg = stack['group1']
-
-        adopt_data = {
-            "status": "COMPLETE",
-            "name": "group1",
-            "resource_data": {},
-            "metadata": {},
-            "resource_id": "test-group1-id",
-            "action": "CREATE",
-            "type": "OS::Heat::ResourceGroup",
-            "resources": {
-                "0": {
-                    "status": "COMPLETE",
-                    "name": "0",
-                    "resource_data": {},
-                    "resource_id": "ID-0",
-                    "action": "CREATE",
-                    "type": "dummy.resource",
-                    "metadata": {}
-                },
-                "1": {
-                    "status": "COMPLETE",
-                    "name": "1",
-                    "resource_data": {},
-                    "resource_id": "ID-1",
-                    "action": "CREATE",
-                    "type": "dummy.resource",
-                    "metadata": {}
-                }
-            }
-        }
-        scheduler.TaskRunner(resg.adopt, adopt_data)()
-        self.assertEqual((resg.ADOPT, resg.COMPLETE), resg.state)
-        self.assertEqual(adopt_data['name'], resg.name)
-        # a new nested stack should be created
-        self.assertIsNotNone(resg.resource_id)
-        # verify all the resources in resource group are adopted.
-        self.assertEqual(adopt_data['resources']['0']['resource_id'],
-                         resg.FnGetAtt('resource.0'))
-        self.assertEqual(adopt_data['resources']['1']['resource_id'],
-                         resg.FnGetAtt('resource.1'))
-        self.assertRaises(exception.InvalidTemplateAttribute, resg.FnGetAtt,
-                          'resource.2')
