@@ -1,3 +1,4 @@
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -13,9 +14,10 @@
 import mock
 
 from heat.common import template_format
-from heat.engine.resources import instance
+from heat.engine.clients.os import nova
+from heat.engine.resources.aws.ec2 import instance
+from heat.engine.resources.openstack.heat import ha_restarter
 from heat.engine import scheduler
-
 from heat.tests import common
 from heat.tests import utils
 
@@ -40,13 +42,15 @@ restarter_template = '''
 class RestarterTest(common.HeatTestCase):
     def setUp(self):
         super(RestarterTest, self).setUp()
-        utils.setup_dummy_db()
 
     def create_restarter(self):
         snippet = template_format.parse(restarter_template)
-        stack = utils.parse_stack(snippet)
-        restarter = instance.Restarter(
-            'restarter', snippet['Resources']['restarter'], stack)
+        self.stack = utils.parse_stack(snippet)
+        resource_defns = self.stack.t.resource_definitions(self.stack)
+        restarter = ha_restarter.Restarter(
+            'restarter', resource_defns['restarter'], self.stack)
+        nova.NovaClientPlugin.get_server = mock.Mock(
+            return_value=mock.MagicMock())
         restarter.handle_create = mock.Mock(return_value=None)
         return restarter
 
@@ -54,6 +58,10 @@ class RestarterTest(common.HeatTestCase):
         inst = mock.Mock(spec=instance.Instance)
         inst.resource_id = '1234'
         inst.name = 'instance'
+        inst.action = inst.CREATE
+        inst.status = inst.COMPLETE
+        inst.state = (inst.action, inst.status)
+        inst.FnGetRefId = lambda: inst.resource_id
         stack.resources['instance'] = inst
 
     def test_create(self):
