@@ -42,7 +42,6 @@ resources:
         node_processes:
           - namenode
           - jobtracker
-        is_proxy_gateway: True
 """
 
 cluster_template = """
@@ -58,53 +57,24 @@ resources:
       neutron_management_network: some_network
 """
 
-cluster_template_without_name = """
-heat_template_version: 2013-05-23
-resources:
-  cluster_template!:
-    type: OS::Sahara::ClusterTemplate
-    properties:
-      plugin_name: vanilla
-      hadoop_version: 2.3.0
-      neutron_management_network: some_network
-"""
-
-node_group_template_without_name = """
-heat_template_version: 2013-05-23
-resources:
-  node_group!:
-    type: OS::Sahara::NodeGroupTemplate
-    properties:
-        plugin_name: vanilla
-        hadoop_version: 2.3.0
-        flavor: m1.large
-        floating_ip_pool: some_pool_name
-        node_processes:
-          - namenode
-          - jobtracker
-"""
-
 
 class FakeNodeGroupTemplate(object):
     def __init__(self):
         self.id = "some_ng_id"
         self.name = "test-cluster-template"
-        self.to_dict = lambda: {"ng-template": "info"}
 
 
 class FakeClusterTemplate(object):
     def __init__(self):
         self.id = "some_ct_id"
         self.name = "node-group-template"
-        self.to_dict = lambda: {"cluster-template": "info"}
 
 
 class SaharaNodeGroupTemplateTest(common.HeatTestCase):
     def setUp(self):
         super(SaharaNodeGroupTemplateTest, self).setUp()
-        self.stub_FlavorConstraint_validate()
-        self.stub_SaharaPluginConstraint()
-        self.stub_VolumeTypeConstraint_validate()
+        self.patchobject(st.constraints.CustomConstraint,
+                         '_is_valid').return_value = True
         self.patchobject(nova.NovaClientPlugin, 'get_flavor_id'
                          ).return_value = 'someflavorid'
         self.patchobject(neutron.NeutronClientPlugin, '_create')
@@ -114,8 +84,6 @@ class SaharaNodeGroupTemplateTest(common.HeatTestCase):
         self.ngt_mgr = sahara_mock.node_group_templates
         self.patchobject(sahara.SaharaClientPlugin,
                          '_create').return_value = sahara_mock
-        self.patchobject(sahara.SaharaClientPlugin, 'validate_hadoop_version'
-                         ).return_value = None
         self.fake_ngt = FakeNodeGroupTemplate()
 
         self.t = template_format.parse(node_group_template)
@@ -156,9 +124,6 @@ class SaharaNodeGroupTemplateTest(common.HeatTestCase):
                            'floating_ip_pool': 'some_pool_id',
                            'node_configs': None,
                            'image_id': None,
-                           'is_proxy_gateway': True,
-                           'volume_local_to_instance': None,
-                           'use_autoconfig': None
                            }
         self.ngt_mgr.create.assert_called_once_with(*expected_args,
                                                     **expected_kwargs)
@@ -218,33 +183,16 @@ class SaharaNodeGroupTemplateTest(common.HeatTestCase):
         self.t['resources']['node-group']['properties'].pop('floating_ip_pool')
         self.t['resources']['node-group']['properties'].pop('volume_type')
         ngt = self._init_ngt(self.t)
-        self.patchobject(nova.FlavorConstraint, 'validate'
+        self.patchobject(st.constraints.CustomConstraint, '_is_valid'
                          ).return_value = False
         self.patchobject(ngt, 'is_using_neutron').return_value = False
 
         ex = self.assertRaises(exception.StackValidationFailed, ngt.validate)
         self.assertEqual(u"Property error: "
                          u"resources.node-group.properties.flavor: "
-                         u"Error validating value 'm1.large'",
+                         u"Error validating value u'm1.large'",
                          six.text_type(ex))
 
-    def test_template_invalid_name(self):
-        tmpl = template_format.parse(node_group_template_without_name)
-        stack = utils.parse_stack(tmpl)
-        ngt = stack['node_group!']
-        self.ngt_mgr.create.return_value = self.fake_ngt
-        scheduler.TaskRunner(ngt.create)()
-        self.assertEqual((ngt.CREATE, ngt.COMPLETE), ngt.state)
-        self.assertEqual(self.fake_ngt.id, ngt.resource_id)
-        name = self.ngt_mgr.create.call_args[0][0]
-        self.assertIn('-nodegroup-', name)
-
-    def test_ngt_show_resource(self):
-        ngt = self._create_ngt(self.t)
-        self.ngt_mgr.get.return_value = self.fake_ngt
-        self.assertEqual({"ng-template": "info"}, ngt.FnGetAtt('show'))
-        self.ngt_mgr.get.assert_called_once_with('some_ng_id')
-
 
 class SaharaClusterTemplateTest(common.HeatTestCase):
     def setUp(self):
@@ -258,8 +206,6 @@ class SaharaClusterTemplateTest(common.HeatTestCase):
         self.ct_mgr = sahara_mock.cluster_templates
         self.patchobject(sahara.SaharaClientPlugin,
                          '_create').return_value = sahara_mock
-        self.patchobject(sahara.SaharaClientPlugin, 'validate_hadoop_version'
-                         ).return_value = None
         self.fake_ct = FakeClusterTemplate()
 
         self.t = template_format.parse(cluster_template)
@@ -286,19 +232,17 @@ class SaharaClusterTemplateTest(common.HeatTestCase):
 
     def test_ct_create(self):
         self._create_ct(self.t)
-        args = {
-            'name': 'test-cluster-template',
-            'plugin_name': 'vanilla',
-            'hadoop_version': '2.3.0',
-            'description': '',
-            'default_image_id': None,
-            'net_id': 'some_network_id',
-            'anti_affinity': None,
-            'node_groups': None,
-            'cluster_configs': None,
-            'use_autoconfig': None
-        }
-        self.ct_mgr.create.assert_called_once_with(**args)
+        expected_args = ('test-cluster-template', 'vanilla',
+                         '2.3.0')
+        expected_kwargs = {'description': '',
+                           'default_image_id': None,
+                           'net_id': 'some_network_id',
+                           'anti_affinity': None,
+                           'node_groups': None,
+                           'cluster_configs': None
+                           }
+        self.ct_mgr.create.assert_called_once_with(*expected_args,
+                                                   **expected_kwargs)
 
     def test_ct_delete(self):
         ct = self._create_ct(self.t)
@@ -331,42 +275,3 @@ class SaharaClusterTemplateTest(common.HeatTestCase):
                                ct.validate)
         self.assertEqual("neutron_management_network must be provided",
                          six.text_type(ex))
-
-    def test_template_invalid_name(self):
-        tmpl = template_format.parse(cluster_template_without_name)
-        stack = utils.parse_stack(tmpl)
-        ct = stack['cluster_template!']
-        self.ct_mgr.create.return_value = self.fake_ct
-        scheduler.TaskRunner(ct.create)()
-        self.assertEqual((ct.CREATE, ct.COMPLETE), ct.state)
-        self.assertEqual(self.fake_ct.id, ct.resource_id)
-        name = self.ct_mgr.create.call_args[1]['name']
-        self.assertIn('-clustertemplate-', name)
-
-    def test_ct_show_resource(self):
-        ct = self._create_ct(self.t)
-        self.ct_mgr.get.return_value = self.fake_ct
-        self.assertEqual({"cluster-template": "info"}, ct.FnGetAtt('show'))
-        self.ct_mgr.get.assert_called_once_with('some_ct_id')
-
-    def test_update(self):
-        ct = self._create_ct(self.t)
-        rsrc_defn = self.stack.t.resource_definitions(self.stack)[
-            'cluster-template']
-        rsrc_defn['Properties']['plugin_name'] = 'hdp'
-        rsrc_defn['Properties']['hadoop_version'] = '1.3.2'
-        scheduler.TaskRunner(ct.update, rsrc_defn)()
-        args = {
-            'name': 'test-cluster-template',
-            'plugin_name': 'hdp',
-            'hadoop_version': '1.3.2',
-            'description': '',
-            'default_image_id': None,
-            'net_id': 'some_network_id',
-            'anti_affinity': None,
-            'node_groups': None,
-            'cluster_configs': None,
-            'use_autoconfig': None
-        }
-        self.ct_mgr.update.assert_called_once_with('some_ct_id', **args)
-        self.assertEqual((ct.UPDATE, ct.COMPLETE), ct.state)
