@@ -14,17 +14,17 @@
 import datetime
 
 from keystoneclient import exceptions as kc_exceptions
-from oslo.config import cfg
+from oslo_config import cfg
 
 from heat.common import exception
 from heat.common import template_format
-from heat.db import api as db_api
 from heat.engine import parser
 from heat.engine import resource
+from heat.engine.resources import stack_user
 from heat.engine import scheduler
-from heat.engine import stack_user
 from heat.engine import template
-from heat.tests.common import HeatTestCase
+from heat.objects import resource_data as resource_data_object
+from heat.tests import common
 from heat.tests import fakes
 from heat.tests import generic_resource
 from heat.tests import utils
@@ -43,7 +43,7 @@ test_template_signal = '''
 '''
 
 
-class SignalTest(HeatTestCase):
+class SignalTest(common.HeatTestCase):
 
     def setUp(self):
         super(SignalTest, self).setUp()
@@ -86,7 +86,7 @@ class SignalTest(HeatTestCase):
         self.stack.create()
 
         rsrc = self.stack['signal_handler']
-        rs_data = db_api.resource_data_get_all(rsrc)
+        rs_data = resource_data_object.ResourceData.get_all(rsrc)
         self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
         self.assertIn('Failed', rsrc.status_reason)
         self.assertEqual('1234', rs_data.get('user_id'))
@@ -94,12 +94,12 @@ class SignalTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_resource_data(self):
-        self.stack = self.create_stack(stack_name='resource_data_test',
-                                       stub=False)
         self.stub_keystoneclient(
             access='anaccesskey',
             secret='verysecret',
             credential_id='mycredential')
+        self.stack = self.create_stack(stack_name='resource_data_test',
+                                       stub=False)
         self.m.ReplayAll()
 
         self.stack.create()
@@ -108,7 +108,7 @@ class SignalTest(HeatTestCase):
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
 
         # Ensure the resource data has been stored correctly
-        rs_data = db_api.resource_data_get_all(rsrc)
+        rs_data = resource_data_object.ResourceData.get_all(rsrc)
         self.assertEqual('mycredential', rs_data.get('credential_id'))
         self.assertEqual('anaccesskey', rs_data.get('access_key'))
         self.assertEqual('verysecret', rs_data.get('secret_key'))
@@ -129,16 +129,17 @@ class SignalTest(HeatTestCase):
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
 
         # Ensure the resource data has been stored correctly
-        rs_data = db_api.resource_data_get_all(rsrc)
+        rs_data = resource_data_object.ResourceData.get_all(rsrc)
         self.assertEqual('1234', rs_data.get('user_id'))
         self.assertEqual('1234', rsrc.resource_id)
         self.assertEqual('1234', rsrc._get_user_id())
 
         # Check user id can still be fetched from resource_id
         # if the resource data is not there.
-        db_api.resource_data_delete(rsrc, 'user_id')
+        resource_data_object.ResourceData.delete(rsrc, 'user_id')
         self.assertRaises(
-            exception.NotFound, db_api.resource_data_get, rsrc, 'user_id')
+            exception.NotFound, resource_data_object.ResourceData.get_val,
+            rsrc, 'user_id')
         self.assertEqual('1234', rsrc._get_user_id())
         self.m.VerifyAll()
 
@@ -242,6 +243,33 @@ class SignalTest(HeatTestCase):
 
         self.m.VerifyAll()
 
+    def test_signal_no_action(self):
+        test_d = {'Data': 'foo', 'Reason': 'bar',
+                  'Status': 'SUCCESS', 'UniqueId': '123'}
+
+        self.stack = self.create_stack()
+        self.stack.create()
+
+        # mock a NoActionRequired from handle_signal()
+        self.m.StubOutWithMock(generic_resource.SignalResource,
+                               'handle_signal')
+        generic_resource.SignalResource.handle_signal(test_d).AndRaise(
+            resource.NoActionRequired())
+
+        # _add_event should not be called.
+        self.m.StubOutWithMock(generic_resource.SignalResource,
+                               '_add_event')
+
+        self.m.ReplayAll()
+
+        rsrc = self.stack['signal_handler']
+        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
+        self.assertTrue(rsrc.requires_deferred_auth)
+
+        rsrc.signal(details=test_d)
+
+        self.m.VerifyAll()
+
     def test_signal_different_reason_types(self):
         self.stack = self.create_stack()
         self.stack.create()
@@ -267,13 +295,13 @@ class SignalTest(HeatTestCase):
         self.m.StubOutWithMock(generic_resource.SignalResource,
                                '_add_event')
         generic_resource.SignalResource._add_event(
-            'signal', 'COMPLETE', ceilo_expected).AndReturn(None)
+            'SIGNAL', 'COMPLETE', ceilo_expected).AndReturn(None)
         generic_resource.SignalResource._add_event(
-            'signal', 'COMPLETE', watch_expected).AndReturn(None)
+            'SIGNAL', 'COMPLETE', watch_expected).AndReturn(None)
         generic_resource.SignalResource._add_event(
-            'signal', 'COMPLETE', str_expected).AndReturn(None)
+            'SIGNAL', 'COMPLETE', str_expected).AndReturn(None)
         generic_resource.SignalResource._add_event(
-            'signal', 'COMPLETE', none_expected).AndReturn(None)
+            'SIGNAL', 'COMPLETE', none_expected).AndReturn(None)
 
         self.m.ReplayAll()
 
@@ -301,7 +329,7 @@ class SignalTest(HeatTestCase):
         self.m.StubOutWithMock(generic_resource.SignalResource,
                                '_add_event')
         generic_resource.SignalResource._add_event(
-            'signal', 'COMPLETE', 'Signal: %s' % ret_expected).AndReturn(None)
+            'SIGNAL', 'COMPLETE', 'Signal: %s' % ret_expected).AndReturn(None)
         self.m.ReplayAll()
 
         rsrc.signal(details=signal_details)
