@@ -24,6 +24,7 @@ import six
 from heat.common import context
 from heat.common import exception
 from heat.common import template_format
+from heat.db import api as db_api
 from heat.engine.clients.os import keystone
 from heat.engine.clients.os import nova
 from heat.engine import environment
@@ -52,6 +53,8 @@ class StackTest(common.HeatTestCase):
 
         resource._register_class('GenericResourceType',
                                  generic_rsrc.GenericResource)
+        resource._register_class('StackResourceType',
+                                 generic_rsrc.StackResourceType)
         resource._register_class('ResourceWithPropsType',
                                  generic_rsrc.ResourceWithProps)
         resource._register_class('ResWithComplexPropsAndAttrs',
@@ -170,45 +173,33 @@ class StackTest(common.HeatTestCase):
     def test_total_resources_empty(self):
         self.stack = stack.Stack(self.ctx, 'test_stack', self.tmpl,
                                  status_reason='flimflam')
+        self.stack.store()
+        self.assertEqual(0, self.stack.total_resources(self.stack.id))
         self.assertEqual(0, self.stack.total_resources())
 
-    def test_total_resources_generic(self):
+    def test_total_resources_not_found(self):
+        self.stack = stack.Stack(self.ctx, 'test_stack', self.tmpl,
+                                 status_reason='flimflam')
+
+        self.assertEqual(0, self.stack.total_resources('1234'))
+
+    @mock.patch.object(db_api, 'stack_count_total_resources')
+    def test_total_resources_generic(self, sctr):
         tpl = {'HeatTemplateFormatVersion': '2012-12-12',
                'Resources':
                {'A': {'Type': 'GenericResourceType'}}}
         self.stack = stack.Stack(self.ctx, 'test_stack',
                                  template.Template(tpl),
                                  status_reason='blarg')
-        self.assertEqual(1, self.stack.total_resources())
-
-    def test_total_resources_nested_ok(self):
-        tpl = {'HeatTemplateFormatVersion': '2012-12-12',
-               'Resources':
-               {'A': {'Type': 'GenericResourceType'}}}
-        self.stack = stack.Stack(self.ctx, 'test_stack',
-                                 template.Template(tpl),
-                                 status_reason='blarg')
-
-        self.stack['A'].nested = mock.Mock()
-        self.stack['A'].nested.return_value.total_resources.return_value = 3
-        self.assertEqual(4, self.stack.total_resources())
-
-    def test_total_resources_nested_not_found(self):
-        tpl = {'HeatTemplateFormatVersion': '2012-12-12',
-               'Resources':
-               {'A': {'Type': 'GenericResourceType'}}}
-        self.stack = stack.Stack(self.ctx, 'test_stack',
-                                 template.Template(tpl),
-                                 status_reason='blarg')
-
-        self.stack['A'].nested = mock.Mock(
-            side_effect=exception.NotFound('gone'))
+        self.stack.store()
+        sctr.return_value = 1
+        self.assertEqual(1, self.stack.total_resources(self.stack.id))
         self.assertEqual(1, self.stack.total_resources())
 
     def test_iter_resources(self):
         tpl = {'HeatTemplateFormatVersion': '2012-12-12',
                'Resources':
-               {'A': {'Type': 'GenericResourceType'},
+               {'A': {'Type': 'StackResourceType'},
                 'B': {'Type': 'GenericResourceType'}}}
         self.stack = stack.Stack(self.ctx, 'test_stack',
                                  template.Template(tpl),
@@ -231,42 +222,6 @@ class StackTest(common.HeatTestCase):
         all_resources = list(self.stack.iter_resources(1))
         self.assertEqual(5, len(all_resources))
 
-    def test_root_stack_no_parent(self):
-        tpl = {'HeatTemplateFormatVersion': '2012-12-12',
-               'Resources':
-               {'A': {'Type': 'GenericResourceType'}}}
-        self.stack = stack.Stack(self.ctx, 'test_stack',
-                                 template.Template(tpl),
-                                 status_reason='blarg')
-
-        self.assertEqual(self.stack, self.stack.root_stack)
-
-    def test_root_stack_parent_no_stack(self):
-        tpl = {'HeatTemplateFormatVersion': '2012-12-12',
-               'Resources':
-               {'A': {'Type': 'GenericResourceType'}}}
-        self.stack = stack.Stack(self.ctx, 'test_stack',
-                                 template.Template(tpl),
-                                 status_reason='blarg',
-                                 parent_resource='parent')
-
-        parent_resource = mock.Mock()
-        parent_resource.stack = None
-        self.stack._parent_stack = dict(parent=parent_resource)
-        self.assertEqual(self.stack, self.stack.root_stack)
-
-    def test_root_stack_with_parent(self):
-        tpl = {'HeatTemplateFormatVersion': '2012-12-12',
-               'Resources':
-               {'A': {'Type': 'GenericResourceType'}}}
-        stk = stack.Stack(self.ctx, 'test_stack', template.Template(tpl),
-                          status_reason='blarg', parent_resource='parent')
-
-        parent_resource = mock.Mock()
-        parent_resource.stack.root_stack = 'test value'
-        stk._parent_stack = dict(parent=parent_resource)
-        self.assertEqual('test value', stk.root_stack)
-
     def test_load_parent_resource(self):
         self.stack = stack.Stack(self.ctx, 'load_parent_resource', self.tmpl,
                                  parent_resource='parent')
@@ -1593,7 +1548,7 @@ class StackTest(common.HeatTestCase):
         self.assertIn('The specified reference "resource" '
                       '(in unknown) is incorrect.', six.text_type(ex))
 
-    def test_incorrect_outputs_cfn_empty_output(self):
+    def test_incorrect_outputs_cfn_missing_value(self):
         tmpl = template_format.parse("""
         HeatTemplateFormatVersion: '2012-12-12'
         Resources:
@@ -1603,6 +1558,7 @@ class StackTest(common.HeatTestCase):
               Foo: abc
         Outputs:
           Resource_attr:
+            Description: the attr
         """)
         self.stack = stack.Stack(self.ctx, 'stack_with_correct_outputs',
                                  template.Template(tmpl))
@@ -1613,6 +1569,40 @@ class StackTest(common.HeatTestCase):
         self.assertIn('Each Output must contain a Value key.',
                       six.text_type(ex))
 
+    def test_incorrect_outputs_cfn_empty_value(self):
+        tmpl = template_format.parse("""
+        HeatTemplateFormatVersion: '2012-12-12'
+        Resources:
+          AResource:
+            Type: ResourceWithPropsType
+            Properties:
+              Foo: abc
+        Outputs:
+          Resource_attr:
+            Value: ''
+        """)
+        self.stack = stack.Stack(self.ctx, 'stack_with_correct_outputs',
+                                 template.Template(tmpl))
+
+        self.assertIsNone(self.stack.validate())
+
+    def test_incorrect_outputs_cfn_none_value(self):
+        tmpl = template_format.parse("""
+        HeatTemplateFormatVersion: '2012-12-12'
+        Resources:
+          AResource:
+            Type: ResourceWithPropsType
+            Properties:
+              Foo: abc
+        Outputs:
+          Resource_attr:
+            Value:
+        """)
+        self.stack = stack.Stack(self.ctx, 'stack_with_correct_outputs',
+                                 template.Template(tmpl))
+
+        self.assertIsNone(self.stack.validate())
+
     def test_incorrect_outputs_cfn_string_data(self):
         tmpl = template_format.parse("""
         HeatTemplateFormatVersion: '2012-12-12'
