@@ -12,20 +12,20 @@
 #    under the License.
 
 import mock
-from oslo import messaging
+import oslo_messaging as messaging
 
 from heat.common import exception
-from heat.db import api as db_api
 from heat.engine import stack_lock
-from heat.tests.common import HeatTestCase
+from heat.objects import stack_lock as stack_lock_object
+from heat.tests import common
 from heat.tests import utils
 
 
-class StackLockTest(HeatTestCase):
+class StackLockTest(common.HeatTestCase):
     def setUp(self):
         super(StackLockTest, self).setUp()
         self.context = utils.dummy_context()
-        self.stack = self.m.CreateMockAnything()
+        self.stack = mock.MagicMock()
         self.stack.id = "aae01f2d-52ae-47ac-8a0d-3fde3d220fea"
         self.stack.name = "test_stack"
         self.stack.action = "CREATE"
@@ -35,179 +35,170 @@ class StackLockTest(HeatTestCase):
             pass
 
     def test_successful_acquire_new_lock(self):
-        self.m.StubOutWithMock(db_api, "stack_lock_create")
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn(None)
-
-        self.m.ReplayAll()
+        mock_create = self.patchobject(stack_lock_object.StackLock,
+                                       'create',
+                                       return_value=None)
 
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
         slock.acquire()
-        self.m.VerifyAll()
+
+        mock_create.assert_called_once_with(self.stack.id, self.engine_id)
 
     def test_failed_acquire_existing_lock_current_engine(self):
-        self.m.StubOutWithMock(db_api, "stack_lock_create")
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn(self.engine_id)
-
-        self.m.ReplayAll()
+        mock_create = self.patchobject(stack_lock_object.StackLock,
+                                       'create',
+                                       return_value=self.engine_id)
 
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
+
         self.assertRaises(exception.ActionInProgress, slock.acquire)
-        self.m.VerifyAll()
+        mock_create.assert_called_once_with(self.stack.id, self.engine_id)
 
     def test_successful_acquire_existing_lock_engine_dead(self):
-        self.m.StubOutWithMock(db_api, "stack_lock_create")
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn("fake-engine-id")
-
-        self.m.StubOutWithMock(db_api, "stack_lock_steal")
-        db_api.stack_lock_steal(self.stack.id, "fake-engine-id",
-                                self.engine_id).AndReturn(None)
-
-        self.m.ReplayAll()
+        mock_create = self.patchobject(stack_lock_object.StackLock,
+                                       'create',
+                                       return_value='fake-engine-id')
+        mock_steal = self.patchobject(stack_lock_object.StackLock,
+                                      'steal',
+                                      return_value=None)
 
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
         self.patchobject(slock, 'engine_alive', return_value=False)
         slock.acquire()
-        self.m.VerifyAll()
+
+        mock_create.assert_called_once_with(self.stack.id, self.engine_id)
+        mock_steal.assert_called_once_with(self.stack.id, 'fake-engine-id',
+                                           self.engine_id)
 
     def test_failed_acquire_existing_lock_engine_alive(self):
-        self.m.StubOutWithMock(db_api, "stack_lock_create")
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn("fake-engine-id")
-
-        self.m.ReplayAll()
+        mock_create = self.patchobject(stack_lock_object.StackLock,
+                                       'create',
+                                       return_value='fake-engine-id')
 
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
         self.patchobject(slock, 'engine_alive', return_value=True)
         self.assertRaises(exception.ActionInProgress, slock.acquire)
-        self.m.VerifyAll()
+
+        mock_create.assert_called_once_with(self.stack.id, self.engine_id)
 
     def test_failed_acquire_existing_lock_engine_dead(self):
-        self.m.StubOutWithMock(db_api, "stack_lock_create")
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn("fake-engine-id")
-
-        self.m.StubOutWithMock(db_api, "stack_lock_steal")
-        db_api.stack_lock_steal(
-            self.stack.id, "fake-engine-id",
-            self.engine_id).AndReturn("fake-engine-id2")
-
-        self.m.ReplayAll()
+        mock_create = self.patchobject(stack_lock_object.StackLock,
+                                       'create',
+                                       return_value='fake-engine-id')
+        mock_steal = self.patchobject(stack_lock_object.StackLock,
+                                      'steal',
+                                      return_value='fake-engine-id2')
 
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
         self.patchobject(slock, 'engine_alive', return_value=False)
         self.assertRaises(exception.ActionInProgress, slock.acquire)
-        self.m.VerifyAll()
+
+        mock_create.assert_called_once_with(self.stack.id, self.engine_id)
+        mock_steal.assert_called_once_with(self.stack.id, 'fake-engine-id',
+                                           self.engine_id)
 
     def test_successful_acquire_with_retry(self):
-        self.m.StubOutWithMock(db_api, "stack_lock_create")
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn("fake-engine-id")
-
-        self.m.StubOutWithMock(db_api, "stack_lock_steal")
-        db_api.stack_lock_steal(
-            self.stack.id, "fake-engine-id", self.engine_id).AndReturn(True)
-
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn("fake-engine-id")
-
-        db_api.stack_lock_steal(
-            self.stack.id, "fake-engine-id", self.engine_id).AndReturn(None)
-
-        self.m.ReplayAll()
+        mock_create = self.patchobject(stack_lock_object.StackLock,
+                                       'create',
+                                       return_value='fake-engine-id')
+        mock_steal = self.patchobject(stack_lock_object.StackLock,
+                                      'steal',
+                                      side_effect=[True, None])
 
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
         self.patchobject(slock, 'engine_alive', return_value=False)
         slock.acquire()
-        self.m.VerifyAll()
+
+        mock_create.assert_has_calls(
+            [mock.call(self.stack.id, self.engine_id)] * 2)
+        mock_steal.assert_has_calls(
+            [mock.call(self.stack.id, 'fake-engine-id', self.engine_id)] * 2)
 
     def test_failed_acquire_one_retry_only(self):
-        self.m.StubOutWithMock(db_api, "stack_lock_create")
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn("fake-engine-id")
-
-        self.m.StubOutWithMock(db_api, "stack_lock_steal")
-        db_api.stack_lock_steal(
-            self.stack.id, "fake-engine-id", self.engine_id).AndReturn(True)
-
-        db_api.stack_lock_create(
-            self.stack.id, self.engine_id).AndReturn("fake-engine-id")
-
-        db_api.stack_lock_steal(
-            self.stack.id, "fake-engine-id", self.engine_id).AndReturn(True)
-
-        self.m.ReplayAll()
+        mock_create = self.patchobject(stack_lock_object.StackLock,
+                                       'create',
+                                       return_value='fake-engine-id')
+        mock_steal = self.patchobject(stack_lock_object.StackLock,
+                                      'steal',
+                                      return_value=True)
 
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
         self.patchobject(slock, 'engine_alive', return_value=False)
         self.assertRaises(exception.ActionInProgress, slock.acquire)
-        self.m.VerifyAll()
+
+        mock_create.assert_has_calls(
+            [mock.call(self.stack.id, self.engine_id)] * 2)
+        mock_steal.assert_has_calls(
+            [mock.call(self.stack.id, 'fake-engine-id', self.engine_id)] * 2)
 
     def test_thread_lock_context_mgr_exception_acquire_success(self):
-        db_api.stack_lock_create = mock.Mock(return_value=None)
-        db_api.stack_lock_release = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.create = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.release = mock.Mock(return_value=None)
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
 
         def check_thread_lock():
             with slock.thread_lock(self.stack.id):
-                self.assertEqual(1, db_api.stack_lock_create.call_count)
+                self.assertEqual(1,
+                                 stack_lock_object.StackLock.create.call_count)
                 raise self.TestThreadLockException
         self.assertRaises(self.TestThreadLockException, check_thread_lock)
-        self.assertEqual(1, db_api.stack_lock_release.call_count)
+        self.assertEqual(1, stack_lock_object.StackLock.release.call_count)
 
     def test_thread_lock_context_mgr_exception_acquire_fail(self):
-        db_api.stack_lock_create = mock.Mock(return_value=self.engine_id)
-        db_api.stack_lock_release = mock.Mock()
+        stack_lock_object.StackLock.create = mock.Mock(
+            return_value=self.engine_id)
+        stack_lock_object.StackLock.release = mock.Mock()
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
 
         def check_thread_lock():
             with slock.thread_lock(self.stack.id):
-                self.assertEqual(1, db_api.stack_lock_create.call_count)
+                self.assertEqual(1,
+                                 stack_lock_object.StackLock.create.call_count)
                 raise exception.ActionInProgress
         self.assertRaises(exception.ActionInProgress, check_thread_lock)
-        assert not db_api.stack_lock_release.called
+        assert not stack_lock_object.StackLock.release.called
 
     def test_thread_lock_context_mgr_no_exception(self):
-        db_api.stack_lock_create = mock.Mock(return_value=None)
-        db_api.stack_lock_release = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.create = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.release = mock.Mock(return_value=None)
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
         with slock.thread_lock(self.stack.id):
-            self.assertEqual(1, db_api.stack_lock_create.call_count)
-        assert not db_api.stack_lock_release.called
+            self.assertEqual(1, stack_lock_object.StackLock.create.call_count)
+        assert not stack_lock_object.StackLock.release.called
 
     def test_try_thread_lock_context_mgr_exception(self):
-        db_api.stack_lock_create = mock.Mock(return_value=None)
-        db_api.stack_lock_release = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.create = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.release = mock.Mock(return_value=None)
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
 
         def check_thread_lock():
             with slock.try_thread_lock(self.stack.id):
-                self.assertEqual(1, db_api.stack_lock_create.call_count)
+                self.assertEqual(1,
+                                 stack_lock_object.StackLock.create.call_count)
                 raise self.TestThreadLockException
         self.assertRaises(self.TestThreadLockException, check_thread_lock)
-        self.assertEqual(1, db_api.stack_lock_release.call_count)
+        self.assertEqual(1, stack_lock_object.StackLock.release.call_count)
 
     def test_try_thread_lock_context_mgr_no_exception(self):
-        db_api.stack_lock_create = mock.Mock(return_value=None)
-        db_api.stack_lock_release = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.create = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.release = mock.Mock(return_value=None)
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
         with slock.try_thread_lock(self.stack.id):
-            self.assertEqual(1, db_api.stack_lock_create.call_count)
-        assert not db_api.stack_lock_release.called
+            self.assertEqual(1, stack_lock_object.StackLock.create.call_count)
+        assert not stack_lock_object.StackLock.release.called
 
     def test_try_thread_lock_context_mgr_existing_lock(self):
-        db_api.stack_lock_create = mock.Mock(return_value=1234)
-        db_api.stack_lock_release = mock.Mock(return_value=None)
+        stack_lock_object.StackLock.create = mock.Mock(return_value=1234)
+        stack_lock_object.StackLock.release = mock.Mock(return_value=None)
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
 
         def check_thread_lock():
             with slock.try_thread_lock(self.stack.id):
-                self.assertEqual(1, db_api.stack_lock_create.call_count)
+                self.assertEqual(1,
+                                 stack_lock_object.StackLock.create.call_count)
                 raise self.TestThreadLockException
         self.assertRaises(self.TestThreadLockException, check_thread_lock)
-        assert not db_api.stack_lock_release.called
+        assert not stack_lock_object.StackLock.release.called
 
     def test_engine_alive_ok(self):
         slock = stack_lock.StackLock(self.context, self.stack, self.engine_id)
