@@ -15,6 +15,7 @@ import uuid
 
 import mock
 from oslo_config import cfg
+from oslo_messaging import exceptions as msg_exceptions
 import six
 
 from heat.common import exception
@@ -197,6 +198,11 @@ class StackResourceTest(common.HeatTestCase):
         nest.return_value.prepare_abandon.assert_called_once_with()
         self.assertEqual({'X': 'Y'}, ret)
 
+    def test_nested_abandon_stack_not_found(self):
+        self.parent_resource.nested = mock.MagicMock(return_value=None)
+        ret = self.parent_resource.prepare_abandon()
+        self.assertEqual({}, ret)
+
     def test_implementation_signature(self):
         self.parent_resource.child_template = mock.Mock(
             return_value=self.simple_template)
@@ -421,7 +427,7 @@ class StackResourceTest(common.HeatTestCase):
                 'Resources': [1]}
         template = stack_resource.template.Template(tmpl)
         root_resources = mock.Mock(return_value=2)
-        self.parent_resource.stack.root_stack.total_resources = root_resources
+        self.parent_resource.stack.total_resources = root_resources
 
         self.assertRaises(exception.RequestLimitExceeded,
                           self.parent_resource._validate_nested_resources,
@@ -459,10 +465,11 @@ class StackResourceTest(common.HeatTestCase):
         parser.Stack.load(self.parent_resource.context,
                           self.parent_resource.resource_id,
                           show_deleted=False,
-                          force_reload=False).AndReturn(None)
+                          force_reload=False).AndRaise(
+            exception.NotFound)
         self.m.ReplayAll()
 
-        self.assertRaises(exception.NotFound, self.parent_resource.nested)
+        self.assertIsNone(self.parent_resource.nested())
         self.m.VerifyAll()
 
     def test_load_nested_cached(self):
@@ -489,26 +496,32 @@ class StackResourceTest(common.HeatTestCase):
         parser.Stack.load(self.parent_resource.context,
                           self.parent_resource.resource_id,
                           show_deleted=False,
-                          force_reload=True).AndReturn(None)
+                          force_reload=True).AndRaise(
+            exception.NotFound)
         self.m.ReplayAll()
-        self.assertRaises(exception.NotFound, self.parent_resource.nested,
-                          force_reload=True)
+        self.assertIsNone(self.parent_resource.nested(force_reload=True))
         self.m.VerifyAll()
 
-    def test_delete_nested_not_found_nested_stack(self):
-
+    def test_delete_nested_none_nested_stack(self):
         self.parent_resource._nested = None
+        self.assertIsNone(self.parent_resource.delete_nested())
+
+    def test_delete_nested_not_found_nested_stack(self):
+        self.parent_resource._nested = mock.MagicMock()
         rpcc = mock.Mock()
         self.parent_resource.rpc_client = rpcc
         rpcc.return_value.delete_stack = mock.Mock(
             side_effect=exception.NotFound())
         self.assertIsNone(self.parent_resource.delete_nested())
+        rpcc.return_value.delete_stack.assert_called_once_with(
+            self.parent_resource.context, mock.ANY)
 
-    def test_need_update_in_failed_state_for_nested_resource(self):
+    def test_need_update_for_nested_resource(self):
         """
-        The resource in any state and has nested stack,
+        The resource in Create or Update state and has nested stack,
         should need update.
         """
+        self.parent_resource.action = self.parent_resource.CREATE
         need_update = self.parent_resource._needs_update(
             self.parent_resource.t,
             self.parent_resource.t,
@@ -518,13 +531,47 @@ class StackResourceTest(common.HeatTestCase):
 
         self.assertEqual(True, need_update)
 
+    def test_need_update_in_failed_state_for_nested_resource(self):
+        """
+        The resource in failed state and has no nested stack,
+        should need update with UpdateReplace.
+        """
+        self.parent_resource.state_set(self.parent_resource.INIT,
+                                       self.parent_resource.FAILED)
+        self.parent_resource._nested = None
+        self.assertRaises(resource.UpdateReplace,
+                          self.parent_resource._needs_update,
+                          self.parent_resource.t,
+                          self.parent_resource.t,
+                          self.parent_resource.properties,
+                          self.parent_resource.properties,
+                          self.parent_resource)
+
+    def test_need_update_in_init_complete_state_for_nested_resource(self):
+        """
+        The resource in failed state and has no nested stack,
+        should need update with UpdateReplace.
+        """
+        self.parent_resource.state_set(self.parent_resource.INIT,
+                                       self.parent_resource.COMPLETE)
+        self.parent_resource._nested = None
+        self.assertRaises(resource.UpdateReplace,
+                          self.parent_resource._needs_update,
+                          self.parent_resource.t,
+                          self.parent_resource.t,
+                          self.parent_resource.properties,
+                          self.parent_resource.properties,
+                          self.parent_resource)
+
 
 class StackResourceLimitTest(common.HeatTestCase):
     scenarios = [
-        ('1', dict(root=3, templ=4, nested=0, max=10, error=False)),
-        ('2', dict(root=3, templ=8, nested=0, max=10, error=True)),
-        ('3', dict(root=3, templ=8, nested=2, max=10, error=False)),
-        ('4', dict(root=3, templ=12, nested=2, max=10, error=True))]
+        ('3_4_0', dict(root=3, templ=4, nested=0, max=10, error=False)),
+        ('3_8_0', dict(root=3, templ=8, nested=0, max=10, error=True)),
+        ('3_8_2', dict(root=3, templ=8, nested=2, max=10, error=True)),
+        ('3_5_2', dict(root=3, templ=5, nested=2, max=10, error=False)),
+        ('3_6_2', dict(root=3, templ=6, nested=2, max=10, error=True)),
+        ('3_12_2', dict(root=3, templ=12, nested=2, max=10, error=True))]
 
     def setUp(self):
         super(StackResourceLimitTest, self).setUp()
@@ -545,14 +592,9 @@ class StackResourceLimitTest(common.HeatTestCase):
                                    self.parent_stack)
 
     def test_resource_limit(self):
-        # mock nested resources
-        nested = mock.MagicMock()
-        nested.resources = range(self.nested)
-        self.res.nested = mock.MagicMock(return_value=nested)
-
         # mock root total_resources
-        self.res.stack.root_stack.total_resources = mock.Mock(
-            return_value=self.root)
+        total_resources = self.root + self.nested
+        parser.Stack.total_resources = mock.Mock(return_value=total_resources)
 
         # setup the config max
         cfg.CONF.set_default('max_resources_per_stack', self.max)
@@ -761,3 +803,53 @@ class StackResourceCheckCompleteTest(common.HeatTestCase):
         self.assertFalse(complete(None))
         self.parent_resource.nested.assert_called_once_with(
             show_deleted=self.show_deleted, force_reload=True)
+
+
+class RaiseLocalException(common.HeatTestCase):
+
+    def setUp(self):
+        super(RaiseLocalException, self).setUp()
+        resource._register_class('some_magic_type',
+                                 MyStackResource)
+        self.ws_resname = "provider_resource"
+        t = templatem.Template(
+            {'HeatTemplateFormatVersion': '2012-12-12',
+             'Resources': {self.ws_resname: ws_res_snippet}})
+        self.parent_stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                                         t, stack_id=str(uuid.uuid4()),
+                                         user_creds_id='uc123',
+                                         stack_user_project_id='aprojectid')
+        resource_defns = t.resource_definitions(self.parent_stack)
+        self.parent_resource = MyStackResource('test',
+                                               resource_defns[self.ws_resname],
+                                               self.parent_stack)
+
+    def test_heat_exception(self):
+        local = exception.ResourceTypeNotFound(type_name='foo')
+        self.assertRaises(exception.ResourceTypeNotFound,
+                          self.parent_resource.raise_local_exception, local)
+
+    def test_messaging_timeout(self):
+        local = msg_exceptions.MessagingTimeout('took too long')
+        self.assertRaises(msg_exceptions.MessagingTimeout,
+                          self.parent_resource.raise_local_exception, local)
+
+    def test_remote_heat_ex(self):
+        remote_exc = exception.ResourceTypeNotFound(type_name='bar')
+        expected_msg = six.text_type(remote_exc)
+        message = (expected_msg + '\n' +
+                   'Traceback (most recent call last):\n'
+                   '  File whatever.py, line 42, in whatever\n'
+                   '    whatever()')
+        str_override = lambda self: message
+        RemoteExcClass = type(type(remote_exc).__name__ + '_Remote',
+                              (type(remote_exc),),
+                              {'__str__': str_override,
+                               '__unicode__': str_override})
+        RemoteExcClass.__module__ = 'heat.common.exception_Remote'
+        remote_exc.__class__ = RemoteExcClass
+
+        exc = self.assertRaises(exception.ResourceFailure,
+                                self.parent_resource.raise_local_exception,
+                                remote_exc)
+        self.assertIn(expected_msg, six.text_type(exc))
