@@ -48,12 +48,11 @@ class FakeTroveCluster(object):
     def __init__(self, status='ACTIVE'):
         self.name = 'cluster'
         self.id = '1189aa64-a471-4aa3-876a-9eb7d84089da'
-        self.ip = ['10.0.0.1']
+        self.ip = '10.0.0.1'
         self.instances = [
             {'id': '416b0b16-ba55-4302-bbd3-ff566032e1c1', 'status': status},
             {'id': '965ef811-7c1d-47fc-89f2-a89dfdd23ef2', 'status': status},
             {'id': '3642f41c-e8ad-4164-a089-3891bf7f2d2b', 'status': status}]
-        self.to_dict = lambda: {'attr': 'val'}
 
     def delete(self):
         pass
@@ -125,12 +124,11 @@ class TroveClusterTest(common.HeatTestCase):
 
     def test_attributes(self):
         tc = self._create_resource('cluster', self.rsrc_defn, self.stack)
-        self.assertEqual(['10.0.0.1'], tc.FnGetAtt('ip'))
+        self.assertEqual('10.0.0.1', tc.FnGetAtt('ip'))
         self.assertEqual(['416b0b16-ba55-4302-bbd3-ff566032e1c1',
                           '965ef811-7c1d-47fc-89f2-a89dfdd23ef2',
                           '3642f41c-e8ad-4164-a089-3891bf7f2d2b'],
                          tc.FnGetAtt('instances'))
-        self.assertEqual({'attr': 'val'}, tc.FnGetAtt('show'))
 
     def test_delete(self):
         tc = self._create_resource('cluster', self.rsrc_defn, self.stack)
