@@ -11,15 +11,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo.config import cfg
+from oslo_config import cfg
 
 from heat.common import exception
 from heat.common import short_id
 from heat.common import template_format
-from heat.db import api as db_api
-from heat.engine.resources import user
+from heat.engine.resources.aws.iam import user
+from heat.engine.resources.openstack.heat import access_policy as ap
 from heat.engine import scheduler
-from heat.tests.common import HeatTestCase
+from heat.objects import resource_data as resource_data_object
+from heat.tests import common
 from heat.tests import fakes
 from heat.tests import utils
 
@@ -100,10 +101,11 @@ user_policy_template = '''
 '''
 
 
-class UserTest(HeatTestCase):
+class UserTest(common.HeatTestCase):
     def setUp(self):
         super(UserTest, self).setUp()
-        self.username = 'test_stack-CfnUser-aabbcc'
+        self.stack_name = 'test_user_stack_%s' % utils.random_name()
+        self.username = '%s-CfnUser-aabbcc' % self.stack_name
         self.fc = fakes.FakeKeystoneClient(username=self.username)
         cfg.CONF.set_default('heat_stack_user_role', 'stack_user_role')
 
@@ -125,7 +127,7 @@ class UserTest(HeatTestCase):
         rsrc._store()
 
         self.m.StubOutWithMock(short_id, 'get_id')
-        short_id.get_id(rsrc.id).MultipleTimes().AndReturn('aabbcc')
+        short_id.get_id(rsrc.uuid).MultipleTimes().AndReturn('aabbcc')
 
         self.m.StubOutWithMock(fakes.FakeKeystoneClient,
                                'create_stack_domain_user')
@@ -140,7 +142,7 @@ class UserTest(HeatTestCase):
 
     def test_user(self):
         t = template_format.parse(user_template)
-        stack = utils.parse_stack(t)
+        stack = utils.parse_stack(t, stack_name=self.stack_name)
 
         rsrc = self.create_user(t, stack, 'CfnUser')
         self.assertEqual('dummy_user', rsrc.resource_id)
@@ -174,7 +176,7 @@ class UserTest(HeatTestCase):
 
     def test_user_password(self):
         t = template_format.parse(user_template_password)
-        stack = utils.parse_stack(t)
+        stack = utils.parse_stack(t, stack_name=self.stack_name)
 
         rsrc = self.create_user(t, stack, 'CfnUser', password=u'myP@ssW0rd')
         self.assertEqual('dummy_user', rsrc.resource_id)
@@ -185,7 +187,7 @@ class UserTest(HeatTestCase):
 
     def test_user_validate_policies(self):
         t = template_format.parse(user_policy_template)
-        stack = utils.parse_stack(t)
+        stack = utils.parse_stack(t, stack_name=self.stack_name)
 
         rsrc = self.create_user(t, stack, 'CfnUser')
         self.assertEqual('dummy_user', rsrc.resource_id)
@@ -221,7 +223,7 @@ class UserTest(HeatTestCase):
     def test_user_create_bad_policies(self):
         t = template_format.parse(user_policy_template)
         t['Resources']['CfnUser']['Properties']['Policies'] = ['NoExistBad']
-        stack = utils.parse_stack(t)
+        stack = utils.parse_stack(t, stack_name=self.stack_name)
         resource_name = 'CfnUser'
         resource_defns = stack.t.resource_definitions(stack)
         rsrc = user.User(resource_name,
@@ -231,14 +233,14 @@ class UserTest(HeatTestCase):
                           rsrc.handle_create)
 
     def test_user_access_allowed(self):
-        self.m.StubOutWithMock(user.AccessPolicy, 'access_allowed')
-        user.AccessPolicy.access_allowed('a_resource').AndReturn(True)
-        user.AccessPolicy.access_allowed('b_resource').AndReturn(False)
+        self.m.StubOutWithMock(ap.AccessPolicy, 'access_allowed')
+        ap.AccessPolicy.access_allowed('a_resource').AndReturn(True)
+        ap.AccessPolicy.access_allowed('b_resource').AndReturn(False)
 
         self.m.ReplayAll()
 
         t = template_format.parse(user_policy_template)
-        stack = utils.parse_stack(t)
+        stack = utils.parse_stack(t, stack_name=self.stack_name)
 
         rsrc = self.create_user(t, stack, 'CfnUser')
         self.assertEqual('dummy_user', rsrc.resource_id)
@@ -250,16 +252,16 @@ class UserTest(HeatTestCase):
         self.m.VerifyAll()
 
     def test_user_access_allowed_ignorepolicy(self):
-        self.m.StubOutWithMock(user.AccessPolicy, 'access_allowed')
-        user.AccessPolicy.access_allowed('a_resource').AndReturn(True)
-        user.AccessPolicy.access_allowed('b_resource').AndReturn(False)
+        self.m.StubOutWithMock(ap.AccessPolicy, 'access_allowed')
+        ap.AccessPolicy.access_allowed('a_resource').AndReturn(True)
+        ap.AccessPolicy.access_allowed('b_resource').AndReturn(False)
 
         self.m.ReplayAll()
 
         t = template_format.parse(user_policy_template)
         t['Resources']['CfnUser']['Properties']['Policies'] = [
             'WebServerAccessPolicy', {'an_ignored': 'policy'}]
-        stack = utils.parse_stack(t)
+        stack = utils.parse_stack(t, stack_name=self.stack_name)
 
         rsrc = self.create_user(t, stack, 'CfnUser')
         self.assertEqual('dummy_user', rsrc.resource_id)
@@ -271,7 +273,7 @@ class UserTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-class AccessKeyTest(HeatTestCase):
+class AccessKeyTest(common.HeatTestCase):
     def setUp(self):
         super(AccessKeyTest, self).setUp()
         self.username = utils.PhysName('test_stack', 'CfnUser')
@@ -319,7 +321,7 @@ class AccessKeyTest(HeatTestCase):
                          rsrc._secret)
 
         # Ensure the resource data has been stored correctly
-        rs_data = db_api.resource_data_get_all(rsrc)
+        rs_data = resource_data_object.ResourceData.get_all(rsrc)
         self.assertEqual(self.fc.secret, rs_data.get('secret_key'))
         self.assertEqual(self.fc.credential_id, rs_data.get('credential_id'))
         self.assertEqual(2, len(rs_data.keys()))
@@ -353,9 +355,9 @@ class AccessKeyTest(HeatTestCase):
         # Delete the resource data for secret_key, to test that existing
         # stacks which don't have the resource_data stored will continue
         # working via retrieving the keypair from keystone
-        db_api.resource_data_delete(rsrc, 'credential_id')
-        db_api.resource_data_delete(rsrc, 'secret_key')
-        rs_data = db_api.resource_data_get_all(rsrc)
+        resource_data_object.ResourceData.delete(rsrc, 'credential_id')
+        resource_data_object.ResourceData.delete(rsrc, 'secret_key')
+        rs_data = resource_data_object.ResourceData.get_all(rsrc)
         self.assertEqual(0, len(rs_data.keys()))
 
         rsrc._secret = None
@@ -389,7 +391,7 @@ class AccessKeyTest(HeatTestCase):
         self.m.VerifyAll()
 
 
-class AccessPolicyTest(HeatTestCase):
+class AccessPolicyTest(common.HeatTestCase):
 
     def test_accesspolicy_create_ok(self):
         t = template_format.parse(user_policy_template)
@@ -397,9 +399,9 @@ class AccessPolicyTest(HeatTestCase):
 
         resource_name = 'WebServerAccessPolicy'
         resource_defns = stack.t.resource_definitions(stack)
-        rsrc = user.AccessPolicy(resource_name,
-                                 resource_defns[resource_name],
-                                 stack)
+        rsrc = ap.AccessPolicy(resource_name,
+                               resource_defns[resource_name],
+                               stack)
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
 
@@ -410,9 +412,9 @@ class AccessPolicyTest(HeatTestCase):
         stack = utils.parse_stack(t)
 
         resource_defns = stack.t.resource_definitions(stack)
-        rsrc = user.AccessPolicy(resource_name,
-                                 resource_defns[resource_name],
-                                 stack)
+        rsrc = ap.AccessPolicy(resource_name,
+                               resource_defns[resource_name],
+                               stack)
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
 
@@ -431,9 +433,9 @@ class AccessPolicyTest(HeatTestCase):
         stack = utils.parse_stack(t)
 
         resource_defns = stack.t.resource_definitions(stack)
-        rsrc = user.AccessPolicy(resource_name,
-                                 resource_defns[resource_name],
-                                 stack)
+        rsrc = ap.AccessPolicy(resource_name,
+                               resource_defns[resource_name],
+                               stack)
         self.assertTrue(rsrc.access_allowed('WikiDatabase'))
         self.assertFalse(rsrc.access_allowed('NotWikiDatabase'))
         self.assertFalse(rsrc.access_allowed(None))
