@@ -15,14 +15,14 @@
 import datetime
 
 import mox
-from oslo.utils import timeutils
+from oslo_utils import timeutils
 
 from heat.common import exception
-from heat.db import api as db_api
 from heat.engine import parser
 from heat.engine import template
 from heat.engine import watchrule
-from heat.tests.common import HeatTestCase
+from heat.objects import watch_rule
+from heat.tests import common
 from heat.tests import utils
 
 
@@ -37,7 +37,7 @@ class DummyAction(object):
     signal = "DummyAction"
 
 
-class WatchRuleTest(HeatTestCase):
+class WatchRuleTest(common.HeatTestCase):
     stack_id = None
 
     def setUpDatabase(self):
@@ -75,8 +75,8 @@ class WatchRuleTest(HeatTestCase):
         if action_expected:
             dummy_action = DummyAction()
             self.m.StubOutWithMock(parser.Stack, 'resource_by_refid')
-            parser.Stack.resource_by_refid(mox.IgnoreArg()).\
-                MultipleTimes().AndReturn(dummy_action)
+            parser.Stack.resource_by_refid(
+                mox.IgnoreArg()).MultipleTimes().AndReturn(dummy_action)
 
         self.m.ReplayAll()
 
@@ -321,7 +321,7 @@ class WatchRuleTest(HeatTestCase):
                                       stack_id=self.stack_id, rule=rule)
         self.wr.store()
 
-        dbwr = db_api.watch_rule_get_by_name(self.ctx, 'storetest')
+        dbwr = watch_rule.WatchRule.get_by_name(self.ctx, 'storetest')
         self.assertIsNotNone(dbwr)
         self.assertEqual('storetest', dbwr.name)
         self.assertEqual(watchrule.WatchRule.NODATA, dbwr.state)
@@ -411,6 +411,34 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(self.wr.SUSPENDED, self.wr.state)
         self.assertEqual([], actions)
 
+    def test_evaluate_ceilometer_controlled(self):
+        rule = {'EvaluationPeriods': '1',
+                'MetricName': 'test_metric',
+                'Period': '300',
+                'Statistic': 'Maximum',
+                'ComparisonOperator': 'GreaterThanOrEqualToThreshold',
+                'Threshold': '30'}
+
+        now = timeutils.utcnow()
+        timeutils.set_time_override(now)
+        self.addCleanup(timeutils.clear_time_override)
+
+        # Now data breaches Threshold, but we're suspended
+        last = now - datetime.timedelta(seconds=300)
+        data = WatchData(35, now - datetime.timedelta(seconds=150))
+        self.wr = watchrule.WatchRule(context=self.ctx,
+                                      watch_name="testwatch",
+                                      rule=rule,
+                                      watch_data=[data],
+                                      stack_id=self.stack_id,
+                                      last_evaluated=last)
+
+        self.wr.state_set(self.wr.CEILOMETER_CONTROLLED)
+
+        actions = self.wr.evaluate()
+        self.assertEqual(self.wr.CEILOMETER_CONTROLLED, self.wr.state)
+        self.assertEqual([], actions)
+
     def test_rule_actions_alarm_normal(self):
         rule = {'EvaluationPeriods': '1',
                 'MetricName': 'test_metric',
@@ -577,6 +605,52 @@ class WatchRuleTest(HeatTestCase):
         self.assertEqual(['DummyAction'], actions)
         self.m.VerifyAll()
 
+    def test_to_ceilometer(self):
+
+        rule = {u'EvaluationPeriods': u'1',
+                u'AlarmDescription': u'test alarm',
+                u'Period': u'300',
+                u'ComparisonOperator': u'GreaterThanThreshold',
+                u'Statistic': u'SampleCount',
+                u'Threshold': u'2',
+                u'MetricName': u'CreateDataMetric'}
+        testdata = {u'CreateDataMetric': {"Unit": "Counter", "Value": "1"}}
+        sample = dict()
+        sample['counter_type'] = 'gauge'
+
+        for k, d in iter(testdata.items()):
+            if k == 'Namespace':
+                continue
+            sample['counter_name'] = k
+            sample['counter_volume'] = d['Value']
+            sample['counter_unit'] = d['Unit']
+            dims = d.get('Dimensions', {})
+            if isinstance(dims, list):
+                dims = dims[0]
+            sample['resource_metadata'] = dims
+            sample['resource_id'] = dims.get('InstanceId')
+
+        self.wr = watchrule.WatchRule(context=self.ctx,
+                                      watch_name='create_data_test',
+                                      stack_id=self.stack_id, rule=rule)
+        self.wr.store()
+
+        self.m.StubOutWithMock(self.wr.context.clients.client('ceilometer').
+                               samples, 'create', True)
+
+        # fake samples.create callback
+        def fake_create(counter_type, counter_name, counter_volume,
+                        counter_unit, resource_metadata, resource_id):
+            pass
+
+        self.wr.context.clients.client('ceilometer').samples.\
+            create(**sample).MultipleTimes().WithSideEffects(fake_create)
+        self.m.ReplayAll()
+        try:
+            self.wr._to_ceilometer(testdata)
+        except mox.UnexpectedMethodCallError:
+            raise KeyError("Error input parameter")
+
     def test_create_watch_data(self):
         rule = {u'EvaluationPeriods': u'1',
                 u'AlarmDescription': u'test alarm',
@@ -596,8 +670,9 @@ class WatchRuleTest(HeatTestCase):
                                       "Dimensions": []}}
         self.wr.create_watch_data(data)
 
-        dbwr = db_api.watch_rule_get_by_name(self.ctx, 'create_data_test')
-        self.assertEqual(data, dbwr.watch_data[0].data)
+        obj_wr = watch_rule.WatchRule.get_by_name(self.ctx, 'create_data_test')
+        obj_wds = [wd for wd in obj_wr.watch_data]
+        self.assertEqual(data, obj_wds[0].data)
 
         # Note, would be good to write another datapoint and check it
         # but sqlite seems to not interpret the backreference correctly
@@ -626,8 +701,9 @@ class WatchRuleTest(HeatTestCase):
                                       "Dimensions": []}}
         self.wr.create_watch_data(data)
 
-        dbwr = db_api.watch_rule_get_by_name(self.ctx, 'create_data_test')
-        self.assertEqual([], dbwr.watch_data)
+        obj_wr = watch_rule.WatchRule.get_by_name(self.ctx, 'create_data_test')
+        obj_wds = [wd for wd in obj_wr.watch_data]
+        self.assertEqual([], obj_wds)
 
     def test_create_watch_data_match(self):
         rule = {u'EvaluationPeriods': u'1',
