@@ -13,19 +13,16 @@
 import copy
 import json
 
-from heatclient import exc
 from oslo_log import log as logging
-import six
 from testtools import matchers
 
 from heat_integrationtests.common import test
-from heat_integrationtests.functional import functional_base
 
 
 LOG = logging.getLogger(__name__)
 
 
-class AutoscalingGroupTest(functional_base.FunctionalTestsBase):
+class AutoscalingGroupTest(test.HeatIntegrationTest):
 
     template = '''
 {
@@ -34,8 +31,7 @@ class AutoscalingGroupTest(functional_base.FunctionalTestsBase):
   "Parameters" : {"size": {"Type": "String", "Default": "1"},
                   "AZ": {"Type": "String", "Default": "nova"},
                   "image": {"Type": "String"},
-                  "flavor": {"Type": "String"},
-                  "user_data": {"Type": "String", "Default": "jsconfig data"}},
+                  "flavor": {"Type": "String"}},
   "Resources": {
     "JobServerGroup": {
       "Type" : "AWS::AutoScaling::AutoScalingGroup",
@@ -54,7 +50,7 @@ class AutoscalingGroupTest(functional_base.FunctionalTestsBase):
         "ImageId"           : {"Ref": "image"},
         "InstanceType"      : {"Ref": "flavor"},
         "SecurityGroups"    : [ "sg-1" ],
-        "UserData"          : {"Ref": "user_data"}
+        "UserData"          : "jsconfig data"
       }
     }
   },
@@ -80,7 +76,7 @@ resources:
   random1:
     type: OS::Heat::RandomString
     properties:
-      salt: {get_param: UserData}
+      salt: {get_param: ImageId}
 outputs:
   PublicIp: {value: {get_attr: [random1, value]}}
   AvailabilityZone: {value: 'not-used11'}
@@ -108,7 +104,7 @@ resources:
   waiter:
     type: AWS::CloudFormation::WaitCondition
     properties:
-      Handle: {get_resource: ready_poster}
+      Handle: {Ref: ready_poster}
       Timeout: 1
 outputs:
   PublicIp:
@@ -117,6 +113,9 @@ outputs:
 
     def setUp(self):
         super(AutoscalingGroupTest, self).setUp()
+        self.client = self.orchestration_client
+        if not self.conf.image_ref:
+            raise self.skipException("No image configured to test")
         if not self.conf.minimal_image_ref:
             raise self.skipException("No minimal image configured to test")
         if not self.conf.instance_type:
@@ -151,7 +150,7 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         files = {'provider.yaml': self.instance_template}
         env = {'resource_registry': {'AWS::EC2::Instance': 'provider.yaml'},
                'parameters': {'size': 4,
-                              'image': self.conf.minimal_image_ref,
+                              'image': self.conf.image_ref,
                               'flavor': self.conf.instance_type}}
         stack_identifier = self.stack_create(template=self.template,
                                              files=files, environment=env)
@@ -168,7 +167,7 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         files = {'provider.yaml': self.instance_template}
         env = {'resource_registry': {'AWS::EC2::Instance': 'provider.yaml'},
                'parameters': {'size': 2,
-                              'image': self.conf.minimal_image_ref,
+                              'image': self.conf.image_ref,
                               'flavor': self.conf.instance_type}}
 
         stack_identifier = self.stack_create(template=self.template,
@@ -180,24 +179,23 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         # Increase min size to 5
         env2 = {'resource_registry': {'AWS::EC2::Instance': 'provider.yaml'},
                 'parameters': {'size': 5,
-                               'image': self.conf.minimal_image_ref,
+                               'image': self.conf.image_ref,
                                'flavor': self.conf.instance_type}}
         self.update_stack(stack_identifier, self.template,
                           environment=env2, files=files)
+        self._wait_for_stack_status(stack_identifier, 'UPDATE_COMPLETE')
         stack = self.client.stacks.get(stack_identifier)
         self.assert_instance_count(stack, 5)
 
     def test_update_group_replace(self):
-        """Test case for ensuring non-updatable props case a replacement.
-
-        Make sure that during a group update the non-updatable
+        """Make sure that during a group update the non updatable
         properties cause a replacement.
         """
         files = {'provider.yaml': self.instance_template}
         env = {'resource_registry':
                {'AWS::EC2::Instance': 'provider.yaml'},
                'parameters': {'size': '1',
-                              'image': self.conf.minimal_image_ref,
+                              'image': self.conf.image_ref,
                               'flavor': self.conf.instance_type}}
 
         stack_identifier = self.stack_create(template=self.template,
@@ -210,9 +208,8 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
                 {'AWS::EC2::Instance': 'provider.yaml'},
                 'parameters': {'size': '1',
                                'AZ': 'wibble',
-                               'image': self.conf.minimal_image_ref,
-                               'flavor': self.conf.instance_type,
-                               'user_data': 'new data'}}
+                               'image': self.conf.image_ref,
+                               'flavor': self.conf.instance_type}}
         self.update_stack(stack_identifier, self.template,
                           environment=env2, files=files)
 
@@ -221,16 +218,14 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         self.assertNotEqual(orig_asg_id, rsrc.physical_resource_id)
 
     def test_create_instance_error_causes_group_error(self):
-        """Test create failing a resource in the instance group.
-
-        If a resource in an instance group fails to be created, the instance
+        """If a resource in an instance group fails to be created, the instance
         group itself will fail and the broken inner resource will remain.
         """
         stack_name = self._stack_rand_name()
         files = {'provider.yaml': self.bad_instance_template}
         env = {'resource_registry': {'AWS::EC2::Instance': 'provider.yaml'},
                'parameters': {'size': 2,
-                              'image': self.conf.minimal_image_ref,
+                              'image': self.conf.image_ref,
                               'flavor': self.conf.instance_type}}
 
         self.client.stacks.create(
@@ -241,7 +236,7 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
             parameters={},
             environment=env
         )
-        self.addCleanup(self._stack_delete, stack_name)
+        self.addCleanup(self.client.stacks.delete, stack_name)
         stack = self.client.stacks.get(stack_name)
         stack_identifier = '%s/%s' % (stack_name, stack.id)
         self._wait_for_stack_status(stack_identifier, 'CREATE_FAILED')
@@ -256,16 +251,14 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         self._assert_instance_state(nested_ident, 0, 2)
 
     def test_update_instance_error_causes_group_error(self):
-        """Test update failing a resource in the instance group.
-
-        If a resource in an instance group fails to be created during an
+        """If a resource in an instance group fails to be created during an
         update, the instance group itself will fail and the broken inner
         resource will remain.
         """
         files = {'provider.yaml': self.instance_template}
         env = {'resource_registry': {'AWS::EC2::Instance': 'provider.yaml'},
                'parameters': {'size': 2,
-                              'image': self.conf.minimal_image_ref,
+                              'image': self.conf.image_ref,
                               'flavor': self.conf.instance_type}}
 
         stack_identifier = self.stack_create(template=self.template,
@@ -317,7 +310,7 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         files = {'provider.yaml': self.instance_template}
         env = {'resource_registry': {'AWS::EC2::Instance': 'provider.yaml'},
                'parameters': {'size': 4,
-                              'image': self.conf.minimal_image_ref,
+                              'image': self.conf.image_ref,
                               'flavor': self.conf.instance_type}}
         stack_identifier = self.stack_create(template=self.template,
                                              files=files, environment=env)
@@ -325,11 +318,21 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         nested_ident = self.assert_resource_is_a_stack(stack_identifier,
                                                        'JobServerGroup')
 
-        self.stack_suspend(stack_identifier)
-        self._wait_for_all_resource_status(nested_ident, 'SUSPEND_COMPLETE')
+        self.client.actions.suspend(stack_id=stack_identifier)
+        self._wait_for_resource_status(
+            stack_identifier, 'JobServerGroup', 'SUSPEND_COMPLETE')
+        for res in self.client.resources.list(nested_ident):
+            self._wait_for_resource_status(nested_ident,
+                                           res.resource_name,
+                                           'SUSPEND_COMPLETE')
 
-        self.stack_resume(stack_identifier)
-        self._wait_for_all_resource_status(nested_ident, 'RESUME_COMPLETE')
+        self.client.actions.resume(stack_id=stack_identifier)
+        self._wait_for_resource_status(
+            stack_identifier, 'JobServerGroup', 'RESUME_COMPLETE')
+        for res in self.client.resources.list(nested_ident):
+            self._wait_for_resource_status(nested_ident,
+                                           res.resource_name,
+                                           'RESUME_COMPLETE')
 
 
 class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
@@ -346,14 +349,15 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
     def update_instance_group(self, updt_template,
                               num_updates_expected_on_updt,
                               num_creates_expected_on_updt,
-                              num_deletes_expected_on_updt):
+                              num_deletes_expected_on_updt,
+                              update_replace):
 
         # setup stack from the initial template
         files = {'provider.yaml': self.instance_template}
         size = 10
         env = {'resource_registry': {'AWS::EC2::Instance': 'provider.yaml'},
                'parameters': {'size': size,
-                              'image': self.conf.minimal_image_ref,
+                              'image': self.conf.image_ref,
                               'flavor': self.conf.instance_type}}
         stack_name = self._stack_rand_name()
         stack_identifier = self.stack_create(
@@ -380,6 +384,7 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         # test stack update
         self.update_stack(stack_identifier, updt_template,
                           environment=env, files=files)
+        self._wait_for_stack_status(stack_identifier, 'UPDATE_COMPLETE')
         updt_stack = self.client.stacks.get(stack_identifier)
 
         # test that the launch configuration is replaced
@@ -413,10 +418,9 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
             self.assertNotIn(deletes_expected, updt_names)
 
     def test_instance_group_update_replace(self):
-        """Test simple update replace.
-
-        Test update replace with no conflict in batch size and minimum
-        instances in service.
+        """
+        Test simple update replace with no conflict in batch size and
+        minimum instances in service.
         """
         updt_template = self.ig_tmpl_with_updt_policy()
         grp = updt_template['Resources']['JobServerGroup']
@@ -424,18 +428,18 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         policy['MinInstancesInService'] = '1'
         policy['MaxBatchSize'] = '3'
         config = updt_template['Resources']['JobServerConfig']
-        config['Properties']['UserData'] = 'new data'
+        config['Properties']['ImageId'] = self.conf.minimal_image_ref
 
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=10,
                                    num_creates_expected_on_updt=0,
-                                   num_deletes_expected_on_updt=0)
+                                   num_deletes_expected_on_updt=0,
+                                   update_replace=True)
 
     def test_instance_group_update_replace_with_adjusted_capacity(self):
-        """Test update replace with capacity adjustment.
-
-        Test update replace with capacity adjustment due to conflict in batch
-        size and minimum instances in service.
+        """
+        Test update replace with capacity adjustment due to conflict in
+        batch size and minimum instances in service.
         """
         updt_template = self.ig_tmpl_with_updt_policy()
         grp = updt_template['Resources']['JobServerGroup']
@@ -443,30 +447,36 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         policy['MinInstancesInService'] = '8'
         policy['MaxBatchSize'] = '4'
         config = updt_template['Resources']['JobServerConfig']
-        config['Properties']['UserData'] = 'new data'
+        config['Properties']['ImageId'] = self.conf.minimal_image_ref
 
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=8,
                                    num_creates_expected_on_updt=2,
-                                   num_deletes_expected_on_updt=2)
+                                   num_deletes_expected_on_updt=2,
+                                   update_replace=True)
 
     def test_instance_group_update_replace_huge_batch_size(self):
-        """Test update replace with a huge batch size."""
+        """
+        Test update replace with a huge batch size.
+        """
         updt_template = self.ig_tmpl_with_updt_policy()
         group = updt_template['Resources']['JobServerGroup']
         policy = group['UpdatePolicy']['AutoScalingRollingUpdate']
         policy['MinInstancesInService'] = '0'
         policy['MaxBatchSize'] = '20'
         config = updt_template['Resources']['JobServerConfig']
-        config['Properties']['UserData'] = 'new data'
+        config['Properties']['ImageId'] = self.conf.minimal_image_ref
 
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=10,
                                    num_creates_expected_on_updt=0,
-                                   num_deletes_expected_on_updt=0)
+                                   num_deletes_expected_on_updt=0,
+                                   update_replace=True)
 
     def test_instance_group_update_replace_huge_min_in_service(self):
-        """Update replace with huge number of minimum instances in service."""
+        """
+        Test update replace with a huge number of minimum instances in service.
+        """
         updt_template = self.ig_tmpl_with_updt_policy()
         group = updt_template['Resources']['JobServerGroup']
         policy = group['UpdatePolicy']['AutoScalingRollingUpdate']
@@ -474,16 +484,16 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         policy['MaxBatchSize'] = '1'
         policy['PauseTime'] = 'PT0S'
         config = updt_template['Resources']['JobServerConfig']
-        config['Properties']['UserData'] = 'new data'
+        config['Properties']['ImageId'] = self.conf.minimal_image_ref
 
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=9,
                                    num_creates_expected_on_updt=1,
-                                   num_deletes_expected_on_updt=1)
+                                   num_deletes_expected_on_updt=1,
+                                   update_replace=True)
 
     def test_instance_group_update_no_replace(self):
-        """Test simple update only and no replace.
-
+        """
         Test simple update only and no replace (i.e. updated instance flavor
         in Launch Configuration) with no conflict in batch size and
         minimum instances in service.
@@ -495,16 +505,16 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         policy['MaxBatchSize'] = '3'
         policy['PauseTime'] = 'PT0S'
         config = updt_template['Resources']['JobServerConfig']
-        config['Properties']['InstanceType'] = self.conf.minimal_instance_type
+        config['Properties']['InstanceType'] = 'm1.tiny'
 
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=10,
                                    num_creates_expected_on_updt=0,
-                                   num_deletes_expected_on_updt=0)
+                                   num_deletes_expected_on_updt=0,
+                                   update_replace=False)
 
     def test_instance_group_update_no_replace_with_adjusted_capacity(self):
-        """Test update only and no replace with capacity adjustment.
-
+        """
         Test update only and no replace (i.e. updated instance flavor in
         Launch Configuration) with capacity adjustment due to conflict in
         batch size and minimum instances in service.
@@ -516,12 +526,13 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         policy['MaxBatchSize'] = '4'
         policy['PauseTime'] = 'PT0S'
         config = updt_template['Resources']['JobServerConfig']
-        config['Properties']['InstanceType'] = self.conf.minimal_instance_type
+        config['Properties']['InstanceType'] = 'm1.tiny'
 
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=8,
                                    num_creates_expected_on_updt=2,
-                                   num_deletes_expected_on_updt=2)
+                                   num_deletes_expected_on_updt=2,
+                                   update_replace=False)
 
 
 class AutoScalingSignalTest(AutoscalingGroupTest):
@@ -624,23 +635,23 @@ outputs:
                      {'custom_lb': {'AWS::EC2::Instance': 'lb.yaml'}},
                      'AWS::EC2::Instance': 'provider.yaml'},
                     'parameters': {'size': 2,
-                                   'image': self.conf.minimal_image_ref,
+                                   'image': self.conf.image_ref,
                                    'flavor': self.conf.instance_type}}
 
     def check_instance_count(self, stack_identifier, expected):
         md = self.client.resources.metadata(stack_identifier, 'custom_lb')
         actual_md = len(md['IPs'].split(','))
         if actual_md != expected:
-            LOG.warning('check_instance_count exp:%d, meta:%s' % (expected,
-                                                                  md['IPs']))
+            LOG.warn('check_instance_count exp:%d, meta:%s' % (expected,
+                                                               md['IPs']))
             return False
 
         stack = self.client.stacks.get(stack_identifier)
         inst_list = self._stack_output(stack, 'InstanceList')
         actual = len(inst_list.split(','))
         if actual != expected:
-            LOG.warning('check_instance_count exp:%d, act:%s' % (expected,
-                                                                 inst_list))
+            LOG.warn('check_instance_count exp:%d, act:%s' % (expected,
+                                                              inst_list))
         return actual == expected
 
     def test_scaling_meta_update(self):
@@ -726,17 +737,11 @@ outputs:
 
         # suspend the top level stack.
         self.client.actions.suspend(stack_id=stack_identifier)
+        self._wait_for_resource_status(
+            stack_identifier, 'JobServerGroup', 'SUSPEND_COMPLETE')
 
-        # Wait for stack to reach SUSPEND_COMPLETE
-        self._wait_for_stack_status(stack_identifier, 'SUSPEND_COMPLETE')
-
-        # Send a signal and an exception will raise
-        ex = self.assertRaises(exc.BadRequest,
-                               self.client.resources.signal,
-                               stack_identifier, 'ScaleUpPolicy')
-
-        error_msg = 'Signal resource during SUSPEND is not supported'
-        self.assertIn(error_msg, six.text_type(ex))
+        # Send a signal and confirm nothing happened.
+        self.client.resources.signal(stack_identifier, 'ScaleUpPolicy')
         ev = self.wait_for_event_with_reason(
             stack_identifier,
             reason='Cannot signal resource during SUSPEND',
