@@ -17,7 +17,7 @@ from oslo_messaging import transport
 import requests
 
 from heat_integrationtests.common import test
-from heat_integrationtests.functional import functional_base
+
 
 BASIC_NOTIFICATIONS = [
     'orchestration.stack.create.start',
@@ -70,7 +70,7 @@ class NotificationHandler(object):
         return self._notifications
 
 
-class NotificationTest(functional_base.FunctionalTestsBase):
+class NotificationTest(test.HeatIntegrationTest):
 
     basic_template = '''
 heat_template_version: 2013-05-23
@@ -124,6 +124,8 @@ outputs:
 
     def setUp(self):
         super(NotificationTest, self).setUp()
+
+        self.client = self.orchestration_client
         self.exchange = kombu.Exchange('heat', 'topic', durable=False)
         queue = kombu.Queue(exchange=self.exchange,
                             routing_key='notifications.info',
@@ -139,9 +141,17 @@ outputs:
         return len(handler.notifications) == count
 
     def test_basic_notifications(self):
-        # disable cleanup so we can call _stack_delete() directly.
-        stack_identifier = self.stack_create(template=self.basic_template,
-                                             enable_cleanup=False)
+        stack_identifier = self._stack_rand_name()
+        # do this manually so we can call _stack_delete() directly.
+        self.client.stacks.create(
+            stack_name=stack_identifier,
+            template=self.basic_template,
+            files={},
+            disable_rollback=True,
+            parameters={},
+            environment={}
+        )
+        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')
         self.update_stack(stack_identifier,
                           template=self.update_basic_template)
         self.stack_suspend(stack_identifier)
@@ -179,12 +189,12 @@ outputs:
                                 callbacks=[handler.process_message],
                                 auto_declare=False):
 
-            requests.post(scale_up_url, verify=self.verify_cert)
+            requests.post(scale_up_url)
             test.call_until_true(20, 0, self.consume_events, handler, 2)
             notifications += handler.notifications
 
             handler.clear()
-            requests.post(scale_down_url, verify=self.verify_cert)
+            requests.post(scale_down_url)
             test.call_until_true(20, 0, self.consume_events, handler, 2)
             notifications += handler.notifications
 
