@@ -12,15 +12,12 @@
 
 import json
 
-from heatclient import exc as heat_exceptions
-import six
 import yaml
 
 from heat_integrationtests.common import test
-from heat_integrationtests.functional import functional_base
 
 
-class TemplateResourceTest(functional_base.FunctionalTestsBase):
+class TemplateResourceTest(test.HeatIntegrationTest):
     """Prove that we can use the registry in a nested provider."""
 
     template = '''
@@ -47,6 +44,10 @@ resource_registry:
   "OS::Heat::RandomString": nested.yaml
 '''
 
+    def setUp(self):
+        super(TemplateResourceTest, self).setUp()
+        self.client = self.orchestration_client
+
     def test_nested_env(self):
         main_templ = '''
 heat_template_version: 2013-05-23
@@ -101,12 +102,17 @@ resource_registry:
         This tests that if you manually delete a nested
         stack, the parent stack is still deletable.
         """
-        # disable cleanup so we can call _stack_delete() directly.
-        stack_identifier = self.stack_create(
+        name = self._stack_rand_name()
+        # do this manually so we can call _stack_delete() directly.
+        self.client.stacks.create(
+            stack_name=name,
             template=self.template,
             files={'nested.yaml': self.nested_templ},
             environment=self.env_templ,
-            enable_cleanup=False)
+            disable_rollback=True)
+        stack = self.client.stacks.get(name)
+        stack_identifier = '%s/%s' % (name, stack.id)
+        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')
 
         nested_ident = self.assert_resource_is_a_stack(stack_identifier,
                                                        'secret1')
@@ -145,7 +151,7 @@ resource_registry:
         self.assertEqual('freddy', secret_out2)
 
 
-class NestedAttributesTest(functional_base.FunctionalTestsBase):
+class NestedAttributesTest(test.HeatIntegrationTest):
     """Prove that we can use the template resource references."""
 
     main_templ = '''
@@ -169,15 +175,16 @@ resource_registry:
   "My::NestedSecret": nested.yaml
 '''
 
+    def setUp(self):
+        super(NestedAttributesTest, self).setUp()
+        self.client = self.orchestration_client
+
     def test_stack_ref(self):
         nested_templ = '''
 heat_template_version: 2014-10-16
 resources:
   secret1:
     type: OS::Heat::RandomString
-outputs:
-  nested_str:
-    value: {get_attr: [secret1, value]}
 '''
         stack_identifier = self.stack_create(
             template=self.main_templ,
@@ -189,12 +196,9 @@ outputs:
         self.assertIn('arn:openstack:heat:', test_ref)
 
     def test_transparent_ref(self):
-        """Test using nested resource more transparently.
-
-        With the addition of OS::stack_id we can now use the nested resource
+        """With the addition of OS::stack_id we can now use the nested resource
         more transparently.
         """
-
         nested_templ = '''
 heat_template_version: 2014-10-16
 resources:
@@ -242,43 +246,7 @@ outputs:
         self.assertEqual(old_way, test_attr2)
 
 
-class TemplateResourceFacadeTest(functional_base.FunctionalTestsBase):
-    """Prove that we can use ResourceFacade in a HOT template."""
-
-    main_template = '''
-heat_template_version: 2013-05-23
-resources:
-  the_nested:
-    type: the.yaml
-    metadata:
-      foo: bar
-outputs:
-  value:
-    value: {get_attr: [the_nested, output]}
-'''
-
-    nested_templ = '''
-heat_template_version: 2013-05-23
-resources:
-  test:
-    type: OS::Heat::TestResource
-    properties:
-      value: {"Fn::Select": [foo, {resource_facade: metadata}]}
-outputs:
-  output:
-    value: {get_attr: [test, output]}
-    '''
-
-    def test_metadata(self):
-        stack_identifier = self.stack_create(
-            template=self.main_template,
-            files={'the.yaml': self.nested_templ})
-        stack = self.client.stacks.get(stack_identifier)
-        value = self._stack_output(stack, 'value')
-        self.assertEqual('bar', value)
-
-
-class TemplateResourceUpdateTest(functional_base.FunctionalTestsBase):
+class TemplateResourceUpdateTest(test.HeatIntegrationTest):
     """Prove that we can do template resource updates."""
 
     main_template = '''
@@ -490,6 +458,10 @@ Outputs:
                                       expect=NOCHANGE)),
     ]
 
+    def setUp(self):
+        super(TemplateResourceUpdateTest, self).setUp()
+        self.client = self.orchestration_client
+
     def test_template_resource_update_template_schema(self):
         stack_identifier = self.stack_create(
             template=self.main_template,
@@ -512,9 +484,8 @@ Outputs:
                                 self._stack_output(stack, 'value'))
 
 
-class TemplateResourceUpdateFailedTest(functional_base.FunctionalTestsBase):
+class TemplateResourceUpdateFailedTest(test.HeatIntegrationTest):
     """Prove that we can do updates on a nested stack to fix a stack."""
-
     main_template = '''
 HeatTemplateFormatVersion: '2012-12-12'
 Resources:
@@ -536,6 +507,7 @@ Resources:
 
     def setUp(self):
         super(TemplateResourceUpdateFailedTest, self).setUp()
+        self.client = self.orchestration_client
         self.assign_keypair()
 
     def test_update_on_failed_create(self):
@@ -556,7 +528,7 @@ Resources:
                           files={'server_fail.yaml': self.nested_templ})
 
 
-class TemplateResourceAdoptTest(functional_base.FunctionalTestsBase):
+class TemplateResourceAdoptTest(test.HeatIntegrationTest):
     """Prove that we can do template resource adopt/abandon."""
 
     main_template = '''
@@ -589,24 +561,30 @@ Outputs:
     Value: {'Fn::GetAtt': [RealRandom, value]}
 '''
 
+    def setUp(self):
+        super(TemplateResourceAdoptTest, self).setUp()
+        self.client = self.orchestration_client
+
     def _yaml_to_json(self, yaml_templ):
-        return yaml.safe_load(yaml_templ)
+        return yaml.load(yaml_templ)
 
     def test_abandon(self):
-        stack_identifier = self.stack_create(
+        stack_name = self._stack_rand_name()
+        self.client.stacks.create(
+            stack_name=stack_name,
             template=self.main_template,
             files={'the.yaml': self.nested_templ},
-            enable_cleanup=False
+            disable_rollback=True,
         )
+        stack = self.client.stacks.get(stack_name)
+        stack_identifier = '%s/%s' % (stack_name, stack.id)
+        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')
 
         info = self.stack_abandon(stack_id=stack_identifier)
         self.assertEqual(self._yaml_to_json(self.main_template),
                          info['template'])
         self.assertEqual(self._yaml_to_json(self.nested_templ),
                          info['resources']['the_nested']['template'])
-        # TODO(james combs): Implement separate test cases for export
-        # once export REST API is available.  Also test reverse order
-        # of invocation: export -> abandon AND abandon -> export
 
     def test_adopt(self):
         data = {
@@ -623,7 +601,7 @@ Outputs:
                 }
             },
             "environment": {"parameters": {}},
-            "template": yaml.safe_load(self.main_template)
+            "template": yaml.load(self.main_template)
         }
 
         stack_identifier = self.stack_adopt(
@@ -635,7 +613,7 @@ Outputs:
         self.assertEqual('goopie', self._stack_output(stack, 'value'))
 
 
-class TemplateResourceCheckTest(functional_base.FunctionalTestsBase):
+class TemplateResourceCheckTest(test.HeatIntegrationTest):
     """Prove that we can do template resource check."""
 
     main_template = '''
@@ -668,19 +646,28 @@ Outputs:
     Value: {'Fn::GetAtt': [RealRandom, value]}
 '''
 
+    def setUp(self):
+        super(TemplateResourceCheckTest, self).setUp()
+        self.client = self.orchestration_client
+
     def test_check(self):
-        stack_identifier = self.stack_create(
+        stack_name = self._stack_rand_name()
+        self.client.stacks.create(
+            stack_name=stack_name,
             template=self.main_template,
-            files={'the.yaml': self.nested_templ}
+            files={'the.yaml': self.nested_templ},
+            disable_rollback=True,
         )
+        stack = self.client.stacks.get(stack_name)
+        stack_identifier = '%s/%s' % (stack_name, stack.id)
+        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')
 
         self.client.actions.check(stack_id=stack_identifier)
         self._wait_for_stack_status(stack_identifier, 'CHECK_COMPLETE')
 
 
-class TemplateResourceErrorMessageTest(functional_base.FunctionalTestsBase):
+class TemplateResourceErrorMessageTest(test.HeatIntegrationTest):
     """Prove that nested stack errors don't suck."""
-
     template = '''
 HeatTemplateFormatVersion: '2012-12-12'
 Resources:
@@ -697,6 +684,10 @@ Resources:
       wait_secs: 2
 '''
 
+    def setUp(self):
+        super(TemplateResourceErrorMessageTest, self).setUp()
+        self.client = self.orchestration_client
+
     def test_fail(self):
         stack_identifier = self.stack_create(
             template=self.template,
@@ -709,274 +700,3 @@ Resources:
         exp = 'Resource CREATE failed: ValueError: %s: %s' % (exp_path,
                                                               exp_msg)
         self.assertEqual(exp, stack.stack_status_reason)
-
-
-class TemplateResourceSuspendResumeTest(functional_base.FunctionalTestsBase):
-    """Prove that we can do template resource suspend/resume."""
-
-    main_template = '''
-heat_template_version: 2014-10-16
-parameters:
-resources:
-  the_nested:
-    type: the.yaml
-'''
-
-    nested_templ = '''
-heat_template_version: 2014-10-16
-resources:
-  test_random_string:
-    type: OS::Heat::RandomString
-'''
-
-    def test_suspend_resume(self):
-        """Basic test for template resource suspend resume."""
-        stack_identifier = self.stack_create(
-            template=self.main_template,
-            files={'the.yaml': self.nested_templ}
-        )
-
-        self.stack_suspend(stack_identifier=stack_identifier)
-        self.stack_resume(stack_identifier=stack_identifier)
-
-
-class ValidateFacadeTest(functional_base.FunctionalTestsBase):
-    """Prove that nested stack errors don't suck."""
-
-    template = '''
-heat_template_version: 2015-10-15
-resources:
-  thisone:
-    type: OS::Thingy
-    properties:
-      one: pre
-      two: post
-outputs:
-  one:
-    value: {get_attr: [thisone, here-it-is]}
-'''
-    templ_facade = '''
-heat_template_version: 2015-04-30
-parameters:
-  one:
-    type: string
-  two:
-    type: string
-outputs:
-  here-it-is:
-    value: noop
-'''
-    env = '''
-resource_registry:
-  OS::Thingy: facade.yaml
-  resources:
-    thisone:
-      OS::Thingy: concrete.yaml
-'''
-
-    def setUp(self):
-        super(ValidateFacadeTest, self).setUp()
-        self.client = self.orchestration_client
-
-    def test_missing_param(self):
-        templ_missing_parameter = '''
-heat_template_version: 2015-04-30
-parameters:
-  one:
-    type: string
-resources:
-  str:
-    type: OS::Heat::RandomString
-outputs:
-  here-it-is:
-    value:
-      not-important
-'''
-        try:
-            self.stack_create(
-                template=self.template,
-                environment=self.env,
-                files={'facade.yaml': self.templ_facade,
-                       'concrete.yaml': templ_missing_parameter},
-                expected_status='CREATE_FAILED')
-        except heat_exceptions.HTTPBadRequest as exc:
-            exp = ('ERROR: Required property two for facade '
-                   'OS::Thingy missing in provider')
-            self.assertEqual(exp, six.text_type(exc))
-
-    def test_missing_output(self):
-        templ_missing_output = '''
-heat_template_version: 2015-04-30
-parameters:
-  one:
-    type: string
-  two:
-    type: string
-resources:
-  str:
-    type: OS::Heat::RandomString
-'''
-        try:
-            self.stack_create(
-                template=self.template,
-                environment=self.env,
-                files={'facade.yaml': self.templ_facade,
-                       'concrete.yaml': templ_missing_output},
-                expected_status='CREATE_FAILED')
-        except heat_exceptions.HTTPBadRequest as exc:
-            exp = ('ERROR: Attribute here-it-is for facade '
-                   'OS::Thingy missing in provider')
-            self.assertEqual(exp, six.text_type(exc))
-
-
-class TemplateResourceNewParamTest(functional_base.FunctionalTestsBase):
-
-    main_template = '''
-heat_template_version: 2013-05-23
-resources:
-  my_resource:
-    type: resource.yaml
-    properties:
-      value1: foo
-'''
-    nested_templ = '''
-heat_template_version: 2013-05-23
-parameters:
-  value1:
-    type: string
-resources:
-  test:
-    type: OS::Heat::TestResource
-    properties:
-      value: {get_param: value1}
-'''
-    main_template_update = '''
-heat_template_version: 2013-05-23
-resources:
-  my_resource:
-    type: resource.yaml
-    properties:
-      value1: foo
-      value2: foo
-'''
-    nested_templ_update_fail = '''
-heat_template_version: 2013-05-23
-parameters:
-  value1:
-    type: string
-  value2:
-    type: string
-resources:
-  test:
-    type: OS::Heat::TestResource
-    properties:
-      fail: True
-      value:
-        str_replace:
-          template: VAL1-VAL2
-          params:
-            VAL1: {get_param: value1}
-            VAL2: {get_param: value2}
-'''
-    nested_templ_update = '''
-heat_template_version: 2013-05-23
-parameters:
-  value1:
-    type: string
-  value2:
-    type: string
-resources:
-  test:
-    type: OS::Heat::TestResource
-    properties:
-      value:
-        str_replace:
-          template: VAL1-VAL2
-          params:
-            VAL1: {get_param: value1}
-            VAL2: {get_param: value2}
-'''
-
-    def test_update(self):
-        stack_identifier = self.stack_create(
-            template=self.main_template,
-            files={'resource.yaml': self.nested_templ})
-
-        # Make the update fails with the new parameter inserted.
-        self.update_stack(
-            stack_identifier,
-            self.main_template_update,
-            files={'resource.yaml': self.nested_templ_update_fail},
-            expected_status='UPDATE_FAILED')
-
-        # Fix the update, it should succeed now.
-        self.update_stack(
-            stack_identifier,
-            self.main_template_update,
-            files={'resource.yaml': self.nested_templ_update})
-
-
-class TemplateResourceRemovedParamTest(functional_base.FunctionalTestsBase):
-
-    main_template = '''
-heat_template_version: 2013-05-23
-parameters:
-  value1:
-    type: string
-    default: foo
-resources:
-  my_resource:
-    type: resource.yaml
-    properties:
-       value1: {get_param: value1}
-'''
-    nested_templ = '''
-heat_template_version: 2013-05-23
-parameters:
-  value1:
-    type: string
-    default: foo
-resources:
-  test:
-    type: OS::Heat::TestResource
-    properties:
-      value: {get_param: value1}
-'''
-    main_template_update = '''
-heat_template_version: 2013-05-23
-resources:
-  my_resource:
-    type: resource.yaml
-'''
-    nested_templ_update = '''
-heat_template_version: 2013-05-23
-parameters:
-  value1:
-    type: string
-    default: foo
-  value2:
-    type: string
-    default: bar
-resources:
-  test:
-    type: OS::Heat::TestResource
-    properties:
-      value:
-        str_replace:
-          template: VAL1-VAL2
-          params:
-            VAL1: {get_param: value1}
-            VAL2: {get_param: value2}
-'''
-
-    def test_update(self):
-        stack_identifier = self.stack_create(
-            template=self.main_template,
-            environment={'parameters': {'value1': 'spam'}},
-            files={'resource.yaml': self.nested_templ})
-
-        self.update_stack(
-            stack_identifier,
-            self.main_template_update,
-            environment={'parameter_defaults': {'value2': 'egg'}},
-            files={'resource.yaml': self.nested_templ_update}, existing=True)
